//*****************************************************************************
// File: nvcfx.java
// Description: Nirva Cold Fusion interface
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

import com.allaire.cfx.* ;


public class nvcfx implements CustomTag
{
	// Native methods declaration
	private native long Int_OpenRequest(java.lang.String ConnectString);
	private native void Int_CloseRequest(long NvHandle);
	private native int Int_Command(long NvHandle, java.lang.String CommandString);
	private native java.lang.String Int_GetResult(long NvHandle);

	public void processRequest( Request request, Response response ) 		
	   throws Exception
	{
		response.setVariable("NVCFX_RESULT", "");
		response.setVariable("NVCFX_RETCODE", "0");
		if(request.attributeExists("NVCFX_ORDER"))
		{
			String strMeta = request.getAttribute( "NVCFX_ORDER" ) ;
			if(strMeta.compareToIgnoreCase("OPENREQUEST") == 0)
			{
				OpenRequest( request, response );
			}
			else if(strMeta.compareToIgnoreCase("CLOSEREQUEST") == 0)
			{
				CloseRequest( request, response );
			}
			else
			{
				Command( request, response );
			}
		}
		else
		{
			Command( request, response );
		}
	}

	// Load the library
	static
	{
		System.loadLibrary("nvc");
	}

	// constructor with default connection string
	
	public void OpenRequest( Request request, Response response ) 		
	{
		String[] attribs = request.getAttributeList() ;
		int nNumAttribs = attribs.length ;          

		String ConnectionString = "";
		for( int i = 0; i < nNumAttribs; i++ )
		{
			String strName = attribs[i] ;
			if(strName.compareToIgnoreCase("NVCFX_ORDER") == 0)
				continue;
			String strValue = request.getAttribute( strName ) ;
			ConnectionString = ConnectionString + " " + strName + "=\"" + strValue + "\"";
		}
		long iHandle = Int_OpenRequest(ConnectionString);
		String sHandle = new Long(iHandle).toString();
		response.setVariable("NVCFX_RESULT", sHandle);
		if(iHandle != 0)
			response.setVariable("NVCFX_RETCODE", "1");
	}
	
	public void CloseRequest( Request request, Response response ) 		
	{
		if(!request.attributeExists("NVCFX_HREQUEST"))
			return;
		String sHandle = request.getAttribute("NVCFX_HREQUEST") ;
		long iHandle = new Long(sHandle).longValue();
		Int_CloseRequest(iHandle);
		response.setVariable("NVCFX_RETCODE", "1");
	}

	
	public void Command( Request request, Response response ) 		
	{
		if(!request.attributeExists("NVCFX_HREQUEST"))
			return;
		String sHandle = request.getAttribute("NVCFX_HREQUEST") ;
		long iHandle = new Long(sHandle).longValue();

		String[] attribs = request.getAttributeList() ;
		int nNumAttribs = attribs.length ;          

		String CommandString = "";
		for( int i = 0; i < nNumAttribs; i++ )
		{
			String strName = attribs[i] ;
			if(strName.compareToIgnoreCase("NVCFX_ORDER") == 0)
				continue;
			if(strName.compareToIgnoreCase("NVCFX_HREQUEST") == 0)
				continue;
			String strValue = request.getAttribute( strName ) ;
			CommandString = CommandString + " " + strName + "=\"" + strValue + "\"";
		}

		int RetVal = 0;
		RetVal = Int_Command(iHandle, CommandString);
		if(RetVal == 1)
		{
			response.setVariable("NVCFX_RETCODE", "1");
			String NvResult = Int_GetResult(iHandle);
			response.setVariable("NVCFX_RESULT", NvResult);
		}
	}
}
