//*****************************************************************************
// File: java.h
// Description: Nirva Java client interface (c++ part)
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifndef _JAVA_H
#define _JAVA_H

#include "../nvs/task.h"
#include "../nvc/error.h"
#include "command.h"
#include <string>
#include <list>



/////////////////////////////////////////////////////////////////////////////
// NvcJavaInt class
//
// This class encapsulates the request for a java call
//
class NvcJavaInt
{
public:
	NVREQUEST m_pRequest;						// pointer to the request
	char *ResultBuffer;							// command result buffer
	int ResultBufferSize;						// command result buffer size
	std::string LastResult;						// last string result
	int m_timeout;								// Request time out
	int m_iInUse;								// In use counter
	
public:
	NvcJavaInt();
	virtual ~NvcJavaInt();

	int SetBufferSize(int DataSize);				// eventually allocate or reallocate the result buffer
	bool OpenRequest(const char *ConnectString);	// Request open (the close is automatically made in the destructor)
	int Command(const char *CommandString);			// Command to nirva
};



/////////////////////////////////////////////////////////////////////////////
// NvRequestList class
class NvJavaRequestList : public NvObject
{

public:

	std::map<void *,time_t> m_vRequests;			// request list
	bool m_bUseChecking;							// use checking flag (slower but more secure)
	
	NvJavaRequestList();
	virtual ~NvJavaRequestList();

	// Removes all requests from the list
	void Clear();
	
	// Creates a new request and add it to the list
	NvcJavaInt* AddRequest(int TimeOut = 0);
	
	// Remove the given request from the list
	void RemoveRequest(NvcJavaInt *Request);
	
	// Check if a given request is in the list and set its inuse flag
	bool CheckRequest(NvcJavaInt *Request);
	
	// Check if a given request is in the list
	void FreeRequest(NvcJavaInt *Request);
	
	// Removes all time outed requests
	void Update();
};


#endif //_JAVA_H
