//*****************************************************************************
// File: util.cpp
// Description: useful functions
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifdef WIN32
#pragma warning( disable : 4996 )
#endif

#include "task.h"
#include "util.h"

#ifndef WIN32
extern char **environ;
int my_system (const char *command, unsigned int TimeOut = 0);
#endif

NvMutex	ProtectTempFile;
int TempFileIndex = 0;

using namespace std;







//**********************************************************************************
//**********************************************************************************
// These very usefull function unfortunatly doesn't exist on unix, so we do them
//**********************************************************************************
//**********************************************************************************
#ifndef WIN32
char *strlwr(char *string)
{
	for(int i=0; i<strlen(string); i++)
		string[i] = (char)(tolower(string[i]));
	return string;
}
char *strupr(char *string)
{
	for(int i=0; i<strlen(string); i++)
		string[i] = (char)(toupper(string[i]));
	return string;
}
#endif


int CopyFileEx(const char *SourceFile, const char *DestFile, int Offset, int BytesToCopy)
{
	FILE *FileSource;
	FILE *FileDest;
	unsigned int ReadBytes;
	unsigned short error;
	char *buffer;
	
	
	if(SourceFile == NULL)
		return 0;
	if(!strlen(SourceFile))
		return 0;
	
	if(DestFile == NULL)
		return 0;
	if(!strlen(DestFile))
		return 0;
	
	buffer = new char[16384];

	if(buffer == NULL)
		return 0;

	FileSource = Nvfopen(SourceFile, "rb");
	if(FileSource == NULL)
	{
		delete buffer;
		return 0;
	}

	FileDest = Nvfopen(DestFile, "wb");
	if(FileDest == NULL)
	{
		fclose(FileSource);
		delete buffer;
		return 0;
	}

	error = 0;
	
	
	// we get the result file size
	fseek(FileSource, 0, SEEK_END);
	int Length = ftell(FileSource);

	if(fseek(FileSource, Offset, SEEK_SET) != 0)
	{
		fclose(FileSource);
		fclose(FileDest);
		delete buffer;
		return 0;
	}


	if(BytesToCopy == -1)
		BytesToCopy = Length - Offset;
	else
		if(BytesToCopy > (Length - Offset))
			BytesToCopy = Length - Offset;

	unsigned int NbLoops = BytesToCopy / 16384;
	unsigned int LastBytes = BytesToCopy % 16384;

	for (unsigned int i=0; i<NbLoops; i++)
	{
		ReadBytes = (unsigned int)(fread(buffer, 1, 16384, FileSource));
		if(ReadBytes != 16384)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete buffer;
			return 0;
		}

	}
	if(LastBytes)
	{
		ReadBytes = (unsigned int)(fread(buffer, 1, LastBytes, FileSource));
		if(ReadBytes != LastBytes)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete buffer;
			return 0;
		}
	}


	fclose(FileSource);
	fclose(FileDest);
	delete buffer;
	
	return 1;
}


int CopyFileEx(FILE *FileSource, FILE *FileDest, int Offset, int BytesToCopy)
{
	unsigned int ReadBytes;
	unsigned short error;
	char *buffer;
	
	buffer = new char[16384];

	if(buffer == NULL)
		return 0;

	error = 0;
	
	// we get the result file size
	fseek(FileSource, 0, SEEK_END);
	int Length = ftell(FileSource);

	if(fseek(FileSource, Offset, SEEK_SET) != 0)
	{
		delete buffer;
		return 0;
	}


	if(BytesToCopy == -1)
		BytesToCopy = Length - Offset;
	else
		if(BytesToCopy > (Length - Offset))
			BytesToCopy = Length - Offset;

	unsigned int NbLoops = BytesToCopy / 16384;
	unsigned int LastBytes = BytesToCopy % 16384;

	for (unsigned int i=0; i<NbLoops; i++)
	{
		ReadBytes = (unsigned int)(fread(buffer, 1, 16384, FileSource));
		if(ReadBytes != 16384)
		{
			delete buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			delete buffer;
			return 0;
		}

	}
	if(LastBytes)
	{
		ReadBytes = (unsigned int)(fread(buffer, 1, LastBytes, FileSource));
		if(ReadBytes != LastBytes)
		{
			delete buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			delete buffer;
			return 0;
		}
	}


	delete buffer;
	
	return 1;
}


int AppendFileEx(const char *SourceFile, const char *DestFile, int Offset, int BytesToCopy)
{
	FILE *FileSource;
	FILE *FileDest;
	unsigned int ReadBytes;
	unsigned short error;
	char *buffer;
	
	
	if(SourceFile == NULL)
		return 0;
	if(!strlen(SourceFile))
		return 0;
	
	if(DestFile == NULL)
		return 0;
	if(!strlen(DestFile))
		return 0;
	
	buffer = new char[16384];

	if(buffer == NULL)
		return 0;

	FileSource = Nvfopen(SourceFile, "rb");
	if(FileSource == NULL)
	{
		delete buffer;
		return 0;
	}

	FileDest = Nvfopen(DestFile, "a+b");
	if(FileDest == NULL)
	{
		fclose(FileSource);
		delete buffer;
		return 0;
	}

	error = 0;
	
	
	// we get the result file size
	fseek(FileSource, 0, SEEK_END);
	int Length = ftell(FileSource);

	if(fseek(FileSource, Offset, SEEK_SET) != 0)
	{
		fclose(FileSource);
		fclose(FileDest);
		delete buffer;
		return 0;
	}


	if(BytesToCopy == -1)
		BytesToCopy = Length - Offset;
	else
		if(BytesToCopy > (Length - Offset))
			BytesToCopy = Length - Offset;

	unsigned int NbLoops = BytesToCopy / 16384;
	unsigned int LastBytes = BytesToCopy % 16384;

	for (unsigned int i=0; i<NbLoops; i++)
	{
		ReadBytes = (unsigned int)(fread(buffer, 1, 16384, FileSource));
		if(ReadBytes != 16384)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete buffer;
			return 0;
		}

	}
	if(LastBytes)
	{
		ReadBytes = (unsigned int)(fread(buffer, 1, LastBytes, FileSource));
		if(ReadBytes != LastBytes)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete buffer;
			return 0;
		}
		if(fwrite(buffer, 1, ReadBytes, FileDest)  != ReadBytes)
		{
			fclose(FileSource);
			fclose(FileDest);
			delete buffer;
			return 0;
		}
	}


	fclose(FileSource);
	fclose(FileDest);
	delete buffer;
	
	return 1;
}

//**********************************************************************************
//**********************************************************************************
// This function creates a unique temporary file
// It is protected by a mutex in order to be shure that the file is unique
//**********************************************************************************
//**********************************************************************************
bool GetTemporaryFile(string &FileName, string Dir, string Extension, string Prefix, string Suffix)
{
	CleanString(Dir);
	CleanString(Extension);
	CleanString(Prefix);
	CleanString(Suffix);

	bool ret = true;

	ProtectTempFile.Lock();

	AddDirSuffix(Dir);
	
	FileName = "";
	char *FileNameHere;
	int FileNameMaxSize = (int)(Dir.length() + Extension.length() + Prefix.length() + Suffix.length()) + 512;
	FileNameHere = (char *)malloc(FileNameMaxSize);
	if(FileNameHere == NULL)
		return false;
	
	// We generate a file name until the file doesn't exist and can be opened in write mode
	unsigned int nbloop = 0;
	while(1)
	{
		sprintf(FileNameHere, "%s%s%x%x%x%s", Dir.c_str(), Prefix.c_str(), TempFileIndex, getpid(), rand(), Suffix.c_str());
		if(Extension.length())
		{
			if(Extension[0] == '.')
				Extension.erase(0, 1);
			if(Extension.length())
			{
				strcat(FileNameHere, ".");
				strcat(FileNameHere, Extension.c_str());
			}
		}
		FILE * File = Nvfopen(FileNameHere, "r");
		if(File == NULL)	// the file doesn't exist
		{
			File = Nvfopen(FileNameHere, "w");	// so we try to open it for writting in order to make a zero length file
			if(File == NULL)				// if error we continue with another file name but no more than 10 times
			{								// because we may have a problem of write permission on a directory
				if(nbloop < 10)
				{
					nbloop++;
					TempFileIndex++;
					continue;
				}
				else
				{
					ret = false;
					break;
				}
			}
			
			fclose(File);					// otherwise we close the file and stop the loop (File is really correct and unique)
			break;
		}
		fclose(File);
		TempFileIndex++;
	}

	if(ret)
		FileName = FileNameHere;

	free(FileNameHere);
	ProtectTempFile.Unlock();
	return ret;
}


void RemoveFile(const char *FileName)
{
	if(FileName == NULL)
		return;
	if(!strlen(FileName))
		return;
	unlink(FileName);
}


int FileExist(const char *FileName)
{
	if(FileName == NULL)
		return 0;
	if(!strlen(FileName))
		return 0;
	FILE *stream = Nvfopen(FileName, "rb");
	if(stream == NULL)
		return 0;
	fclose(stream);
	return 1;
}


int DirExist(const char *DirName)
{
	if(DirName == NULL)
		return 0;

	char DirNameHere[NV_MAX_PATHLENGTH];
	strcpy(DirNameHere, DirName);
	if(strlen(DirNameHere))			// we remove eventual last slash
		if(DirNameHere[strlen(DirNameHere)-1] == '/')
			DirNameHere[strlen(DirNameHere)-1] = '\0';
	if(strlen(DirNameHere))			// we remove eventual last backslash
		if(DirNameHere[strlen(DirNameHere)-1] == '\\')
			DirNameHere[strlen(DirNameHere)-1] = '\0';
	if(!strlen(DirNameHere))
		return 0;
#ifdef WIN32
	// We use functions for finding directories given a path
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(DirNameHere, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return 0;	// No file
	FindClose(hFileFind);	// we close the finder
#else

	// for unix, we try to open the directory file

	// open directory file
	DIR *Dir = opendir(DirNameHere);
	if(Dir == NULL)
		return 0;		// cannot open directory file
	closedir(Dir);	// close the directory file
#endif

	return 1;
}


bool DirList(const char *DirName, const char *Root, vector<string> *List)
{
	if(DirName == NULL)
		return false;

	char DirNameHere[NV_MAX_PATHLENGTH];
	strcpy(DirNameHere, DirName);
	char FilesToFind[NV_MAX_PATHLENGTH];
#ifdef WIN32
	strcpy(FilesToFind, DirNameHere);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	if(Root == NULL)
		strcat(FilesToFind, "*");
	else
		strcat(FilesToFind, Root);
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return true;	// No file
	while(1)
	{
		// We remove all the found files
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0)
		{
			if(FindData.cFileName[0] != '.')
			{
				string FoundDirName = FindData.cFileName;
				List->push_back(FoundDirName);	// This directory is a good candidate
			}
		}

		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file

	}
	FindClose(hFileFind);	// we close the finder
#else

	string RootHere = "";
	if(Root != NULL)
		RootHere = Root;
	if(RootHere == "*")
		RootHere = "";
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strcpy(WorkDirHere, DirNameHere);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return true;		// cannot open directory file
	
	dirent *DirEntry;
	while((DirEntry = readdir(Dir)) != 0)
	{
		if(DirEntry->d_name[0] == '.')
			continue;
		strcpy(FilesToFind, WorkDirHere);
		if(strlen(FilesToFind))
		{
			// We add the last \ character
			if(FilesToFind[strlen(FilesToFind) - 1] != '/')
				strcat(FilesToFind, "/");
		}
		strcat(FilesToFind, DirEntry->d_name);
		DIR *DirHere = NULL;
		if((DirHere = opendir(FilesToFind)) == NULL)
			continue;
		closedir(DirHere);
		// We have found a real subdirectory
		// The subdirectory name seems to be OK
		string FoundDirName = DirEntry->d_name;
		if(RootHere.length())
			if(strstr(DirEntry->d_name, RootHere.c_str()) != DirEntry->d_name)
				continue;
		List->push_back(FoundDirName);	// This directory is a good candidate
	}
	closedir(Dir);	// close the directory file
#endif

	return true;
}


bool IsDirEmpty(const char *DirName)
{
	if(DirName == NULL)
		return true;

	bool IsEmpty = true;

	char DirNameHere[NV_MAX_PATHLENGTH];
	strcpy(DirNameHere, DirName);
#ifdef WIN32
	char FilesToFind[NV_MAX_PATHLENGTH];
	strcpy(FilesToFind, DirNameHere);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	strcat(FilesToFind, "*");
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return true;	// No file
	while(1)
	{
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
		{
			// File
			IsEmpty = false;
			break;
		}
		else
		{
			// Directory
			if(FindData.cFileName[0] != '.')
			{
				IsEmpty = false;
				break;
			}
		}

		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file

	}
	FindClose(hFileFind);	// we close the finder
#else

	char WorkDirHere[NV_MAX_PATHLENGTH];
	strcpy(WorkDirHere, DirNameHere);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return true;		// cannot open directory file
	
	dirent *DirEntry;
	while((DirEntry = readdir(Dir)) != 0)
	{
		if(DirEntry->d_name[0] != '.')
		{
			// We have found a file or a directory
			IsEmpty = false;
			break;
		}
	}
	closedir(Dir);	// close the directory file
#endif

	return IsEmpty;
}


bool FileList(const char *DirName, const char *Root, vector<string> *List)
{
	if(DirName == NULL)
		return false;

	char DirNameHere[NV_MAX_PATHLENGTH];
	strcpy(DirNameHere, DirName);
	char FilesToFind[NV_MAX_PATHLENGTH];
#ifdef WIN32
	strcpy(FilesToFind, DirNameHere);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	if(Root == NULL)
		strcat(FilesToFind, "*");
	else
		strcat(FilesToFind, Root);
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return true;	// No file
	while(1)
	{
		// We remove all the found files
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
		{
			string FoundFileName = FindData.cFileName;
			List->push_back(FoundFileName);
		}

		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file

	}
	FindClose(hFileFind);	// we close the finder
#else

	string RootHere = "";
	if(Root != NULL)
		RootHere = Root;
	if(RootHere == "*")
		RootHere = "";
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strcpy(WorkDirHere, DirNameHere);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return true;		// cannot open directory file
	
	dirent *DirEntry;
	while((DirEntry = readdir(Dir)) != 0)
	{
		if(DirEntry->d_name[0] == '.')
			continue;
		strcpy(FilesToFind, WorkDirHere);
		if(strlen(FilesToFind))
		{
			// We add the last \ character
			if(FilesToFind[strlen(FilesToFind) - 1] != '/')
				strcat(FilesToFind, "/");
		}
		strcat(FilesToFind, DirEntry->d_name);
		DIR *DirHere = NULL;
		if((DirHere = opendir(FilesToFind)) != NULL)
		{
			// This is a directory
			closedir(DirHere);
			continue;
		}
		// We have found a real file
		string FoundFileName = DirEntry->d_name;
		if(RootHere.length())
			if(strstr(DirEntry->d_name, RootHere.c_str()) != DirEntry->d_name)
				continue;
		List->push_back(FoundFileName);	// This directory is a good candidate
	}
	closedir(Dir);	// close the directory file
#endif

	return true;
}




bool CreateDir(const char *DirName)
{
	if(DirName == NULL)
		return false;

	if(DirExist(DirName))
		return true;
	
#ifdef WIN32
		if(mkdir(DirName) != 0)
#else
		if(mkdir(DirName, S_IRWXU | S_IRWXG | S_IRWXO) != 0)
#endif
			return false;

	return true;
}



// Removes a directory and all its subdirectories and files
bool RemoveDir(const char *DirName, bool Self, const char *FileRoot, const char *SubDirRoot)
{
	if(DirName == NULL)
		return true;

	if(!DirExist(DirName))
		return true;

	bool WithFiles = true;
	bool WithSubDir = true;

	char FilesToRemove[NV_MAX_PATHLENGTH];
	char DirsToRemove[NV_MAX_PATHLENGTH];
	strcpy(FilesToRemove, "");
	strcpy(DirsToRemove, "");


	if(Self)
	{
		WithSubDir = true;
		WithFiles = true;
	}
	else
	{
		if(SubDirRoot != NULL)
		{
			if(strlen(SubDirRoot))
				strcpy(DirsToRemove, SubDirRoot);
			else
				WithSubDir = false;
		}
		if(FileRoot != NULL)
		{
			if(strlen(FileRoot))
				strcpy(FilesToRemove, FileRoot);
			else
				WithFiles = false;
		}
		if(strcmp(DirsToRemove, "*") == 0)
			strcpy(DirsToRemove, "");
		if(strcmp(FilesToRemove, "*") == 0)
			strcpy(FilesToRemove, "");
	}


	bool RetVal = true;
	char FileToRemove[NV_MAX_PATHLENGTH];
#ifdef WIN32
	// We use functions for finding files given a path and wildcards
	char FilesToFind[NV_MAX_PATHLENGTH];
	strcpy(FilesToFind, DirName);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	strcat(FilesToFind, "*");
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return false;	// No file
	while(1)
	{
		// We remove all the found files
		strcpy(FileToRemove, DirName);
		if(strlen(FileToRemove))
		{
			// We add the last \ character
			if(FileToRemove[strlen(FileToRemove) - 1] != '\\')
				strcat(FileToRemove, "\\");
		}
		strcat(FileToRemove, FindData.cFileName);
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
		{
			if(WithFiles)
			{
				bool ToRemove = false;
				if(strlen(FilesToRemove))
				{
					// Case insensitive for windows
					if(strstrnocase(FindData.cFileName, FilesToRemove) == FindData.cFileName)
						ToRemove = true;
				}
				else
					ToRemove = true;
				if(ToRemove)
					if(unlink(FileToRemove) != 0)
						RetVal = false;
			}
		}
		else if(FindData.cFileName[0] != '.')
		{
			if(WithSubDir)
			{
				bool ToRemove = false;
				if(strlen(DirsToRemove))
				{
					// Case insensitive for windows
					if(strstrnocase(FindData.cFileName, DirsToRemove) == FindData.cFileName)
						ToRemove = true;
				}
				else
					ToRemove = true;
				if(ToRemove)
					if(!RemoveDir(FileToRemove))
						RetVal = false;
			}
		}
		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file
	}
	FindClose(hFileFind);	// we close the finder
#else

	// for unix, we open the directory file and we search for entries
	// corresponding to our temporary file names
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strcpy(WorkDirHere, DirName);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return false;		// cannot open directory file
	
	dirent *DirEntry;
	while((DirEntry = readdir(Dir)) != 0)
	{
		// we remove files
		strcpy(FileToRemove, WorkDirHere);
		if(strlen(FileToRemove))
		{
			// We add the last \ character
			if(FileToRemove[strlen(FileToRemove) - 1] != '/')
				strcat(FileToRemove, "/");
		}
		strcat(FileToRemove, DirEntry->d_name);
		if(DirExist(FileToRemove))
		{
			// Its a directory
			if(WithSubDir && (DirEntry->d_name[0] != '.'))
			{
				bool ToRemove = false;
				if(strlen(DirsToRemove))
				{
					if(strstr(DirEntry->d_name, DirsToRemove) == DirEntry->d_name)
						ToRemove = true;
				}
				else
					ToRemove = true;
				if(ToRemove)
					if(!RemoveDir(FileToRemove))
						RetVal = false;
			}
		}
		else
		{
			//it's a file
			if(WithFiles)
			{
				bool ToRemove = false;
				if(strlen(FilesToRemove))
				{
					if(strstr(DirEntry->d_name, FilesToRemove) == DirEntry->d_name)
						ToRemove = true;
				}
				else
					ToRemove = true;
				if(ToRemove)
					if(unlink(FileToRemove) != 0)
						RetVal = false;
			}
		}

	}
	closedir(Dir);	// close the directory file
#endif


	if(Self)
		if(rmdir(DirName) != 0)
			return false;

	return RetVal;
}


int freadline(char *Line, int LineLength, FILE *stream)
{
	int LineLengthHere = LineLength-1;
	if(LineLength == 0)
		LineLengthHere = 0;

	if(fgets(Line, LineLengthHere, stream) <= 0)
		return 0;

	if(Line[strlen(Line)-1] == '\n')
		Line[strlen(Line)-1] = '\0';
	if(Line[strlen(Line)-1] == '\r')
		Line[strlen(Line)-1] = '\0';

	return 1;
}

	
char *strstrnocase(const char *string, const char *strCharSet)
{
	if(string == NULL)
		return NULL;
	if(strCharSet == NULL)
		return NULL;

	char *stringhere = (char *)malloc(strlen(string)+1);
	char *strCharSethere = (char *)malloc(strlen(strCharSet)+1);
	strcpy(stringhere, string);
	strcpy(strCharSethere, strCharSet);
	strlwr(stringhere);
	strlwr(strCharSethere);
	char *buf = strstr(stringhere, strCharSethere);
	if(buf != NULL)
	{
		int index = (int)(buf-stringhere);
		buf = (char *)string + index;
	}
	free(stringhere);
	free(strCharSethere);

	return buf;
}


int strcmpnocase(const char *string1, const char *string2)
{
	if(string1 == NULL)
		return 0;
	if(string2 == NULL)
		return 0;

	char *string1here = (char *)malloc(strlen(string1)+1);
	char *string2here = (char *)malloc(strlen(string2)+1);
	strcpy(string1here, string1);
	strcpy(string2here, string2);
	strlwr(string1here);
	strlwr(string2here);
	int ret = strcmp(string1here, string2here);
	free(string1here);
	free(string2here);

	return ret;
}


void CleanString(char *String)
{
	if(String == NULL)
		return;
	if(!strlen(String))
		return;
	char *String2 = (char *)malloc(strlen(String) + 1);
	char *buf = String2;
	unsigned int i;
	for(i=0; i<strlen(String); i++)
	{
		if(String[i] == '\t')	// replace tab with space
			String[i] = ' ';
		if(String[i] == '\n')	// replace line feed with space
			String[i] = ' ';
		if(String[i] == '\r')	// eliminate carriage return
			continue;
		if(i && (String[i] == ' ') && (String[i-1] == ' '))	// concatenate 2 consecutive spaces
			continue;
		if((buf == String2) && (String[i] == ' '))	// eliminate first space
			continue;
		*buf = String[i];
		buf++;
	}
	*buf = '\0';	// we end the string
	if(strlen(String2))			// we remove eventual last blank
		if(String2[strlen(String2)-1] == ' ')
			String2[strlen(String2)-1] = '\0';

	strcpy(String, String2);
	free(String2);

}
	
// make some cleaning on strings
void CleanString(string& StringToClean)
{
	if(!StringToClean.size())
		return;
	
	string::size_type index;
	
	// replace tab with space
	while((index = StringToClean.find('\t')) != string::npos)
		StringToClean.replace(index, 1, 1, ' ');
	// remove line feeds
	while((index = StringToClean.find('\n')) != string::npos)
		StringToClean.erase(index, 1);
	// remove carriage returns
	while((index = StringToClean.find('\r')) != string::npos)
		StringToClean.erase(index, 1);
	// concatenate 2 consecutive spaces
	while((index = StringToClean.find("  ")) != string::npos)
		StringToClean.replace(index, 2, " ");
	// eliminate first space
	if(StringToClean.size() && StringToClean[0] == ' ')
		StringToClean.erase(0, 1);
	// eliminate last space
	if(StringToClean.size() && StringToClean[StringToClean.size()-1] == ' ')
		StringToClean.erase(StringToClean.size()-1, 1);
	
}


	

	
// make some cleaning on strings
void AddDirSuffix(string& DirName)
{
#ifdef WIN32
	if(DirName.size() && DirName[DirName.size()-1] != '\\')
		DirName += "\\";
#else
	if(DirName.size() && DirName[DirName.size()-1] != '/')
		DirName += "/";
#endif
}

	
// make some cleaning on strings
void RemoveDirSuffix(string& DirName)
{
	// eliminate last slash
	if(DirName.size() && DirName[DirName.size()-1] == '/')
		DirName.erase(DirName.size()-1, 1);
	// eliminate last backslash
	if(DirName.size() && DirName[DirName.size()-1] == '\\')
		DirName.erase(DirName.size()-1, 1);
}


	


bool CompareWorkDir(string Dir1, string Dir2)
{
	CleanString(Dir1);
	CleanString(Dir2);
	
	string::size_type index;
	// remove slashes
	while((index = Dir1.find('/')) != string::npos)
		Dir1.erase(index, 1);
	while((index = Dir2.find('/')) != string::npos)
		Dir2.erase(index, 1);
	// remove backslashes
	while((index = Dir1.find('\\')) != string::npos)
		Dir1.erase(index, 1);
	while((index = Dir2.find('\\')) != string::npos)
		Dir2.erase(index, 1);

#ifdef WIN32
	MakeLower(Dir1);
	MakeLower(Dir2);
#endif

	if(!Dir1.length())
		return false;

	if(!Dir2.length())
		return false;

	if(Dir1 == Dir2)
		return true;

	return false;
}

	
	

	
// Add /r and /n to a string
void AddEndLine(string& String)
{
#ifdef WIN32
	String += "\r";
#endif
	String += "\n";
}



void RemoveFirstAndEndBlanks(string& String)
{
	// eliminate first spaces
	while(String.size() && String[0] == ' ')
		String.erase(0, 1);
	// eliminate last spaces
	while(String.size() && String[String.size()-1] == ' ')
		String.erase(String.size()-1, 1);
}

	
void RemoveDoubleQuotes(string& String)
{
	string NewValue = "";
	while(1)
	{
		string::size_type index = String.find('"');
		if(index == string::npos)
			break;
		NewValue += String.substr(0, index+1);
		if((index+1) >= String.size())
			break;
		String = String.substr(index+1);
		if(!String.length())
			break;
		if(String[0] == '"')
			String = String.substr(1);
	}
	NewValue += String;
	String = NewValue;
}


//**********************************************************************************
//**********************************************************************************
// Read information from an ini file
//**********************************************************************************
//**********************************************************************************
void GetFromProfile(const char *FileName, char *Output, unsigned int size, const char *Section, const char *Entry, const char *Default)
{
#ifdef WIN32
	// for windows, the function exists
	GetPrivateProfileString(Section, Entry, Default, Output, size, FileName);
#else
	strcpy(Output, Default);	// We write the default to the output
	char Line[256];
	char SectionHere[100];
	char EntryHere[100];
	// we prepare string for comparaison
//#ifdef AIX
//	sprintf(SectionHere, "[%s]", Section);
//#else
	snprintf(SectionHere, sizeof(SectionHere), "[%s]", Section);
//#endif
	strlwr(SectionHere);	// we translate section to lowercase for comparaison
//#ifdef AIX
//	sprintf(EntryHere, "%s=", Entry);
//#else
	snprintf(EntryHere, sizeof(EntryHere), "%s=", Entry);
//#endif
	strlwr(EntryHere);	// we translate entry to lowercase for comparaison
	FILE *stream = Nvfopen(FileName, "rt");
	if(stream == NULL)
		return;
	int SectionFound = 0;
	while(fgets(Line, sizeof(Line)-1, stream) > 0)
	{
		if(Line[strlen(Line)-1] == '\n')
			Line[strlen(Line)-1] = '\0';
		if(Line[strlen(Line)-1] == '\r')
			Line[strlen(Line)-1] = '\0';
		
		if(strlen(Line) == 0)			// read next line if line is blank
			continue;

		char *Line2 = (char *)malloc(strlen(Line) + 1);
		char *Line3 = (char *)malloc(strlen(Line) + 1);
		char *buf = Line2;
		unsigned int i;
		for(i=0; i<strlen(Line); i++)
		{
			if(Line[i] == '\t')	// replace tab with space
				Line[i] = ' ';
			if(i && (Line[i] == ' ') && (Line[i-1] == ' '))	// concatenate 2 consecutive spaces
				continue;
			if(i && (Line[i] == ' ') && (Line[i-1] == '='))	// remove space after '=' character
				continue;
			if(i && (Line[i] == ' ') && (Line[i-1] == '['))	// remove space after '[' character
				continue;
			if((Line[i] == ' ') && (Line[i+1] == '='))	// remove space before '=' character
				continue;
			if((Line[i] == ' ') && (Line[i+1] == '['))	// remove space before '[' character
				continue;
			if((Line[i] == ' ') && (Line[i+1] == ']'))	// remove space before ']' character
				continue;
			if((buf == Line2) && (Line[i] == ' '))	// eliminate first space
				continue;
			*buf++ = Line[i];
		}
		*buf = '\0';	// we end the string
		if(strlen(Line2))			// we remove eventual last blank
			if(Line2[strlen(Line2)-1] == ' ')
				Line2[strlen(Line2)-1] = '\0';
		strcpy(Line3, Line2);
		strlwr(Line3);			// Line3 is used for comparaison
		if(strlen(Line3) == 0)	// go to next line if this one is empty
		{
			free(Line2);
			free(Line3);
			continue;
		}
		
		if(SectionFound)
		{
			if(Line3[0] == '[')	// other section found
			{
				free(Line2);
				free(Line3);
				break;			// so break without success
			}
			if(strstr(Line3, EntryHere) == Line3)
			{
				// we found the entry
				// so we stop after getting the value
				strncpy(Output, Line2 + strlen(EntryHere), size);
				free(Line2);
				free(Line3);
				break;
			}
		}
		else
		{
			if(strstr(Line3, SectionHere) == Line3)
				SectionFound = 1;	// We have found the section
		}

		free(Line2);
		free(Line3);
	}
	
	fclose(stream);	       // close ini file
#endif
}





//**********************************************************************************
//**********************************************************************************
// removes the temporary files, if some exist, from the working directory
//**********************************************************************************
//**********************************************************************************
void RemoveFiles(const char *WorkDir, const char *Root)
{
	char FileToRemove[NV_MAX_PATHLENGTH];
#ifdef WIN32
	// We use functions for finding files given a path and wildcards
	char FilesToFind[NV_MAX_PATHLENGTH];
	strcpy(FilesToFind, WorkDir);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	strcat(FilesToFind, Root);
	strcat(FilesToFind, "*");
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return;	// No file
	while(1)
	{
		// We remove all the found files
		strcpy(FileToRemove, WorkDir);
		strcat(FileToRemove, FindData.cFileName);
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
			unlink(FileToRemove);
		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file
	}
	FindClose(hFileFind);	// we close the finder
#else

	// for unix, we open the directory file and we search for entries
	// corresponding to our temporary file names
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strcpy(WorkDirHere, WorkDir);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return;		// cannot open directory file
	
	dirent *DirEntry;
	bool AllFiles = false;
	if(strcmp(Root, "*") == 0)
		AllFiles = true;
	while((DirEntry = readdir(Dir)) != 0)
	{
		// we remove files
		if(AllFiles || (strstr(DirEntry->d_name, Root) == DirEntry->d_name))
		{
			strcpy(FileToRemove, WorkDir);
			strcat(FileToRemove, DirEntry->d_name);
			unlink(FileToRemove);
		}
	}
	closedir(Dir);	// close the directory file
#endif
}






//**********************************************************************************
//**********************************************************************************
// get the first file of a directory corresponding to the search string
//**********************************************************************************
//**********************************************************************************
string GetFirstFileDir(const char *WorkDir, const char *Root)
{
	string RetVal = "";
	char FileToSearch[NV_MAX_PATHLENGTH];
#ifdef WIN32
	// We use functions for finding files given a path and wildcards
	char FilesToFind[NV_MAX_PATHLENGTH];
	strcpy(FilesToFind, WorkDir);
	if(strlen(FilesToFind))
	{
		// We add the last \ character
		if(FilesToFind[strlen(FilesToFind) - 1] != '\\')
			strcat(FilesToFind, "\\");
	}
	strcat(FilesToFind, Root);
	strcat(FilesToFind, "*");
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return RetVal;	// No file
	while(1)
	{
		strcpy(FileToSearch, WorkDir);
		strcat(FileToSearch, FindData.cFileName);
		if((FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
		{
			RetVal = FileToSearch;
			break;
		}
		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file
	}
	FindClose(hFileFind);	// we close the finder
#else

	// for unix, we open the directory file and we search for entries
	// corresponding to our temporary file names
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strcpy(WorkDirHere, WorkDir);
	if(strlen(WorkDirHere))
	{
		// We remove the last / character
		if(WorkDirHere[strlen(WorkDirHere) - 1] == '/')
			WorkDirHere[strlen(WorkDirHere) - 1] = '\0';
	}
	else
		strcpy(WorkDirHere, ".");	// we are in the current directory

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return RetVal;		// cannot open directory file
	
	dirent *DirEntry;
	bool AllFiles = false;
	if(strcmp(Root, "*") == 0)
		AllFiles = true;
	while((DirEntry = readdir(Dir)) != 0)
	{
		// we remove files
		if(AllFiles || (strstr(DirEntry->d_name, Root) == DirEntry->d_name))
		{
			strcpy(FileToSearch, WorkDir);
			strcat(FileToSearch, DirEntry->d_name);
			RetVal = FileToSearch;
			break;
		}
	}
	closedir(Dir);	// close the directory file
#endif

	return RetVal;
}








//**********************************************************************************
//**********************************************************************************
// returns the name of a file without reagrding its case
//**********************************************************************************
//**********************************************************************************
bool GetNoCaseRealFileName(string& FileName)
{
	bool RetVal = false;
	string FileNameHere = FileName;
	string DirName = ".";
	string::size_type index = FileNameHere.rfind('/');
	if(index == string::npos)
		index = FileNameHere.rfind('\\');
	if(index != string::npos)
	{
		DirName = FileNameHere.substr(0, index);
		FileNameHere = FileNameHere.substr(index+1);
	}
	MakeLower(FileNameHere);

#ifdef WIN32
	// We use functions for finding files given a path and wildcards
	char FilesToFind[NV_MAX_PATHLENGTH];
	strcpy(FilesToFind, DirName.c_str());
	strcat(FilesToFind, "\\");
	strcat(FilesToFind, "*");
	WIN32_FIND_DATA FindData;
	// Try to find the first file
	HANDLE hFileFind = FindFirstFile(FilesToFind, &FindData);
	if(hFileFind == INVALID_HANDLE_VALUE)
		return RetVal;	// No file
	while(1)
	{
		string FileInDir = FindData.cFileName;
		MakeLower(FileInDir);
		if(FileInDir == FileNameHere)
		{
			FileName = DirName;
			FileName += "\\";
			FileName += FindData.cFileName;
			RetVal = true;
			break;
		}
		if(!FindNextFile(hFileFind, &FindData))
			break;	// no more file
	}
	FindClose(hFileFind);	// we close the finder
#else

	// for unix, we open the directory file and we search for entries
	// corresponding to our temporary file names
	char WorkDirHere[NV_MAX_PATHLENGTH];
	strcpy(WorkDirHere, DirName.c_str());

	// open directory file
	DIR *Dir = opendir(WorkDirHere);
	if(Dir == NULL)
		return RetVal;		// cannot open directory file
	
	dirent *DirEntry;
	while((DirEntry = readdir(Dir)) != 0)
	{
		string FileInDir = DirEntry->d_name;
		MakeLower(FileInDir);
		if(FileInDir == FileNameHere)
		{
			FileName = DirName;
			FileName += "/";
			FileName += DirEntry->d_name;
			RetVal = true;
			break;
		}
	}
	closedir(Dir);	// close the directory file
#endif

	return RetVal;
}



unsigned int GetAllocatedMemory()
{
#ifdef WIN32
	MEMORYSTATUS Status;
	GlobalMemoryStatus(&Status);
	return (unsigned int)(Status.dwTotalVirtual - Status.dwAvailVirtual);
#endif
	return 0;	// for now for unix
}



// Return codes: -1 = error, -2 = time out, 0 = OK
int ExecPrgm(const char *CommandLine, int *RetCode, int TimeOut)
{
	*RetCode = 0;

	int RetVal = 0;

	if(CommandLine == NULL)
		return -1;

	string CommandLineHere = CommandLine;
	CleanString(CommandLineHere);
	if(!CommandLineHere.length())
		return -1;

#ifdef WIN32
	char *buffer = new char[NV_MAX_PATHLENGTH];
	::GetCurrentDirectory(NV_MAX_PATHLENGTH, buffer);
	string CurrentDirectory = buffer;
	delete buffer;

	if(CurrentDirectory[CurrentDirectory.length() -1] != '\\')
		CurrentDirectory += "\\";

	string::size_type index;

	// we try to get the current directory from command line
	string Executable = CommandLineHere;
	if((Executable[0] == '\"') || (Executable[0] == '\''))
	{
		// The command probably contain spaces
		Executable.erase(0, 1);
		index = Executable.find('\"');
		if(index == string::npos)
			index = Executable.find('\'');
		if(index != string::npos)
			Executable = Executable.substr(0, index);
	
		// remove all double quotes
		while((index = Executable.find('\"')) != string::npos)
			Executable.erase(index, 1);
	
		// remove all quotes
		while((index = Executable.find('\'')) != string::npos)
			Executable.erase(index, 1);
	}
	else
	{
		// The command doesn't contain spaces
		// so the end of the executable is marked by an eventual space
		index = Executable.find(' ');
		if(index != string::npos)
			Executable = Executable.substr(0, index);
	}

	index = Executable.rfind('\\');
	if(index == string::npos)
		index = Executable.rfind('/');
	if(index != string::npos)
		CurrentDirectory = Executable.substr(0, index+1);


	buffer = new char[NV_MAX_PATHLENGTH];
	strncpy(buffer, CommandLineHere.c_str(), NV_MAX_PATHLENGTH-1);

	
	PROCESS_INFORMATION ProcessInfo;

	STARTUPINFO StartInfo = {sizeof(STARTUPINFO), NULL, NULL, NULL,
					0, 0, 0, 0, 0, 0, 0,
					STARTF_USESHOWWINDOW, SW_HIDE,
					0, NULL, NULL, NULL, NULL};
	

	if(!CreateProcess(NULL, buffer, NULL, NULL,
						FALSE, NORMAL_PRIORITY_CLASS, NULL,
						CurrentDirectory.c_str(), &StartInfo,
						&ProcessInfo))
	{
		delete buffer;
		return -1;
	}
	
	delete buffer;

	DWORD ReturnCode;
	
	int CheckEvery = 1;
	int CheckEverySteps = 1;
	int InitialTime = (int)time(NULL);
	while(GetExitCodeProcess(ProcessInfo.hProcess, &ReturnCode))
	{
		SLEEP(10);
		if(ReturnCode != STILL_ACTIVE)
		{
			*RetCode = ReturnCode;
			break;
		}

		if(TimeOut == 0)
				break;		// No wait, returns immediatly
		if(TimeOut != -1)
		{
			int ActualTime = (int)time(NULL);
			if((ActualTime - InitialTime) > TimeOut)
			{
				// Time out, we try to kill the process
				TerminateProcess(ProcessInfo.hProcess, -1);
				RetVal = -2;	// Time out condition
				break;
			}
		}
		SLEEP(CheckEvery);

		// We first do the check very often but less and less while the time is growing
		CheckEvery += CheckEverySteps;
		CheckEverySteps = (CheckEvery/10)+1;
		if(CheckEverySteps > 10000)		// When the check time is 100 seconds, the steps are maximum of 10 seconds
			CheckEverySteps = 10000;
	}
	CloseHandle(ProcessInfo.hProcess);
	CloseHandle(ProcessInfo.hThread);

#else

	int pid;
	int status;

	char buffer[NV_MAX_PATHLENGTH];
	strncpy(buffer, CommandLineHere.c_str(), NV_MAX_PATHLENGTH-1);


	pid = fork();
	if(pid == -1)
		return -1;
	if(pid == 0)
	{
		// We are in the child
		char *argv[4];
		argv[0] = (char *)"sh";
		argv[1] = (char *)"-c";
		argv[2] = buffer;
		argv[3] = 0;
		execve("/bin/sh", argv, environ);
		exit(-1);
	}

	int CheckEvery = 1;
	int CheckEverySteps = 1;
	int InitialTime = (int)time(NULL);
	while(1)
	{
		SLEEP(10);
		pid_t child_pid = waitpid(pid, &status, WNOHANG);

		if(child_pid > 0)
		{
			// The process has terminated
			if(!WIFEXITED(status))
				RetVal = -1;	// Error running filter
			else if(WEXITSTATUS(status) == 127)
				RetVal = -1;	// Error running filter
			else
				*RetCode = WEXITSTATUS(status);	// Good return
			break;
		}

		if(TimeOut == 0)
				break;		// No wait, returns immediatly
		if(TimeOut != -1)
		{
			int ActualTime = (int)time(NULL);
			if((ActualTime - InitialTime) > TimeOut)
			{
				// Time out, we try to kill the process
				kill(pid, SIGTERM);
				waitpid(pid, &status, 0);	// We must wait to avoid zombies
				RetVal = -2;	// Time out condition
				break;
			}
		}
		SLEEP(CheckEvery);

		// We first do the check very often but less and less while the time is growing
		CheckEvery += CheckEverySteps;
		CheckEverySteps = (CheckEvery/10)+1;
		if(CheckEverySteps > 10000)		// When the check time is 100 seconds, the steps are maximum of 10 seconds
			CheckEverySteps = 10000;
	}

#endif
	return RetVal;
}



// Loads a library
HMODULE MyLoadLibrary(const char *LibraryPath)
{
	if(LibraryPath == NULL)
		return NULL;
	if(!strlen(LibraryPath))
		return NULL;
	HMODULE RetHandle = NULL;
#ifdef WIN32
	RetHandle = LoadLibrary(LibraryPath);
#else
#ifdef SOLARIS
	RetHandle = dlopen(LibraryPath, RTLD_NOW | RTLD_GLOBAL | RTLD_GROUP);
#else
	RetHandle = dlopen(LibraryPath, RTLD_LAZY);
#endif
#endif
	return RetHandle;
}


// Unload a library
void MyFreeLibrary(HMODULE hModule)
{
	if(hModule == NULL)
		return;

#ifdef WIN32
	FreeLibrary(hModule);
#else
	dlclose(hModule);
#endif
}


// Gets a pointer to a given function in a library
FARPROC MyGetLibFunction(HMODULE hModule, const char *FunctionName)
{
	if(hModule == NULL)
		return NULL;
	if(FunctionName == NULL)
		return NULL;
	if(!strlen(FunctionName))
		return NULL;

	FARPROC RetVal = NULL;
#ifdef WIN32
	RetVal = GetProcAddress(hModule, FunctionName);
#else
	RetVal = (FARPROC)dlsym(hModule, FunctionName);
#endif

	return RetVal;
}


// Encode 
// replaces some characters with special ones for xml
string EncodeStringForXML(string StringToEncode)
{

	string Result = "";
	
	string::size_type index;
	while((index = StringToEncode.find("&lt;")) != string::npos)
		StringToEncode.replace(index, 4, "<");
	while((index = StringToEncode.find("&gt;")) != string::npos)
		StringToEncode.replace(index, 4, ">");
	while((index = StringToEncode.find("&apos;")) != string::npos)
		StringToEncode.replace(index, 6, "\'");
	while((index = StringToEncode.find("&quot;")) != string::npos)
		StringToEncode.replace(index, 6, "\"");
	while((index = StringToEncode.find("&amp;")) != string::npos)
		StringToEncode.replace(index, 5, "&");

	while((index = StringToEncode.find('&')) != string::npos)
		StringToEncode.replace(index, 1, "$$@@##%%amp;");
	while((index = StringToEncode.find('<')) != string::npos)
		StringToEncode.replace(index, 1, "&lt;");
	while((index = StringToEncode.find('>')) != string::npos)
		StringToEncode.replace(index, 1, "&gt;");
	while((index = StringToEncode.find('\'')) != string::npos)
		StringToEncode.replace(index, 1, "&apos;");
	while((index = StringToEncode.find('\"')) != string::npos)
		StringToEncode.replace(index, 1, "&quot;");
	while((index = StringToEncode.find("$$@@##%%")) != string::npos)
		StringToEncode.replace(index, 8, "&");

	return StringToEncode;
}

// Sets the file name for UNIX or WINDOWS
void SetDirName(std::string& DirName)
{
	string::size_type index;
	
#ifdef WIN32
	// replace \ with /
	while((index = DirName.find('/')) != string::npos)
		DirName.replace(index, 1, 1, '\\');
#else
	// replace \ with /
	while((index = DirName.find('\\')) != string::npos)
		DirName.replace(index, 1, 1, '/');
#endif
}


// Reformat a date
// returns false if the date format is not valid
bool ArrangeDate(std::string& DateToArrange)
{
	string DateToArrangeHere = DateToArrange;
	CleanString(DateToArrangeHere);
	MakeUpper(DateToArrangeHere);

	string::size_type index;
	// replace / with .
	while((index = DateToArrangeHere.find('/')) != string::npos)
		DateToArrangeHere.replace(index, 1, 1, '.');
	// replace space with .
	while((index = DateToArrangeHere.find(' ')) != string::npos)
		DateToArrangeHere.replace(index, 1, 1, '.');
	// replace \ with .
	while((index = DateToArrangeHere.find('\\')) != string::npos)
		DateToArrangeHere.replace(index, 1, 1, '.');

	if(!DateToArrangeHere.length())
		return false;

	string Day = "";
	string Month = "";
	string Year = "";
	index = DateToArrangeHere.find('.');
	if(index != string::npos)
	{
		Day = DateToArrangeHere.substr(0, index);
		DateToArrangeHere = DateToArrangeHere.substr(index+1);
	}
	index = DateToArrangeHere.find('.');
	if(index != string::npos)
	{
		Month = DateToArrangeHere.substr(0, index);
		Year = DateToArrangeHere.substr(index+1);
	}

	if(!Day.length())
		return false;
	if(!Month.length())
		return false;
	if(!Year.length())
		return false;

	int iDay = atoi(Day.c_str());
	int iMonth = atoi(Month.c_str());
	int iYear = atoi(Year.c_str());

	if(iDay < 1)
		return false;
	if(iDay > 31)
		return false;

	if(iMonth < 1)
		return false;
	if(iMonth > 12)
		return false;

	if(iYear < 100)
	{
		if(iYear < 50)
			iYear += 2000;
		else
			iYear += 1900;
	}

	if(iYear < 1970)
		return false;
	if(iYear > 2037)
		return false;

	char Buf[100];
	sprintf(Buf, "%02d.%02d.%04d", iDay, iMonth, iYear);

	DateToArrange = Buf;

	return true;
}




