// ***************************************************************************************
// File: error.cpp
// Description: Nirva error object implementation
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************


#include "nvs.h"

#ifdef NV_SERVER
	#ifdef WIN32
		#ifdef _DEBUG
			#include <stdlib.h>
			#include <crtdbg.h>
			#define NVNEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
		#else // _DEBUG
			#define NVNEW new
		#endif // _DEBUG
	#else	//WIN32
		#define NVNEW new
	#endif	//WIN32
#else	//NV_SERVER
	#define NVNEW new
#endif	//NV_SERVER

using namespace std;


/////////////////////////////////////////////////////////////////////////////
// NvError

NvError::NvError()
{
	m_iErrorCode = NV_OK;
	m_sErrorService = "";
	m_sErrorClass = "";
	m_sErrorInfo = "";
}

NvError::~NvError()
{
}

void NvError::SetError(string ErrorService, string ErrorClass, int ErrorCode, string ErrorInfo)
{
	m_sErrorService = ErrorService;
	MakeUpper(m_sErrorService);
	m_sErrorClass = ErrorClass;
	MakeUpper(m_sErrorClass);
	m_iErrorCode = ErrorCode;
	m_sErrorInfo = ErrorInfo;
}


string NvError::GetErrorDesc(const char *Language)
{
	// We get the error description from the error tables
	string RetVal = "";
	if(m_iErrorCode == NV_OK)
		return RetVal;
	string ErrorServiceHere = m_sErrorService;
	CleanString(ErrorServiceHere);
	MakeUpper(ErrorServiceHere);
	NvObjTable *ErrorTable = NULL;
	NvService *Service = NULL;
	string ErrorTableName = "ERROR_CLASS_";
	string ErrorClass = m_sErrorClass;
	CleanString(ErrorClass);
	ErrorTableName += ErrorClass;
	MakeUpper(ErrorTableName);
	if(ErrorServiceHere == "SYSTEM")
		ErrorTable = (NvObjTable *)(GetMain()->Info.GetObject(ErrorTableName.c_str()));
	else
	{
		// The error table comes from a service
		Service = GetMain()->Services.GetService(ErrorServiceHere.c_str());
		if(Service != NULL)
		{
			NV_LOCKOBJECT(Service);
			ErrorTable = (NvObjTable *)(Service->m_Info.GetObject(ErrorTableName.c_str()));
		}
	}


	if(ErrorTable != NULL)
	{
		//PM TODO
		NvObjTable ResultTable;
		char Buf[50];
		sprintf(Buf, "%d", m_iErrorCode);
		string ErrorCode = Buf;
		CleanString(ErrorCode);
		string Query = "CODE=";
		Query += ErrorCode;

		if(ErrorTable->Search(Query.c_str(), &ResultTable, (const char *)NULL, 1))
		{
			// We found a document
			if(ResultTable.GetNumRow() == 1)
			{
				string LanguageHere = "ENGLISH";
				if(Language != NULL)
					LanguageHere = Language;
				CleanString(LanguageHere);
				MakeUpper(LanguageHere);
				string FieldLanguage = "DESC_";
				FieldLanguage += LanguageHere;
				int iFieldIndex = ResultTable.GetColumnIndex(FieldLanguage.c_str());
				if(iFieldIndex == -1)
					iFieldIndex = ResultTable.GetColumnIndex("DESC_ENGLISH");
				if(iFieldIndex != -1)
				{
					// We got it
					RetVal = ResultTable.GetCellLine(0, iFieldIndex);
				}
			}
		}

	}

	if(Service != NULL)
	{
		NV_UNLOCKOBJECT(Service);
		GetMain()->Services.FreeService(ErrorServiceHere.c_str());
	}

	return RetVal;
}

