// ***************************************************************************************
// File: nvs.h
// Description: Nvs main server classes.
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************

#ifdef WIN32
#pragma warning( disable : 4996 )
#endif


#ifndef _NVS_H
#define _NVS_H

//#ifdef WIN32
//#define _WIN32_WINNT 0x0500
//#endif

// Classes declared in this file
class NvsServer;

// Uncomment when the bug is solved
// #define	PB_PERMISSION_LIST		1


// XML Models
#define	XML_MODEL_NORMAL	0
#define	XML_MODEL_SIMPLE	1
#define	XML_MODEL_COMPACT	2
#define	XML_MODEL_TINY		3

#include "json.h"
#include "task.h"
#include "encrypt.h"
#include "md5.h"
#include "compress.h"
#include "lzma/lzma.h"
#include "util.h"
#include "base64.h"
#include "error.h"
#include "package.h"
#include "locks.h"
#include "logs.h"
#include "application.h"
#include "service.h"
#include "command.h"
#include "nidget.h"
#include "request.h"
#include "session.h"
#include "http.h"
#include "container.h"
#include "garbage.h"
#include "security.h"
#include "obj.h"
#include "license.h"
#include "transaction.h"
#include "scheduler.h"
#include "perlint.h"
#include "javaint.h"
#include "dotnetint.h"
#include "mqint.h"
#include "webs.h"
#include "xml.h"
#include "sqlite.h"
#include "listener.h"
#include "thread.h"
#include "test.h"
#ifdef NV_SERVER
#ifdef WIN32
#ifdef _DEBUG
#include <crtdbg.h>
#endif
#endif
#endif


#define SZAPPNAME			"nvs"

class NvThread;
class NvThreadList;
class NvsServer;

// nirva service ID used for checking
#define NVS_SERVICE_ID		"6680143801F1CEAE514D637BA2B90E67"

#ifdef WIN32
typedef struct _ASTAT_				// For retrieving ethernet MAC number
{

	ADAPTER_STATUS adapt;
	NAME_BUFFER    NameBuff [30];

}ASTAT, * PASTAT;
#endif
	

#define	LICENSE_NONE	0
#define	LICENSE_DEMO	1
#define	LICENSE_DEV		2
#define	LICENSE_TEST	3
#define	LICENSE_PROD	4
#define	LICENSE_FAIL	5

#define THREAD_TYPE_UNKNOWN					0
#define THREAD_TYPE_HTTP_SERVER				1
#define THREAD_TYPE_HTTP_CLIENT				2
#define THREAD_TYPE_HTTP_CLIENT_BUILDER		3
#define THREAD_TYPE_LISTENER				4
#define THREAD_TYPE_MQ						5
#define THREAD_TYPE_MAIN					6
#define THREAD_TYPE_SCHEDULER_SERVER		7
#define THREAD_TYPE_SCHEDULER_TASK			8
#define THREAD_TYPE_TRANSACTION_SERVER		9
#define THREAD_TYPE_TRANSACTION_CLIENT		10
#define THREAD_TYPE_JAVA					11
#define THREAD_TYPE_PERL					12
#define THREAD_TYPE_DOTNET					13
#define THREAD_TYPE_SESSION					14

#define ATTACH_REQUEST_TYPE_ANY				0
#define ATTACH_REQUEST_TYPE_CLIENT			1
#define ATTACH_REQUEST_TYPE_WEB				2
#define ATTACH_REQUEST_TYPE_TRANSACTION		3
#define ATTACH_REQUEST_TYPE_SCHEDULER		4
#define ATTACH_REQUEST_TYPE_NAMED			5
#define ATTACH_REQUEST_TYPE_LISTENER		6
#define ATTACH_REQUEST_TYPE_THREAD			7
#define ATTACH_REQUEST_TYPE_INTERNAL		8
#define ATTACH_REQUEST_TYPE_WEBS			9
#define ATTACH_REQUEST_TYPE_XML				10
#define ATTACH_REQUEST_TYPE_SOAP			11
#define ATTACH_REQUEST_TYPE_MQ				12


// The NvThread and NvTHreadList classes are used to view thread information
class NvThread
{
public:
	NvMutex LockObject;

	long m_iTid;						// Thread id
	time_t m_tCreationTime;			// last lock use
	int m_iType;					// Thread type

	NvThread();
	~NvThread();
};



class NvThreadList
{
public:
	NvMutex ProtectList;					// Protection for the list
	std::map<long,NvThread *> m_mList;	// thread list

	NvThreadList();
	~NvThreadList();


	void Clear();
	void Add(int Type);
	void Remove();
	bool List(NvObjTable *ThreadList);
};


// The NvAttachRequest and NvAttachRequestList classes are used to managed requests to attach next sessions to consoles
class NvAttachRequest
{
public:
	std::string m_sPipeFile;			// Pipe file name for attached console (unix only)
	PIPE_HANDLE hServerWritePipehandle;	// Write pipe to attached console (unix + windows)
	int m_iAttachedConsolePid;			// Console Pid
	int m_itype;						// Request type
	std::string m_sApplication;			// Application (empty for any)
	bool m_bUpdateToRemove;				// Used in Update
	int m_iLevel;						// Verbose level

	NvAttachRequest();
	~NvAttachRequest();
};


class NvAttachRequestList
{
public:
	NvMutex ProtectList;						// Protection for the list
	std::vector<NvAttachRequest *> m_mList;		// attach request list

	NvAttachRequestList();
	~NvAttachRequestList();

	void Clear();
	bool Add(NvSession *Session, NvCommand *CommandToExecute, int AttachedConsolePid, int Type = ATTACH_REQUEST_TYPE_ANY, std::string Application = "", int Level = 1);
	void Get(NvSession *Session, int Type = ATTACH_REQUEST_TYPE_ANY, std::string Application = "");
	void Update();
};



// Nvs server class. This is the main class of the program
class NvsServer : public NvTask
{
// Attributes
public:
	NvEvent AskToTerminate;
	NvHttpServer HttpServer;				// HTTP server
	NvHttpServer HttpServer6;				// HTTP server
	NvHttpServer HttpServerSSL;				// HTTPS server
	NvHttpServer HttpServerSSL6;			// HTTPS server
	NvApplicationList Applications;
	NvServiceList Services;
	NvWebsList Webservices;
	NvSessionList Sessions;
	NvRegistryContainer SystemRegistry;
	NvRegistryContainer BasicSystemRegistry;
	NvGarbage *Garbage;						// Global garbage collector
	NvObjIndStringList MimeTypes;			// Mime types
	NvObjIndStringList Aliases;				// HTTP Aliases
	NvLockList LockList;					// system lock list
	NvSemaList SemaList;					// system semaphore list
	NvLockList FilterLockList;				// filter lock list
	NvDescriptionContainer Info;			// System information
	NvLicense Licenses;						// Licenses
	NvPerl PerlInterface;					// Interface to perl
	NvJava JavaInterface;					// Interface to java
	NvDotnet DotnetInterface;				// Interface to dotnet
	NvSqLite SqLiteInterface;				// Interface to sqlite
	NvMqLib MqInterface;					// Interface to mq
	NvLcLib LcInterface;					// Interface to license library
	NvMqList Queues;						// Queue list
	NvLogList Logs;							// System logs
	NvTransactionServer TransactionServer;	// Transaction server
	NvSchedulerServer SchedulerServer;		// Scheduler server
	NvHttpConnectionList Connections;		// Current connections to other NIRVA servers
	NvContainerList	Containers;				// System container list
	NvHostList VirtualHosts;				// Virtual host list
	NvRegistryContainerList Registries;		// System user registries
	NvThreadList Threads;					// Nirva threads
	NvAttachRequestList AttachRequestList;	// Requests to attach future sessions	

	// Server parameters
	int m_iHttpServerPort;				// HTTP server port
	int m_iHttpSessionTimeOut;			// HTTP session time out
	int m_iHttpStopWaitTime;			// HTTP stop wait time
	int m_iHttpMaxMemBody;				// HTTP maximum body size
	bool m_bUseSSL;						// Use also an SSL HTTP server (HTTPS)
	int m_iHttpsServerPort;				// HTTPS server port
	bool m_bSSLV3Only;					// Use sslv3 only
	bool m_bHttpLocal;					// Use HTTP local only
	bool m_bHttpIpV6;					// Use HTTP IP V6
	bool m_bHttpBlockServerHeader;		// Send/block the http header server
	bool m_bHttpsUseClientCertificate;	// Request client certificate with HTTPS
	std::string m_sHttpsCAFile;			// File containing trusted Certificate Authorities
	bool m_bAutoDetectEncoding;			// Enable automatic detection of form and url encoding
	std::string m_sNirvaDefaultPage;	// default html page
	bool m_bDebug;						// Debug mode
	bool m_bRemoteAdmin;				// Allow remote admin connection
	std::string m_sXslProcessorPath;	// XSLT processor path
	bool m_bInternalXsl;				// Internal or external XSL processor
	bool m_bEnableXslCache;				// Enbale internal xslt cache or not
	int m_iXslMaxDepth;					// Xsl max depth
	int m_iXslParserMaxDepth;			// Xsl parser max depth
	bool m_bLocalTime;					// Local or GMT time for server time information
	int m_JVMMinMemSize;				// Minimum size for JVM allocation pool in megabytes
	int m_JVMMaxMemSize;				// Maximum size for JVM allocation pool in megabytes
	std::string m_sJVMParam;			// Other jvm parameters
	std::string m_sJVMHomeDir;			// Java home directory for an external JVM
	std::string m_sJVMPathName;			// JVM shared library path name for an external JVM
	bool m_bInternalJvm;				// Internal or external JVM
	bool m_bUseEnvClassLoader;			// Use environment class loader for java procedures
	bool m_bUseClassCache;			// Use java class cache
	std::string m_sRegistryHomeDir;		// Home registry directory
	std::string m_sLogHomeDir;			// Home log directory
	std::string m_sAppWorkHomeDir;		// Home application work directory
	std::string m_sNewRegistryHomeDir;	// New Home registry directory
	std::string m_sNewLogHomeDir;		// New Home log directory
	std::string m_sNewAppWorkHomeDir;	// New Home application work directory
	bool m_bUseTransactionServer;		// Enable the transaction server
	int m_iMaxTransactions;				// Transaction server: max simultaneous transactions
	int m_iTransactionCheckEvery;		// Transaction server: time between each check of input queue
	int m_iMaxSchedulerTasks;			// Scheduler server: max simultaneous tasks
	int m_iSchedulerCheckEvery;			// Scheduler server: time between each check of input queue
	int m_iDefaultAppTimeOut;			// Default time out for the default application (nvdef)
	std::string m_sDefaultAppSecuApp;	// Default secu app for the default application (nvdef)
	bool m_bDefaultAppUseScheduled;		// Default secu app mode for scheduled transactions
	bool m_bDefaultAppUseTrigger;		// Default secu app use trigger
	bool m_bDefaultAppAllowWebXml;		// Default secu app flag for allowing display xml in browser
	bool m_bDefaultAppAllowNotOwnedSession;		// Default secu app flag for allowing clients to access sessions they don't own
	int m_iDefaultAppAuthMode;			// Authentication mode for the default application (nvdef)
	bool m_bDefaultNirvaIsSsoUser;		// Set to true when Nirva user is sso user
	std::string m_sDefaultAppCommandClientFilter;
	std::string m_sDefaultAppCommandWebFilter;
	int m_iDefaultAppCommandClientFilterMode;
	int m_iDefaultAppCommandWebFilterMode;
	bool m_bDemoMode;					// Demonstration mode (Old license mode only)
	int m_iMaxSessions;					// Maximum sessions allowed (-1 = infinite)
	int m_iLicenseType;					// Global license type
	int m_iNumProcs;					// Number of processors
	time_t tStartTime;					// Time of server start
	std::string m_sAdminPassword;		// administrator password (nvadmin user)
	NvSecurity Security;				// Security object that maintains users and roles at system level
	std::string m_sDefaultMailServer;	// Default mail SMTP server
	std::string m_sDefaultMailUser;		// Default mail user for authentication
	std::string m_sDefaultMailPassword;	// Default mail password for authentication
	std::string m_sHostWebs;			// Host address for web services
	unsigned int m_iUniqCounter;		// Uniq string counter (used in GetUniq())
	bool m_bMqAvailable;				// MQ available or not
	bool m_bLcAvailable;				// License library available or not
	bool m_bLongSessionId;				// Flag for using long or short session IDs
	std::string m_sMachineId;			// Optional machine Id used with sessions, the final session ID is sessionid.machineid
	std::string m_sServerStatus;		// Server status can be RUNNING, STARTING, STOPPING or STANDBY
	bool m_bStopRequest;				// Set to true when entering close mode (stop nirva)
	std::string m_sVersion;				// Nirva version
	std::string m_sVersionType;			// Nirva version type
	bool m_bOldObjectsUnicode;			// Tells if persistent objects (registry and storage) without encoding saved are unicode or not
	std::string m_sDebugMode;			// Debug mode of the -g option (string separated by , characters ex. http=1,sso=1)
	std::string m_sCtrlInstance;		// Server instance. Unique string that identifies the server instance.
	NvTestSetList TestSets;				// Test sets
	int m_iNumExceptions;				// Exception and signal counter
	bool m_bDefaultUnicode;				// If true, nirva starts as unicode

#ifdef WIN32
#ifdef _DEBUG
	_CrtMemState s0;
#endif
#endif

	std::string m_sNirvaDir;
	std::string m_sMachineName;
	std::string m_sMachineUser;

	bool b_ConsoleMode;
	bool b_WithTime;
	bool b_Verbose;
	bool b_Crash;
	bool b_Initialized;
	bool b_Unicode;

	// Overridable callbacks
protected:
	virtual int Main();
	virtual void OnException(std::string sInfo);
	virtual bool OnInit();
	virtual void OnCleanup();

public:

	NvsServer();

	void Stop();
	bool LoadInfo();
	bool LoadParameters(bool Default = false, bool NoSave = false);
	bool LoadBasicParameters(bool Default = false);
	bool SaveParameters();
	bool SaveBasicParameters();

#ifdef WIN32
#ifdef _DEBUG
	// Debug only
	void DebugState();
	void DebugDump();
#endif
#endif

	std::string GetNirvaDir();
	std::string GetMachineName();
	std::string GetMachineUser();
	
	std::string GetStatus();
	void SetStatus(const char *Value);
	bool GetSSL();
	void SetSSL(bool Value);
	bool GetSSLV3Only();
	void SetSSLV3Only(bool Value);
	bool GetDefaultUnicode();
	void SetDefaultUnicode(bool Value);
	bool GetHttpLocal();
	void SetHttpLocal(bool Value);
	bool GetHttpIpV6();
	void SetHttpIpV6(bool Value);
	bool GetHttpBlockServerHeader();
	void SetHttpBlockServerHeader(bool Value);
	bool GetHttpsUseClientCertificate();
	void SetHttpsUseClientCertificate(bool Value);
	std::string GetHttpsCAFile();
	bool SetHttpsCAFile(const char *Value);
	bool GetAutoDetectEncoding();
	void SetAutoDetectEncoding(bool Value);
	bool GetLongSessionId();
	void SetLongSessionId(bool Value);
	std::string GetMachineIdForSessions();
	bool SetMachineIdForSessions(const char *Value);
	bool GetOldObjectsUnicode();
	void SetOldObjectsUnicode(bool Value);
	bool GetDebug();
	void SetDebug(bool Value);
	bool GetRemoteAdmin();
	void SetRemoteAdmin(bool Value);
	bool GetInternalXsl();
	void SetInternalXsl(bool Value);
	bool GetEnableXslCache();
	void SetEnableXslCache(bool Value);
	bool GetLocalTime();
	void SetLocalTime(bool Value);
	int GetXslMaxDepth();
	void SetXslMaxDepth(int Value);
	int GetXslParserMaxDepth();
	void SetXslParserMaxDepth(int Value);
	std::string GetXslPath();
	bool SetXslPath(const char *Value);
	int GetHttpPort();
	bool SetHttpPort(int Port);
	int GetHttpsPort();
	bool SetHttpsPort(int Port);
	int GetHttpSessionTimeOut();
	bool SetHttpSessionTimeOut(int SessionTimeOut);
	int GetHttpStopWaitTime();
	bool SetHttpStopWaitTime(int StopWaitTime);
	int GetHttpMaxMemBody();
	bool SetHttpMaxMemBody(int MaxMemBody);
	int GetJvmMinMem();
	bool SetJvmMinMem(int MemSize);
	int GetJvmMaxMem();
	bool SetJvmMaxMem(int MemSize);
	bool GetInternalJvm();
	void SetInternalJvm(bool Value);
	bool GetUseEnvClassLoader();
	void SetUseEnvClassLoader(bool Value);
	bool GetUseClassCache();
	void SetUseClassCache(bool Value);
	std::string GetJvmParam();
	bool SetJvmParam(std::string Value);
	std::string GetJvmHomeDir();
	bool SetJvmHomeDir(std::string HomeDir);
	std::string GetJvmPath();
	bool SetJvmPath(std::string Path);
	bool GetUseTransactionServer();
	void SetUseTransactionServer(bool Value);
	int GetMaxTransactions();
	bool SetMaxTransactions(int MaxTransactions);
	int GetTransactionCheckEvery();
	bool SetTransactionCheckEvery(int CheckEvery);
	int GetMaxSchedulerTasks();
	bool SetMaxSchedulerTasks(int MaxSchedulerTasks);
	int GetSchedulerCheckEvery();
	bool SetSchedulerCheckEvery(int CheckEvery);
	int GetDefaultAppTimeOut();
	bool SetDefaultAppTimeOut(int TimeOut);
	int GetDefaultAppAuthMode();
	bool SetDefaultAppAuthMode(int AuthMode);
	bool GetDefaultAppNirvaIsSsoUser();
	bool SetDefaultAppNirvaIsSsoUser(bool NirvaIsSsoUser);
	std::string GetDefaultAppSecuApp();
	bool SetDefaultAppSecuApp(const char *Value);
	bool GetDefaultAppUseScheduled();
	void SetDefaultAppUseScheduled(bool Value);
	bool GetDefaultAppUseTrigger();
	void SetDefaultAppUseTrigger(bool Value);
	bool GetDefaultAppAllowWebXml();
	void SetDefaultAppAllowWebXml(bool Value);
	bool GetDefaultAppAllowNotOwnedSession();
	void SetDefaultAppAllowNotOwnedSession(bool Value);
	int GetDefaultAppCommandClientFilterMode();
	bool SetDefaultAppCommandClientFilterMode(int Mode);
	std::string GetDefaultAppCommandClientFilter();
	bool SetDefaultAppCommandClientFilter(const char* Value);
	int GetDefaultAppCommandWebFilterMode();
	bool SetDefaultAppCommandWebFilterMode(int Mode);
	std::string GetDefaultAppCommandWebFilter();
	bool SetDefaultAppCommandWebFilter(const char* Value);
	std::string GetRegistryHomeDir(bool NewVal = false);
	bool SetRegistryHomeDir(const char *Value);
	std::string GetLogHomeDir(bool NewVal = false);
	bool SetLogHomeDir(const char *Value);
	std::string GetAppWorkHomeDir(bool NewVal = false);
	bool SetAppWorkHomeDir(const char *Value);
	std::string GetAdminPassword();
	bool SetAdminPassword(const char *Value);
	bool GetInfoEx(NvObjIndStringList *SystemInfo);
	int GetMaxSessions();
	bool GetPermissions(NvObjTable *SystemPermissions);
	std::string GetDefaultMailServer();
	bool SetDefaultMailServer(const char *Value);
	std::string GetDefaultMailUser();
	bool SetDefaultMailUser(const char *Value);
	std::string GetDefaultMailPassword();
	bool SetDefaultMailPassword(const char *Value);
	std::string GetHostWebs();
	bool SetHostWebs(const char *Value);
	bool IsUnicode();
	std::string GetUniqString();
	bool IsMqAvailable();
	bool IsLcAvailable();
	void LoadMimeTable();
	void SaveMimeTable();
	void SetMimeType(std::string Extension, std::string MimeType);
	std::string GetMimeType(std::string Extension);
	void RemoveMimeType(std::string Extension);
	void GetMimeTable(NvObjIndStringList *Result);
	void LoadAliasTable();
	void SaveAliasTable();
	void SetAlias(std::string Path, std::string Alias);
	std::string GetAlias(std::string Path);
	void RemoveAlias(std::string Path);
	void GetAliasTable(NvObjIndStringList *Result);
	void SetStopRequest();
	bool GetStopRequest();
	std::string GetNirvaDefaultPage();
	bool SetNirvaDefaultPage(const char *Value);
	int GetLicenseType();
	bool CheckOverTime();
	int GetNumExceptions();
	void IncNumExceptions();
};


// Used to get a pointer to the unique NvsServer object
NvsServer* GetMain();



#endif //_NVS_H



