// request.cpp : implementation file
//


#include "nvcc.h"


using namespace std;



/////////////////////////////////////////////////////////////////////////////
// NvRequestList

NvRequestList::NvRequestList()
{
}

NvRequestList::~NvRequestList()
{
	Clear();
}



void NvRequestList::Clear()
{
	int i;
	Lock();
	for(i=0; i<(int)m_vRequests.size(); i++)
	{
		NvRequest *Request = (NvRequest *)m_vRequests[i];
		if(Request != NULL)
			delete Request;
	}
	m_vRequests.clear();
	Unlock();
}



NvRequest* NvRequestList::AddRequest()
{
	Lock();
	NvRequest *Request = new NvRequest;
	if(Request != NULL)
		m_vRequests.push_back((void *)Request);
	Unlock();

	// Update connection list each time a new request is added
	GetMain()->Connections.Update();
	return Request;
}


void NvRequestList::RemoveRequest(NvRequest *Request)
{
	if(Request == NULL)
		return;

	Lock();
	int i;
	for(i=0; i<(int)m_vRequests.size(); i++)
	{
		NvRequest *RequestInList = (NvRequest *)m_vRequests[i];
		if(RequestInList == Request)
		{
			m_vRequests.erase(m_vRequests.begin()+i);
			delete Request;
			break;
		}
	}
	Unlock();
}


bool NvRequestList::CheckRequest(NvRequest *Request)
{
	if(Request == NULL)
		return false;

	Lock();
	int i;
	bool RequestFound = false;
	for(i=0; i<(int)m_vRequests.size(); i++)
	{
		NvRequest *RequestInList = (NvRequest *)m_vRequests[i];
		if(RequestInList == Request)
		{
			RequestFound = true;
			break;
		}
	}
	Unlock();
	return RequestFound;
}



/////////////////////////////////////////////////////////////////////////////
// NvRequest

NvRequest::NvRequest()
{
	m_sServer = "127.0.0.1:1081";		// Default nvs server address and port
	m_sUser = "";
	m_sHash = "";
	m_sPassword = "";
	m_sNewPassword = "";
	m_sNewPasswordConfirm = "";
	m_sAppli = "";
	m_sOpen = "";
	m_sClose = "";
	m_sCertificate = "";
	m_sSessionId = "";
	m_bAutoClose = false;
	m_iSessionTimeOut = 0;
	m_iServerConnectionTimeOut = 10;
	CurrentConnection = NULL;
	b_UseSSL = false;
	b_UseSSO = false;
	b_ClientUnicode = false;
	bWithNewPassword = false;
	bWithNewPasswordConfirm = false;
	SSOPrincipal = "";
	m_sSocketLog = "";
	m_sServerVersion = "";
}

NvRequest::~NvRequest()
{
	Clear();
}

void NvRequest::Clear()
{
	Lock();
	if(CurrentConnection != NULL)
		GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
	CurrentConnection = NULL;
	ResetCommandBuffer();
	ClearLocalContainer();
	Unlock();
}

void NvRequest::ResetCommandBuffer()
{
	int i;
	for(i=0; i<(int)m_vCommands.size(); i++)
	{
		NvServerCommand *Command = (NvServerCommand *)m_vCommands[i];
		if(Command != NULL)
			delete Command;
	}
	m_vCommands.clear();
}

void NvRequest::ClearLocalContainer()
{
	m_cContainer.ClearObjects();
}





// Internal function
void NvRequest::GetStringValue(NvServerCommand *Command, const char *String, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// All the local functions that return a value goes here
	if(String == NULL)
	{
		if(DataSize != NULL)
			*DataSize = 0;
		if(Buffer == NULL)
			return;
		if(BufferSize == 0)
		{
			// Buffer contains a variable name (is it really used ?)
			m_oVariables.SetValue(Buffer, "");
			return;
		}
		Buffer[0] = 0;
		return;
	}

	// The return value is always a string
	// Even if the result string has to be retruned into a buffer, we also
	// always set it into a request variable. This variable is NV_RESULT by default
	// but can be changed by the value of the parameter NV_VAR
	string VarName = Command->m_Parameters.GetValue("NV_VAR");
	if(!VarName.length())
		VarName = "NV_RESULT";
	m_oVariables.SetValue(VarName, String);

	if((Buffer == NULL) && (DataSize == NULL))
		return;	// Nothing to do in this case

	if((Buffer != NULL) && (BufferSize == 0))
	{
		// Buffer contains a variable name (is it really used ?)
		string Variable = Buffer;
		m_oVariables.SetValue(Buffer, String);
		return;
	}

	string RetVal = String;
	EncodeCommandResult(iEncoding, RetVal, (void*)this);

	// Keep a copy of last output buffer
	m_sLastOutputBuffer = RetVal;

	if(DataSize != NULL)
		*DataSize = (int)RetVal.length();

	if(Buffer == NULL)
		return;


	// Empty buffer by default;
	Buffer[0] = 0;


	if((int)(RetVal.length() + 1) > BufferSize)
		RetVal.erase(BufferSize-1);

	if(RetVal.length())
		strcpy(Buffer, RetVal.c_str());
}


bool NvRequest::NvCommand(const char *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// Check special command for retreiving the last output buffer
	if(strlen(Command) == 37)
	{
		string CommandStringHere = Command;
		MakeUpper(CommandStringHere);
		if(CommandStringHere == "NV_CMD=|LOCAL:GET_LAST_OUTPUT_BUFFER|")
		{
			GetLastOutputBuffer(Buffer, BufferSize);
			return true;
		}
	}

	Lock();					// Each command to a given request is serialized

	if(DataSize != NULL)
		*DataSize = 0;

	// We first reset the buffer;
	if((Buffer != NULL) && (BufferSize > 0))
		*Buffer = '\0';
	
	if(Command == NULL)
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "COMMAND:No command string");
		Unlock();
		return false;
	}

	NvServerCommand *NewCommand = new NvServerCommand;
	NewCommand->ParseParameters(Command, &m_oVariables);
	
	NewCommand->m_sCommand = NewCommand->m_Parameters.GetValue("NV_CMD");
	if(NewCommand->m_sCommand.length())
	{
		string::size_type index = NewCommand->m_sCommand.rfind(':');
		if(index != string::npos)
		{
			NewCommand->m_sClass = NewCommand->m_sCommand.substr(0, index);
			NewCommand->m_sCommand = NewCommand->m_sCommand.substr(index+1);
			index = NewCommand->m_sClass.rfind(':');
			if(index != string::npos)
			{
				NewCommand->m_sService = NewCommand->m_sClass.substr(0, index);
				NewCommand->m_sClass = NewCommand->m_sClass.substr(index+1);
			}
		}
	}
	else
	{
		NewCommand->m_sService = NewCommand->m_Parameters.GetValue("NV_SERVICE");
		NewCommand->m_sClass = NewCommand->m_Parameters.GetValue("NV_CLASS");
		NewCommand->m_sCommand = NewCommand->m_Parameters.GetValue("NV_COMMAND");
	}
	CleanString(NewCommand->m_sService, true);
	CleanString(NewCommand->m_sClass, true);
	CleanString(NewCommand->m_sCommand, true);
	if((!NewCommand->m_sService.length()) && (!NewCommand->m_sClass.length()) && (!NewCommand->m_sCommand.length()))
	{
		NewCommand->m_sService = "SYSTEM";
		NewCommand->m_sClass = "MISC";
		NewCommand->m_sCommand = "NOP";
	}
	if(!NewCommand->m_sService.length())
		NewCommand->m_sService = "SYSTEM";
	
	
	NewCommand->m_sInContainer = NewCommand->m_Parameters.GetValue("NV_CONTAINER");
	NewCommand->m_sOutContainer = NewCommand->m_sInContainer;
	if(NewCommand->m_Parameters.KeyExist("NV_IN_CONTAINER"))
		NewCommand->m_sInContainer = NewCommand->m_Parameters.GetValue("NV_IN_CONTAINER");
	if(NewCommand->m_Parameters.KeyExist("NV_OUT_CONTAINER"))
		NewCommand->m_sOutContainer = NewCommand->m_Parameters.GetValue("NV_OUT_CONTAINER");
	NewCommand->m_sLanguage = NewCommand->m_Parameters.GetValue("NV_LANGUAGE");;
	CleanString(NewCommand->m_sService);
	CleanString(NewCommand->m_sClass);
	CleanString(NewCommand->m_sCommand);
	CleanString(NewCommand->m_sInContainer);
	CleanString(NewCommand->m_sOutContainer);
	CleanString(NewCommand->m_sLanguage);
	MakeUpper(NewCommand->m_sService);				// Service is always uppercase
	MakeUpper(NewCommand->m_sClass);				// Class is always uppercase
	MakeUpper(NewCommand->m_sCommand);				// Command is always uppercase
	MakeLower(NewCommand->m_sInContainer);			// In container is always lowercase
	MakeLower(NewCommand->m_sOutContainer);			// Out container is always lowercase
	MakeUpper(NewCommand->m_sLanguage);				// language is always uppercase

	// Remove reserved parameters from command
	NewCommand->m_Parameters.Remove("NV_SERVICE");
	NewCommand->m_Parameters.Remove("NV_CLASS");
	NewCommand->m_Parameters.Remove("NV_COMMAND");
	NewCommand->m_Parameters.Remove("NV_IN_CONTAINER");
	NewCommand->m_Parameters.Remove("NV_OUT_CONTAINER");
	NewCommand->m_Parameters.Remove("NV_LANGUAGE");

	string::size_type index;

	// remove blanks in service name
	while((index = NewCommand->m_sService.find(' ')) != string::npos)
		NewCommand->m_sService.erase(index, 1);

	if(!NewCommand->m_sService.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "COMMAND:NV_SERVICE");
		delete NewCommand;
		Unlock();
		return false;
	}
	// remove blanks in class name
	while((index = NewCommand->m_sClass.find(' ')) != string::npos)
		NewCommand->m_sClass.erase(index, 1);

//	if(!NewCommand->m_sCommand.length())
//	{
//		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "COMMAND:NV_COMMAND");
//		delete NewCommand;
//		Unlock();
//		return false;
//	}

	bool RetVal = true;
	if(NewCommand->m_sService == "LOCAL")
		RetVal = ProcessLocalCommand(NewCommand, Buffer, BufferSize, DataSize);
	else
		RetVal = ProcessServerCommand(NewCommand, Buffer, BufferSize, DataSize);
	Unlock();
	return RetVal;
}


// Internal function
void NvRequest::GetLastOutputBuffer(char *Buffer, int BufferSize)
{
	if((Buffer == NULL) || (BufferSize == 0))
		return;	// Nothing to do in this case

	// Empty buffer by default;
	Buffer[0] = 0;

	string RetVal = m_sLastOutputBuffer;

	if((int)(RetVal.length() + 1) > BufferSize)
		RetVal.erase(BufferSize-1);

	if(RetVal.length())
		strcpy(Buffer, RetVal.c_str());
}

// Dispatch here all the local commands
bool NvRequest::ProcessLocalCommand(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	bool RetVal = true;

	// For all commands except REQUEST ERROR_INFO, we must clear the global error object
	if((Command->m_sClass != "REQUEST") || (Command->m_sCommand != "ERROR_INFO"))
		GlobalError.SetError();
	
	if(Command->m_sClass == "REQUEST")
	{
		if(Command->m_sCommand == "RESET_COMMAND_BUFFER")
			ResetCommandBuffer();
		else if(Command->m_sCommand == "SEND_COMMANDS")
			RetVal = SendCommands(Command);
		else if(Command->m_sCommand == "GET_NUM_COMMANDS")
			GetNumCommands(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "REMOVE_COMMAND")
			RetVal = RemoveCommand(Command);
		else if(Command->m_sCommand == "ERROR_INFO")
			GetError(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "GET_SESSION_ID")
			GetSessionId(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "GET_SERVER_VERSION")
			GetServerVersion(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "XML_COMMAND")
			XmlCommand(Command);
		else
		{
			RetVal = false;
			string sInfoMessage = Command->m_sClass;
			sInfoMessage+= ":";
			sInfoMessage+= Command->m_sCommand;
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_UNKNOWN_COMMAND, sInfoMessage);
		}
	}
	else if(Command->m_sClass == "VARIABLE")
	{
		if(Command->m_sCommand == "SET")
			SetVariable(Command);
		else if(Command->m_sCommand == "GET")
			GetVariable(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "REMOVE")
			RemoveVariable(Command);
		else if(Command->m_sCommand == "EXIST")
			VariableExist(Command, Buffer, BufferSize, DataSize);
		else
		{
			RetVal = false;
			string sInfoMessage = Command->m_sClass;
			sInfoMessage+= ":";
			sInfoMessage+= Command->m_sCommand;
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_UNKNOWN_COMMAND, sInfoMessage);
		}
	}
	else if(Command->m_sClass == "OBJECT")
	{
		if(Command->m_sCommand == "CREATE")
			RetVal = CreateObject(Command);
		else if(Command->m_sCommand == "GET_NUM")
			GetNumObjects(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "GET_NAME")
			RetVal = GetObjectName(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "GET_TYPE")
			RetVal = GetObjectType(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "CLEAR_ALL")
			ClearLocalContainer();
		else if(Command->m_sCommand == "COPY")
			RetVal = CopyObject(Command);
		else if(Command->m_sCommand == "SET_NAME")
			RetVal = SetObjectName(Command);
		else if(Command->m_sCommand == "REMOVE")
			RetVal = RemoveObject(Command);
		else if(Command->m_sCommand == "EXIST")
			IsObjectExist(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "BOOLEAN_SET_VALUE")
			RetVal = ObjBooleanSetValue(Command);
		else if(Command->m_sCommand == "BOOLEAN_GET_VALUE")
			RetVal = ObjBooleanGetValue(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "INTEGER_SET_VALUE")
			RetVal = ObjIntegerSetValue(Command);
		else if(Command->m_sCommand == "INTEGER_GET_VALUE")
			RetVal = ObjIntegerGetValue(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "STRING_SET_VALUE")
			RetVal = ObjStringSetValue(Command);
		else if(Command->m_sCommand == "STRING_GET_VALUE")
			RetVal = ObjStringGetValue(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "STRINGLIST_SET_VALUE")
			RetVal = ObjStringListSetValue(Command);
		else if(Command->m_sCommand == "STRINGLIST_GET_VALUE")
			RetVal = ObjStringListGetValue(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "STRINGLIST_GET_SIZE")
			RetVal = ObjStringListGetSize(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "STRINGLIST_REMOVE")
			RetVal = ObjStringListRemove(Command);
		else if(Command->m_sCommand == "STRINGLIST_INSERT")
			RetVal = ObjStringListInsert(Command);
		else if(Command->m_sCommand == "STRINGLIST_ADD_STRINGLIST")
			RetVal = ObjStringListAddStringList(Command);
		else if(Command->m_sCommand == "STRINGLIST_SWAP")
			RetVal = ObjStringListSwap(Command);
		else if(Command->m_sCommand == "STRINGLIST_SORT")
			RetVal = ObjStringListSort(Command);
		else if(Command->m_sCommand == "INDSTRINGLIST_SET_VALUE")
			RetVal = ObjIndStringListSetValue(Command);
		else if(Command->m_sCommand == "INDSTRINGLIST_GET_VALUE")
			RetVal = ObjIndStringListGetValue(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "INDSTRINGLIST_GET_VALUES")
			RetVal = ObjIndStringListGetValues(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "INDSTRINGLIST_GET_SIZE")
			RetVal = ObjIndStringListGetSize(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "INDSTRINGLIST_REMOVE")
			RetVal = ObjIndStringListRemove(Command);
		else if(Command->m_sCommand == "INDSTRINGLIST_ADD_INDSTRINGLIST")
			RetVal = ObjIndStringListAddIndStringList(Command);
		else if(Command->m_sCommand == "INDSTRINGLIST_KEY_EXIST")
			RetVal = ObjIndStringListKeyExist(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "INDSTRINGLIST_GET_KEY")
			RetVal = ObjIndStringListGetKey(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_SET_DESCRIPTION")
			RetVal = ObjTableSetDescription(Command);
		else if(Command->m_sCommand == "TABLE_GET_DESCRIPTION")
			RetVal = ObjTableGetDescription(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_CLEAR")
			RetVal = ObjTableClear(Command);
		else if(Command->m_sCommand == "TABLE_CLEAR_DATA")
			RetVal = ObjTableClearData(Command);
		else if(Command->m_sCommand == "TABLE_SEARCH")
			RetVal = ObjTableSearch(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_SORT")
			RetVal = ObjTableSort(Command);
		else if(Command->m_sCommand == "TABLE_CREATE_FROM")
			RetVal = ObjTableCreateFrom(Command);
		else if(Command->m_sCommand == "TABLE_SAVE")
			RetVal = ObjTableSave(Command);
		else if(Command->m_sCommand == "TABLE_LOAD")
			RetVal = ObjTableLoad(Command);
		else if(Command->m_sCommand == "TABLE_IMPORT")
			RetVal = ObjTableImport(Command);
		else if(Command->m_sCommand == "TABLE_EXPORT")
			RetVal = ObjTableExport(Command);
		else if(Command->m_sCommand == "TABLE_SELECT_FROM")
			RetVal = ObjTableSelectFrom(Command);
		else if(Command->m_sCommand == "TABLE_JOIN")
			RetVal = ObjTableJoin(Command);
		else if(Command->m_sCommand == "TABLE_ADD_TABLE")
			RetVal = ObjTableAddTable(Command);
		else if(Command->m_sCommand == "TABLE_CLEAR_CELL")
			RetVal = ObjTableClearCell(Command);
		else if(Command->m_sCommand == "TABLE_SORT_CELL")
			RetVal = ObjTableSortCell(Command);
		else if(Command->m_sCommand == "TABLE_GET_CELL_NUM_LINES")
			RetVal = ObjTableGetCellNumLines(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_GET_CELL_LINE")
			RetVal = ObjTableGetCellLine(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_SET_CELL_LINE")
			RetVal = ObjTableSetCellLine(Command);
		else if(Command->m_sCommand == "TABLE_GET_CELL")
			RetVal = ObjTableGetCell(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_SET_CELL")
			RetVal = ObjTableSetCell(Command);
		else if(Command->m_sCommand == "TABLE_INSERT_CELL_LINE")
			RetVal = ObjTableInsertCellLine(Command);
		else if(Command->m_sCommand == "TABLE_REMOVE_CELL_LINE")
			RetVal = ObjTableRemoveCellLine(Command);
		else if(Command->m_sCommand == "TABLE_SWAP_CELL_LINES")
			RetVal = ObjTableSwapCellLines(Command);
		else if(Command->m_sCommand == "TABLE_GET_NUM_ROWS")
			RetVal = ObjTableGetNumRows(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_REMOVE_ROW")
			RetVal = ObjTableRemoveRow(Command);
		else if(Command->m_sCommand == "TABLE_REMOVE_ROWS")
			RetVal = ObjTableRemoveRows(Command);
		else if(Command->m_sCommand == "TABLE_CLEAR_ROW")
			RetVal = ObjTableClearRow(Command);
		else if(Command->m_sCommand == "TABLE_CLEAR_ROWS")
			RetVal = ObjTableClearRows(Command);
		else if(Command->m_sCommand == "TABLE_INSERT_ROWS")
			RetVal = ObjTableInsertRows(Command);
		else if(Command->m_sCommand == "TABLE_SWAP_ROWS")
			RetVal = ObjTableSwapRows(Command);
		else if(Command->m_sCommand == "TABLE_GET_ROW")
			RetVal = ObjTableGetRow(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_GET_COLUMN")
			RetVal = ObjTableGetColumn(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_GET_ROWS")
			RetVal = ObjTableGetRows(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_GET_NUM_COLUMNS")
			RetVal = ObjTableGetNumColums(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_GET_COLUMN_NAME")
			RetVal = ObjTableGetColumnName(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_GET_COLUMN_DESCRIPTION")
			RetVal = ObjTableGetColumnDescription(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_IS_COLUMN_NUMERIC")
			RetVal = ObjTableIsColumnNumeric(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_GET_COLUMN_INDEX")
			RetVal = ObjTableGetColumnIndex(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_CLEAR_COLUMN")
			RetVal = ObjTableClearColumn(Command);
		else if(Command->m_sCommand == "TABLE_REMOVE_COLUMN")
			RetVal = ObjTableRemoveColumn(Command);
		else if(Command->m_sCommand == "TABLE_INSERT_COLUMN")
			RetVal = ObjTableInsertColumn(Command);
		else if(Command->m_sCommand == "TABLE_ADD_COLUMNS")
			RetVal = ObjTableAddColumns(Command);
		else if(Command->m_sCommand == "TABLE_MODIFY_COLUMN")
			RetVal = ObjTableModifyColumn(Command);
		else if(Command->m_sCommand == "TABLE_SWAP_COLUMNS")
			RetVal = ObjTableSwapColumns(Command);
		else if(Command->m_sCommand == "TABLE_FILTER_COLUMNS")
			RetVal = ObjTableFilterColumns(Command);
		else if(Command->m_sCommand == "TABLE_SET_PRIMARY_COLUMN")
			RetVal = ObjTableSetPrimaryColumn(Command);
		else if(Command->m_sCommand == "TABLE_GET_PRIMARY_INFO")
			RetVal = ObjTableGetPrimaryInfo(Command);
		else if(Command->m_sCommand == "TABLE_GET_ROW_INDEX")
			RetVal = ObjTableGetRowIndex(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "TABLE_SET_ROW")
			RetVal = ObjTableSetRow(Command);
		else if(Command->m_sCommand == "FILE_SET_FILENAME")
			RetVal = ObjFileSetFileName(Command);
		else if(Command->m_sCommand == "FILE_SET_PERSIST")
			RetVal = ObjFileSetPersist(Command);
		else if(Command->m_sCommand == "FILE_GET_PATHNAME")
			RetVal = ObjFileGetPathName(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "FILE_GET_DIRNAME")
			RetVal = ObjFileGetDirName(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "FILE_GET_FILENAME")
			RetVal = ObjFileGetFileName(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "FILE_GET_EXTENSION")
			RetVal = ObjFileGetExtension(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "FILE_GET_SIZE")
			RetVal = ObjFileGetSize(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "FILE_REMOVE")
			RetVal = ObjFileRemove(Command);
		else if(Command->m_sCommand == "FILE_CREATE")
			RetVal = ObjFileCreate(Command);
		else if(Command->m_sCommand == "FILE_CLEAR")
			RetVal = ObjFileClear(Command);
		else if(Command->m_sCommand == "FILE_APPEND")
			RetVal = ObjFileAppend(Command);
		else if(Command->m_sCommand == "FILE_EXIST")
			RetVal = ObjFileExist(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "FILE_SET_ORIGIN")
			RetVal = ObjFileSetOrigin(Command);
		else if(Command->m_sCommand == "FILE_GET_ORIGIN")
			RetVal = ObjFileGetOrigin(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "FILE_SET_ASCII")
			RetVal = ObjFileSetAscii(Command);
		else if(Command->m_sCommand == "FILE_COMPRESS")
			RetVal = ObjFileCompress(Command);
		else if(Command->m_sCommand == "FILE_DECOMPRESS")
			RetVal = ObjFileDecompress(Command);
		else if(Command->m_sCommand == "BINARY_GET_SIZE")
			RetVal = ObjBinaryGetSize(Command, Buffer, BufferSize, DataSize);
		else if(Command->m_sCommand == "BINARY_CLEAR")
			RetVal = ObjBinaryClear(Command);
		else if(Command->m_sCommand == "BINARY_APPEND")
			RetVal = ObjBinaryAppend(Command);
		else if(Command->m_sCommand == "BINARY_SAVE")
			RetVal = ObjBinarySave(Command);
		else if(Command->m_sCommand == "BINARY_LOAD")
			RetVal = ObjBinaryLoad(Command);
		else
		{
			RetVal = false;
			string sInfoMessage = Command->m_sClass;
			sInfoMessage+= ":";
			sInfoMessage+= Command->m_sCommand;
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_UNKNOWN_COMMAND, sInfoMessage);
		}
	}
	else
	{
		RetVal = false;
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_UNKNOWN_CLASS, Command->m_sClass);
	}

	delete Command;

	return RetVal;
}


// Dispatch here all the server commands
// If the COMMAND_INDEX parameter exists in the command then
// the command is inserted into the command buffer before the given index
// or at the end of the buffer if the index is 0.
// Otherwise, the command is immediatly executed except if the command buffer
// contains some un-sent commands. When the command is executed, the command
// buffer is reset.
bool NvRequest::ProcessServerCommand(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	GlobalError.SetLocalError();

	// Check if we must queue the command or not
	bool ToQueue = false;
	if(Command->m_Parameters.KeyExist("NV_COMMAND_INDEX"))
		ToQueue = true;


	Command->m_Error.SetLocalError("REQUEST", NV_LOCAL_COMMAND_NOT_SENT, "");
	
	if(ToQueue)
	{
		// If the COMMAND_INDEX parameter is set to 0,
		// we queue at the end of the list
		// Otherwise, we insert the command before the given index
		int iCommandIndex = 0;
		string CommandIndex = Command->m_Parameters.GetValue("NV_COMMAND_INDEX");
		CleanString(CommandIndex);
		if(CommandIndex.length())
			iCommandIndex = atoi(CommandIndex.c_str());
		
		if(iCommandIndex > (int)m_vCommands.size())
			iCommandIndex = 0;

		if((iCommandIndex == 0) || (!m_vCommands.size()))
		{
			// In these cases we just add the command at the end of the buffer
			m_vCommands.push_back((void *)Command);
			return true;
		}

		if(iCommandIndex < 1)
			iCommandIndex = 1;

		// The real index starts at 0 and not 1
		iCommandIndex--;

		// We just insert the command
		m_vCommands.insert(m_vCommands.begin() + iCommandIndex, (void *)Command);	// This is an insert
		return true;
	}
	
	// No queuing
	// Now we check that the command buffer doesn't contain un-sent commands
	int i;
	bool Found = false;
	for(i=0; i<(int)m_vCommands.size(); i++)
	{
		NvServerCommand *CommandInList = (NvServerCommand *)(m_vCommands[i]);
		if(CommandInList->m_Error.m_iErrorCode == NV_LOCAL_COMMAND_NOT_SENT)
		{
			Found = true;
			break;
		}
	}
	if(Found)
	{
		// We have found some commands not sent in the command buffer
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_COMMAND_BUFFER_NOT_EMPTY);
		return false;
	}

	// We remove all the commands from the buffer
	ResetCommandBuffer();

	// We add the new one
	m_vCommands.push_back((void *)Command);


	// And then send the command to the server
	bool RetVal = true;
	RetVal = SendServerCommands(false, Buffer, BufferSize, DataSize, iEncoding);

	// With a single command, an error in the command replaces the global error
	if(!RetVal && (GlobalError.m_sErrorClass == "REQUEST") && (GlobalError.m_iErrorCode == NV_LOCAL_SERVER_COMMAND_ERROR))
		GlobalError.SetError(Command->m_Error.m_sErrorService, Command->m_Error.m_sErrorClass, Command->m_Error.m_iErrorCode, Command->m_Error.m_sErrorInfo, Command->m_Error.m_sErrorDesc);


	// We remove the command from the buffer
	ResetCommandBuffer();

	return RetVal;
}



bool NvRequest::SendServerCommands(bool StopOnError, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	GlobalError.SetLocalError();

	string Address = m_sServer;
	CleanString(Address);

	string Proxy = "";
	string ProxyUser = "";
	string ProxyPassword = "";

	// The proxy is eventually in the address
	// at this time the address starts with proxy::proxyaddress::...
	// proxyaddress can have some optional user and password information in brackets separated by a ; character
	string AddressHere = Address;
	MakeUpper(AddressHere);
	string::size_type index = AddressHere.find("PROXY::");
	if(index != string::npos)
	{
		Address = Address.substr(7);
		index = Address.find("::");
		if(index != string::npos)
		{
			Proxy = Address.substr(0, index);
			Address = Address.substr(index+2);
		}
		index = Proxy.find('(');
		if(index != string::npos)
		{
			ProxyUser = Proxy.substr(index+1);
			Proxy = Proxy.substr(0, index);
		}
		index = ProxyUser.find(')');
		if(index != string::npos)
			ProxyUser = ProxyUser.substr(0, index);
		
		index = ProxyUser.find(';');
		if(index != string::npos)
		{
			ProxyPassword = ProxyUser.substr(index+1);
			ProxyUser = ProxyUser.substr(0, index);
		}
	}
	else
	{
		// Compatibiliy IPV6
		index = AddressHere.find("PROXY%%");
		if(index != string::npos)
		{
			Address = Address.substr(7);
			index = Address.find("%%");
			if(index != string::npos)
			{
				Proxy = Address.substr(0, index);
				Address = Address.substr(index+2);
			}
			index = Proxy.find('(');
			if(index != string::npos)
			{
				ProxyUser = Proxy.substr(index+1);
				Proxy = Proxy.substr(0, index);
			}
			index = ProxyUser.find(')');
			if(index != string::npos)
				ProxyUser = ProxyUser.substr(0, index);
			
			index = ProxyUser.find(';');
			if(index != string::npos)
			{
				ProxyPassword = ProxyUser.substr(index+1);
				ProxyUser = ProxyUser.substr(0, index);
			}
		}
	}
	CleanString(Proxy);
	CleanString(ProxyUser);
	CleanString(ProxyPassword);
	CleanString(Address);
	

	int Port = 1081;
	bool UseSsl = b_UseSSL;
	if(UseSsl)
		Port = 1082;
	SplitAddress(Address, Port);
/*	index = Address.find(':');
	if(index != string::npos)
	{
		string Provistr = Address.substr(index+1);
		CleanString(Provistr);
		Port = atoi(Provistr.c_str());
		Address.erase(index);
		CleanString(Provistr);
	}
*/
	if(!Address.length())
		Address = "127.0.0.1";

	string ConnectedServer = m_sServer; 
	string FinalAddress = Address;
	int FinalPort = Port;
	bool FinalUseSsl = UseSsl;
	bool bUseProxy = false;
	if(Proxy.length())
	{
		string ProxyHere = Proxy;
		MakeUpper(ProxyHere);
		CleanString(ProxyHere);
		index = ProxyHere.find("HTTP://");
		if(index != string::npos)
		{
			Proxy = Proxy.substr(index+7);
			bUseProxy = true;
			UseSsl = false;
		}
		if(!bUseProxy)
		{
			index = ProxyHere.find("HTTPS://");
			if(index != string::npos)
			{
				Proxy = Proxy.substr(index+8);
				bUseProxy = true;
				UseSsl = true;
			}
		}
		if(bUseProxy)
		{
			Address = Proxy;
			Port = 4480;
			SplitAddress(Address, Port);
/*			index = Address.find(':');
			if(index != string::npos)
			{
				string Provistr = Address.substr(index+1);
				CleanString(Provistr);
				Port = atoi(Provistr.c_str());
				Address.erase(index);
			}*/
		}
	}

	int ProxyMode = NO_PROXY;
	if(bUseProxy)
	{
		if(FinalUseSsl)
			ProxyMode = PROXY_FINAL_SSL;
		else
			ProxyMode = PROXY_FINAL_NORMAL;
	}

	NvHttpConnection* Connection = (NvHttpConnection*)CurrentConnection;
	if(Connection == NULL)
	{
		Connection = GetMain()->Connections.GetConnection(Address.c_str(), Port, m_iServerConnectionTimeOut, UseSsl, ProxyMode, FinalAddress.c_str(), FinalPort, m_sCertificate.c_str());
		CurrentConnection = Connection;
	}
	if(Connection == NULL)
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_CANNOT_GET_SERVER_CONNECTION, Address);
		return false;
	}
	// We got a connected connection
	Connection->sSocket.LogFile = m_sSocketLog;
	Connection->ProxyAuthInfo.m_sUser = ProxyUser;
	Connection->ProxyAuthInfo.m_sPassword = ProxyPassword;
	Connection->AuthInfo.SsoInfo.m_sPrincipal = SSOPrincipal;
	
	// PM Modif for proxy HTTPS
	if(bUseProxy && FinalUseSsl)
		Connection->b_ProxyHttps = true;
	else
		Connection->b_ProxyHttps = false;

	bool RetVal = false;
	
	// we clear the input message
	Connection->InMessage.Clear();
	
	// we clear the outpput message
	Connection->OutMessage.Clear();

	NvData *Data = new NvData;

	// For HTTP 1.1 we must include the host header
	char buffer[20];
	sprintf(buffer, "%d", FinalPort);
	FinalAddress += ":";
	FinalAddress += buffer;
	
	Data->sInfo = "POST ";
	// PM Modif for proxy HTTPS
	//if(bUseProxy)
	if(bUseProxy && (!FinalUseSsl))
	{
		if(FinalUseSsl)
			Data->sInfo += "https://";
		else
			Data->sInfo += "http://";
		Data->sInfo += FinalAddress;
	}
	Data->sInfo += "/NVS?Nirva";
	if(m_sSessionId.length())
	{
		Data->sInfo += "&NV_SESSION_ID=";
		Data->sInfo += m_sSessionId;
	}

	Data->sInfo += " HTTP/1.1";
	
	Connection->OutMessage.vDatas.push_back(Data);
	
	// by default, we have no content in the body
	Connection->OutMessage.AddHeader("Content-Length", "0");
	
	Connection->OutMessage.AddHeader("Host", FinalAddress);
	//Connection->OutMessage.AddHeader("Host", "Nirva");

	Connection->OutMessage.AddHeader("Nv_Source", HTTP_CLIENT_STRING);

	// We prepare the output global headers
	
	// Set content-type
	string sContentType = "multipart/form-data; boundary=";
	sContentType += HTTP_SEND_BOUNDARY;
	Connection->OutMessage.AddHeader("Content-Type", sContentType);

	// Set Unicode
	if(b_ClientUnicode)
		Connection->OutMessage.AddHeader("Nv_Client_Unicode", "YES");

	// Set NvMachineName
	Connection->OutMessage.AddHeader("Nv_Machine_Name", GetMain()->MachineName);

	// Set NvMachineUser
	Connection->OutMessage.AddHeader("Nv_Machine_User", GetMain()->MachineUser);

	// Set NvSessionId
	Connection->OutMessage.AddHeader("Nv_Session_Id", m_sSessionId);


	if(m_sSessionId.length() && (m_sSessionId[0] != '.'))
	{
		// Send	control hash
	
		// Set NvUser
		Connection->OutMessage.AddHeader("Nv_Control", m_sHash);

	}
	else
	{
		// Send connection parameters
	
		// Set NvUser
		Connection->OutMessage.AddHeader("Nv_User", m_sUser);

		// Set NvPassword
		string PasswordHere = m_sPassword;
		if(PasswordHere.length() && (PasswordHere.length() < 256))
		{
			// The string must be encrypted
			string EncryptionKey = GetMain()->MachineName;
			EncryptionKey += "nvc";
			EncryptionKey += m_sAppli;
			MakeLower(EncryptionKey);

			int BufferLength = (int)(100 + (4*PasswordHere.length()));
			char *Buffer = (char *)malloc(BufferLength);
			strcpy(Buffer, PasswordHere.c_str());
			char *Buffer2 = (char *)malloc(4*EncryptionKey.length());
			strcpy(Buffer2, EncryptionKey.c_str());
			EncryptString(Buffer2, Buffer, BufferLength);
			PasswordHere = Buffer;
			free(Buffer);
			free(Buffer2);
		}
		Connection->OutMessage.AddHeader("Nv_Password", PasswordHere);

		if(bWithNewPassword)
		{
			// Set NvNewPassword
			string NewPasswordHere = m_sNewPassword;
			if(NewPasswordHere.length() && (NewPasswordHere.length() < 256))
			{
				// The string must be encrypted
				string EncryptionKey = GetMain()->MachineName;
				EncryptionKey += "nvc";
				EncryptionKey += m_sAppli;
				MakeLower(EncryptionKey);

				int BufferLength = (int)(100 + (4*NewPasswordHere.length()));
				char *Buffer = (char *)malloc(BufferLength);
				strcpy(Buffer, NewPasswordHere.c_str());
				char *Buffer2 = (char *)malloc(4*EncryptionKey.length());
				strcpy(Buffer2, EncryptionKey.c_str());
				EncryptString(Buffer2, Buffer, BufferLength);
				NewPasswordHere = Buffer;
				free(Buffer);
				free(Buffer2);
			}
			Connection->OutMessage.AddHeader("Nv_New_Password", NewPasswordHere);
			
			if(bWithNewPasswordConfirm)
			{
				// Set NvNewPasswordConfirm
				string NewPasswordConfirmHere = m_sNewPasswordConfirm;
				if(NewPasswordConfirmHere.length() && (NewPasswordConfirmHere.length() < 256))
				{
					// The string must be encrypted
					string EncryptionKey = GetMain()->MachineName;
					EncryptionKey += "nvc";
					EncryptionKey += m_sAppli;
					MakeLower(EncryptionKey);

					int BufferLength = (int)(100 + (4*NewPasswordConfirmHere.length()));
					char *Buffer = (char *)malloc(BufferLength);
					strcpy(Buffer, NewPasswordConfirmHere.c_str());
					char *Buffer2 = (char *)malloc(4*EncryptionKey.length());
					strcpy(Buffer2, EncryptionKey.c_str());
					EncryptString(Buffer2, Buffer, BufferLength);
					NewPasswordConfirmHere = Buffer;
					free(Buffer);
					free(Buffer2);
				}
				Connection->OutMessage.AddHeader("Nv_New_Password_Confirm", NewPasswordConfirmHere);
			}
		}

		// Set NvApplication
		Connection->OutMessage.AddHeader("Nv_Application", m_sAppli);

		// Set NvTimeOut
		sprintf(buffer, "%d", m_iSessionTimeOut);
		Connection->OutMessage.AddHeader("Nv_TimeOut", buffer);

		// Set NvSessionOpen procedure
		if(m_sOpen.length())
			Connection->OutMessage.AddHeader("Nv_Session_Open", m_sOpen);

		// Set NvSessionClose procedure
		if(m_sClose.length())
			Connection->OutMessage.AddHeader("Nv_Session_Close", m_sClose);

		// Set SSO
		if(b_UseSSO)
			Connection->OutMessage.AddHeader("Nv_Sso", "yes");
	}

	// Set NvStopOnError
	if(StopOnError)
		Connection->OutMessage.AddHeader("Nv_Stop_On_Error", "yes");

	
	// Set the auto close session flag
	if(m_bAutoClose)
		Connection->OutMessage.AddHeader("SESSION", "close");
	
		
	// Write now each command
	string sBoundary = "\r\n--";
	sBoundary += HTTP_SEND_BOUNDARY;

	int i= 0;
	for(i=0; i<(int)m_vCommands.size(); i++)
	{
		NvServerCommand *Command = (NvServerCommand *)(m_vCommands[i]);
		
		// Clears the command error object
		Command->m_Error.SetLocalError("REQUEST", NV_LOCAL_UNKNOWN_RESULT, "");
		NvData *Data = new NvData;
		Connection->OutMessage.vDatas.push_back(Data);
		Connection->OutMessage.vDatas[i+1]->sInfo = sBoundary;
		Connection->OutMessage.AddHeader("Nv_Service", Command->m_sService, i+1);
		Connection->OutMessage.AddHeader("Nv_Class", Command->m_sClass, i+1);
		Connection->OutMessage.AddHeader("Nv_Command", Command->m_sCommand, i+1);
		Connection->OutMessage.AddHeader("Nv_In_Container", Command->m_sInContainer, i+1);
		Connection->OutMessage.AddHeader("Nv_Out_Container", Command->m_sOutContainer, i+1);
		Connection->OutMessage.AddHeader("Nv_Language", Command->m_sLanguage, i+1);
		Connection->OutMessage.AddHeader("Content-Type", "nirva/command", i+1);

		map<string,string>::iterator pos;
		for(pos = Command->m_Parameters.m_mList.begin(); pos != Command->m_Parameters.m_mList.end(); ++pos)
			Connection->OutMessage.AddHeader(pos->first, pos->second, i+1, true);

	
		// PM TODO here
		// We must set the data for the SendObject command of the system service
	
		string ServiceHere = Command->m_sService;
		MakeUpper(ServiceHere);			// The service is set to uppercase by the server
		CleanString(ServiceHere);
	
		string ClassHere = Command->m_sClass;
		MakeUpper(ClassHere);			// The class is set to uppercase by the server
		CleanString(ClassHere);

		string CommandHere = Command->m_sCommand;
		MakeUpper(CommandHere);
		CleanString(CommandHere);
		
		if((ServiceHere == "SYSTEM") && (ClassHere == "OBJECT") && (CommandHere == "SEND"))
		{

			// Get server object name that should be in parameters
			string ServerObjName = Command->m_Parameters.GetValue("NAME");
			CleanString(ServerObjName);
			if(!ServerObjName.length())
				ServerObjName = NV_OBJ_DEFAULT_NAME;

			// Get local object name that may be in parameters
			// If this parameter is not here or blank, the local object name is
			// the same than the server object name
			string LocalObjName = Command->m_Parameters.GetValue("LNAME");
			CleanString(LocalObjName);
			if(!LocalObjName.length())
				LocalObjName = ServerObjName;


			NvObj* Object = m_cContainer.GetObject(LocalObjName.c_str());
			if(Object == NULL)
			{
				GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_GET, LocalObjName);
				Connection->OutMessage.Clear();
				if(m_bAutoClose)
				{
					if(CurrentConnection != NULL)
						GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
					CurrentConnection = NULL;
				}
//				GetMain()->Connections.FreeConnection(Connection);
				return false;
			}
			int iObjectType = Object->GetType();
			if(iObjectType == NV_OBJTYPE_FILE)
			{
				NvInt64 FileSize = ((NvObjFile *)Object)->GetSize();
				bool bFileMessage = false;
				if(FileSize > (NvInt64)m_iMaxMessMemSize)	// for files grater than 1024 kbytes
					bFileMessage = true;
				
				if(bFileMessage)
				{
					string ObjectFileName = ((NvObjFile *)Object)->GetFilePathName();
					Connection->OutMessage.vDatas[i+1]->DataFile = ObjectFileName;
					Connection->OutMessage.vDatas[i+1]->bDeleteDataFile = false;	// Don't remove the file because we directly use the object file
				}
				else
				{
					// Not a too big file so we use memory data
					if(!((NvObjFile *)Object)->GetData(&(Connection->OutMessage.vDatas[i+1]->Data), &(Connection->OutMessage.vDatas[i+1]->DataSize)))
					{
						GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_GET_DATA, LocalObjName);
						Connection->OutMessage.Clear();
						if(m_bAutoClose)
						{
							if(CurrentConnection != NULL)
								GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
							CurrentConnection = NULL;
						}
		//				GetMain()->Connections.FreeConnection(Connection);
						return false;
					}
				}

				// For file object we must send the file origin in a dedicated http header
				Connection->OutMessage.AddHeader("Nv_File_Origin", ((NvObjFile *)Object)->GetOrigin(), i+1);

			}
			else
			{
				if(!(m_cContainer.GetObjectData(LocalObjName.c_str(), &(Connection->OutMessage.vDatas[i+1]->Data), &(Connection->OutMessage.vDatas[i+1]->DataSize))))
				{
					GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_GET_DATA, LocalObjName);
					Connection->OutMessage.Clear();
					if(m_bAutoClose)
					{
						if(CurrentConnection != NULL)
							GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
						CurrentConnection = NULL;
					}
	//				GetMain()->Connections.FreeConnection(Connection);
					return false;
				}
			}

			Connection->OutMessage.AddHeader("Content-Length", "0", i+1);
			NvInt64 bodysize = (NvInt64)(Connection->OutMessage.vDatas[i+1]->DataSize);
			if(Connection->OutMessage.vDatas[i+1]->DataFile.length())
				bodysize = GetFileLength(Connection->OutMessage.vDatas[i+1]->DataFile.c_str());
			if(bodysize)
			{
				char ProviBuf[50];
				sprintf(ProviBuf, "%llu", bodysize);
				Connection->OutMessage.AddHeader("Content-Length", ProviBuf, i+1);
			}
			
			// The object name and type should be send in parameters
			Connection->OutMessage.AddHeader("NAME", ServerObjName, i+1);
			char Buffer[50];
			sprintf(Buffer, "%d", iObjectType);
			Connection->OutMessage.AddHeader("TYPE", Buffer, i+1);
			

			// PM TODO
			// We can call an Encode function here that operates on Command->Data
			// This Encode function can also compress the data stream
		}
	}

	// Write the end of the message
	Data = new NvData;
	Connection->OutMessage.vDatas.push_back(Data);
	sBoundary += "--";		// last boundary
	Connection->OutMessage.vDatas[i+1]->sInfo = sBoundary;

	// Set the real body size
	NvInt64 BodySize = Connection->OutMessage.GetWriteBodySize();
	sprintf(buffer, "%llu", BodySize);
	Connection->OutMessage.AddHeader("Content-Length", buffer);

	// Now send the order
	RetVal = true;
	string ErrorInfo = "";
	int OrderResult = Connection->SendOrder(true, m_iMaxMessMemSize, m_sTempDir.c_str(), ErrorInfo);

	// we clear the output message
	Connection->OutMessage.Clear();

	if(OrderResult != 1)
	{
		// An error occured
		// we clear the input message
		Connection->InMessage.Clear();

		if(OrderResult == -2)
		{
			string Info = Address;
			Info += " - ";
			Info += ErrorInfo;
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_HTTP_PROXY_AUTHENTICATION_ERROR, Info);
		}
		else if(OrderResult == -1)
		{
			string Info = ConnectedServer;
			Info += " - ";
			Info += ErrorInfo;
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_HTTP_AUTHENTICATION_ERROR, Info);
		}
		else
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_NETWORK_ERROR, ConnectedServer);
		if(m_bAutoClose)
		{
			if(CurrentConnection != NULL)
				GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
			CurrentConnection = NULL;
		}
		return false;
	}
	
	// Get the Connection->OutMessage here

//PMARC

	// The result should have the same number of parts than the number of commands
	// + 1 for the global result + 1 for the first part 
	if((m_vCommands.size() + 2) != Connection->InMessage.vDatas.size())
	{
		// An error occured
		// we clear the input message
		Connection->InMessage.Clear();

		GlobalError.SetLocalError("REQUEST", NV_LOCAL_NETWORK_ERROR, ConnectedServer);
		if(m_bAutoClose)
		{
			if(CurrentConnection != NULL)
				GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
			CurrentConnection = NULL;
		}
//		GetMain()->Connections.FreeConnection(Connection);
		return false;
	}

	if((Connection->InMessage.vDatas.size()) < 2)
	{
		// This is not a Nirva server
		// we clear the input message
		Connection->InMessage.Clear();

		GlobalError.SetLocalError("REQUEST", NV_LOCAL_NETWORK_ERROR, ConnectedServer);
		if(m_bAutoClose)
		{
			if(CurrentConnection != NULL)
				GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
			CurrentConnection = NULL;
		}
//		GetMain()->Connections.FreeConnection(Connection);
		return false;
	}

	if(!Connection->InMessage.HeaderExist("NV_SESSION_ID", 1))
	{
		// This is not a Nirva server
		// we clear the input message
		Connection->InMessage.Clear();

		GlobalError.SetLocalError("REQUEST", NV_LOCAL_NETWORK_ERROR, ConnectedServer);
		if(m_bAutoClose)
		{
			if(CurrentConnection != NULL)
				GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
			CurrentConnection = NULL;
		}
//		GetMain()->Connections.FreeConnection(Connection);
		return false;
	}

	m_sSessionId = Connection->InMessage.GetHeader("NV_SESSION_ID", 1);
	m_sServerVersion = "";
	if(Connection->InMessage.HeaderExist("NV_SERVER_VERSION", 1))
		m_sServerVersion = Connection->InMessage.GetHeader("NV_SERVER_VERSION", 1);
	CleanString(m_sServerVersion, true);

	if(Connection->InMessage.GetHeader("NVERROR", 1) == "yes")
		RetVal = false;

	bool bGlobalError = false;
	string sGlobalErrorCode = Connection->InMessage.GetHeader("NV_ERROR_CODE", 1);
	int iGlobalErrorCode = atoi(sGlobalErrorCode.c_str());
	if(iGlobalErrorCode != NV_OK)
		bGlobalError = true;
	string sGlobalErrorService = Connection->InMessage.GetHeader("NV_ERROR_SERVICE", 1);
	string sGlobalErrorClass = Connection->InMessage.GetHeader("NV_ERROR_CLASS", 1);
	string sGlobalErrorDesc = Connection->InMessage.GetHeader("NV_ERROR_DESC", 1);
	string sGlobalErrorInfo = Connection->InMessage.GetHeader("NV_ERROR_INFO", 1);
	GlobalError.SetError(sGlobalErrorService, sGlobalErrorClass, iGlobalErrorCode, sGlobalErrorInfo, sGlobalErrorDesc);

	for(i=0; i<(int)m_vCommands.size(); i++)
	{
		NvServerCommand *Command = (NvServerCommand *)(m_vCommands[i]);
		string ServiceHere = Command->m_sService;
		MakeUpper(ServiceHere);			// The service is set to uppercase by the server
		CleanString(ServiceHere);
		string CommandHere = Command->m_sCommand;
		MakeUpper(CommandHere);
		CleanString(CommandHere);

		string ClassHere = Command->m_sClass;
		MakeUpper(ClassHere);
		CleanString(ClassHere);
		string sErrorCodeHere = Connection->InMessage.GetHeader("NV_ERROR_CODE", i+2);
		int iErrorCodeHere = atoi(sErrorCodeHere.c_str());
		if(ServiceHere != Connection->InMessage.GetHeader("NV_SERVICE", i+2))
		{
			Command->m_Error.SetLocalError("REQUEST", NV_LOCAL_SYNC_ERROR, Command->m_sService);
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
			RetVal = false;
		}
		else
		{
			string sErrorServiceHere = Connection->InMessage.GetHeader("NV_ERROR_SERVICE", i+2);
			string sErrorClassHere = Connection->InMessage.GetHeader("NV_ERROR_CLASS", i+2);
			string sErrorDescHere = Connection->InMessage.GetHeader("NV_ERROR_DESC", i+2);
			string sErrorInfoHere = Connection->InMessage.GetHeader("NV_ERROR_INFO", i+2);
			Command->m_Error.SetError(sErrorServiceHere, sErrorClassHere, iErrorCodeHere, sErrorInfoHere, sErrorDescHere);
			if(iErrorCodeHere != NV_OK)
			{
				GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
				RetVal = false;
			}
			if(bGlobalError)
			{
				// if we have a global error, we copy it to the command error (it has priority)
				Command->m_Error.SetError(sGlobalErrorService, sGlobalErrorClass, iGlobalErrorCode, sGlobalErrorInfo, sGlobalErrorDesc);
			}

			// We must set the data for the GetObject command of the system service
			if((iErrorCodeHere == NV_OK) && (ServiceHere == "SYSTEM") && (ClassHere == "OBJECT") && (CommandHere == "GET"))
			{

				// Get server object name and type that should be in returned parameters
				string ServerObjName = Connection->InMessage.GetHeader("NAME", i+2);
				string ObjType = Connection->InMessage.GetHeader("TYPE", i+2);
				CleanString(ServerObjName);
				CleanString(ObjType);
				if(!ServerObjName.length())
					ServerObjName = NV_OBJ_DEFAULT_NAME;

				if(!ObjType.length())
				{
					Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_NO_TYPE, ServerObjName);
					GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
					RetVal = false;
					continue;
				}

				int iObjectType = atoi(ObjType.c_str());
				if((iObjectType < 0) || (iObjectType > NV_OBJTYPE_MAXTYPEVALUE))
					iObjectType = NV_OBJTYPE_UNKNOWN;

				if(iObjectType == NV_OBJTYPE_UNKNOWN)
				{
					Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_UNKNOWN_TYPE, ServerObjName);
					GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
					RetVal = false;
					continue;
				}

				// Get local object name that may be in parameters
				// If this parameter is not here or blank, the local object name is
				// the same than the server object name
				string LocalObjName = Command->m_Parameters.GetValue("LNAME");
				CleanString(LocalObjName);
				if(!LocalObjName.length())
					LocalObjName = ServerObjName;


				if(iObjectType == NV_OBJTYPE_FILE)
				{
					// For a file object we must have the local file name in parameters
					// or have the object ever existing before the command
					string LocalFileInCommand = Command->m_Parameters.GetValue("FILENAME");
					NvObjFile *Object = (NvObjFile *)(m_cContainer.GetObject(LocalObjName.c_str()));
					if((Object != NULL) && (Object->GetType() != NV_OBJTYPE_FILE))
					{
						// The object exists but is not a file object
						// so we remove it
						m_cContainer.RemoveObject(LocalObjName.c_str());
						Object = NULL;
					}

					bool ObjectCreated = false;
					if(Object == NULL)
					{
						// The object doesn't exist so we create it if a file name has been given
//						if(!Command->m_Parameters.KeyExist("FILENAME"))
//						{
//							Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_NO_FILENAME, LocalObjName);
//							GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
//							RetVal = false;
//							continue;
//						}
						if(!m_cContainer.CreateObject(LocalObjName.c_str(), iObjectType))
						{
							Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_CREATE, LocalObjName);
							GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
							RetVal = false;
							continue;
						}
						Object = (NvObjFile *)(m_cContainer.GetObject(LocalObjName.c_str()));
						ObjectCreated = true;
						Object->SetPersistTime(-1);	// Created object is considered to be persistent
						if(!LocalFileInCommand.length())
							Object->SetPersistTime(0);	// Except if no filename is given

					}
					int iPersist = Object->m_iPersistTime;
					if(Command->m_Parameters.KeyExist("PERSIST"))
					{
						string PersistTimeFileInCommand = Command->m_Parameters.GetValue("PERSIST");
						CleanString(PersistTimeFileInCommand);
						MakeUpper(PersistTimeFileInCommand);
						int iPersistTimeFileInCommand = -1;
						if(PersistTimeFileInCommand.length())
							iPersistTimeFileInCommand = atoi(PersistTimeFileInCommand.c_str());
						// Cached files not authorized locally
						if(iPersistTimeFileInCommand != 0)
							iPersistTimeFileInCommand = -1;
						iPersist = iPersistTimeFileInCommand;
					}
					

					if(LocalFileInCommand.length())
						Object->SetFileName(LocalFileInCommand);
					if(!Object->GetFileName().length())
					{
						// If no file name given
						// We try to create automatically a file following some information in the command
						string LocalFileDirInCommand = m_sTempDir;
						if(Command->m_Parameters.KeyExist("DIRECTORY"))
							LocalFileDirInCommand = Command->m_Parameters.GetValue("DIRECTORY");
						CleanString(LocalFileDirInCommand);
						string LocalFileExtensionInCommand = Command->m_Parameters.GetValue("EXTENSION");
						CleanString(LocalFileExtensionInCommand);
						string LocalFilePrefixInCommand = Command->m_Parameters.GetValue("PREFIX");
						CleanString(LocalFilePrefixInCommand);
						string LocalFileSuffixInCommand = Command->m_Parameters.GetValue("SUFFIX");
						CleanString(LocalFileSuffixInCommand);
						if(!Object->SetAutoFileName(LocalFileDirInCommand, LocalFileExtensionInCommand, LocalFilePrefixInCommand, LocalFileSuffixInCommand))
						{
							char *ErrorMessage = (char *)malloc(LocalObjName.length() + LocalFileDirInCommand.length() + LocalFileExtensionInCommand.length()  + LocalFilePrefixInCommand.length()  + LocalFileSuffixInCommand.length() + 100);
							sprintf(ErrorMessage, "obj:%s - dir:%s - ext:%s - prefix:%s - suffix:%s", LocalObjName.c_str(), LocalFileDirInCommand.c_str(), LocalFileExtensionInCommand.c_str(), LocalFilePrefixInCommand.c_str(), LocalFileSuffixInCommand.c_str());
							Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_FILE_CANNOT_CREATE, ErrorMessage);
							free(ErrorMessage);
							GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
							RetVal = false;
							continue;
						}
					}
			
					Object->SetPersistTime(iPersist);
				}
				else
				{
					// We first remove the object if it ever exist in the container
					// In fact, the new object should replace the existing one
					m_cContainer.RemoveObject(LocalObjName.c_str());

					if(!m_cContainer.CreateObject(LocalObjName.c_str(), iObjectType))
					{
						Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_CREATE, LocalObjName);
						GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
						RetVal = false;
						continue;
					}
					if(iObjectType == NV_OBJTYPE_INDSTRINGLIST)
					{
						NvObjIndStringList *Object = (NvObjIndStringList *)(m_cContainer.GetObject(LocalObjName.c_str()));
						if(Object != NULL)
						{
							string CaseSensitive = Command->m_Parameters.GetValue("CASE_SENSITIVE");
							CleanString(CaseSensitive);
							MakeUpper(CaseSensitive);
							if(CaseSensitive == "YES")
								Object->m_bCaseSensitive = true;
						}
					}
				}


				// PM TODO
				// We can call a Decode function here that operates on Command->Data
				// This Decode function can also uncompress the data stream

				// We populate the object
				bool bPopulateFromData = true;
				if(iObjectType == NV_OBJTYPE_FILE)
				{
					if(Connection->InMessage.vDatas[i+2]->DataFile.length())
					{
						bPopulateFromData = false;
						// We populate the file object directly from the data file in command
						NvObjFile *Object = (NvObjFile *)(m_cContainer.GetObject(LocalObjName.c_str()));
						if(Object != NULL)
						{
							string ObjectFileName = Object->GetFilePathName();
							if(!CopyFileEx(Connection->InMessage.vDatas[i+2]->DataFile.c_str(), ObjectFileName.c_str()))
							{
								Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_POPULATE, LocalObjName);
								GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
								RetVal = false;
								continue;
							}
						}
					}

					// Check the ASCII parameter
					string sAscii = Command->m_Parameters.GetValue("ASCII");
					CleanString(sAscii);
					if(sAscii == "YES")
					{
						NvObjFile *Object = (NvObjFile *)(m_cContainer.GetObject(LocalObjName.c_str()));
						if(Object != NULL)
							Object->SetAscii();
					}
				}
				else
				{
					if(Connection->InMessage.vDatas[i+2]->DataFile.length())
						bPopulateFromData = false;
				}

				if(bPopulateFromData)
				{
					if(!m_cContainer.PopulateObject(LocalObjName.c_str(), Connection->InMessage.vDatas[i+2]->Data, Connection->InMessage.vDatas[i+2]->DataSize))
					{
						Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_POPULATE, LocalObjName);
						GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
						RetVal = false;
						continue;
					}
				}
				else if(iObjectType != NV_OBJTYPE_FILE)
				{
					// The work has ever been done for a file object
					if(!m_cContainer.PopulateObjectFromFile(LocalObjName.c_str(), Connection->InMessage.vDatas[i+2]->DataFile.c_str()))
					{
						Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_POPULATE, LocalObjName);
						GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
						RetVal = false;
						continue;
					}
				}
			}
		}
		if((m_vCommands.size() == 1) && (iErrorCodeHere == NV_OK) && (Buffer != NULL) && ((ServiceHere != "SYSTEM") || (ClassHere != "OBJECT") || (CommandHere != "GET")))
		{
			// If the command is successful and this is not an objetc:get, we get the result in the output buffer if there is one
			// The incoming data then contain the end of string character
			// The proper encoding for client has been ever set on the server side
			// All that is faisible only when we send one commande at a time
			unsigned long InDataSize = Connection->InMessage.vDatas[2]->DataSize;
			char *Data = Connection->InMessage.vDatas[2]->Data;
			if(InDataSize)
				Data[InDataSize-1] = 0;
			string VarName = Command->m_Parameters.GetValue("NV_VAR");
			if(!VarName.length())
				VarName = "NV_RESULT";
			if(InDataSize)
			{
				m_oVariables.SetValue(VarName.c_str(), Connection->InMessage.vDatas[2]->Data);
				GetStringValue(Command, Connection->InMessage.vDatas[2]->Data, Buffer, BufferSize, DataSize, ENCODING_NONE);
			}
			else
				m_oVariables.SetValue(VarName.c_str(), "");
		}
	}

	// we clear the input message
	Connection->InMessage.Clear();

	if(m_bAutoClose)
	{
		if(CurrentConnection != NULL)
			GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
		CurrentConnection = NULL;
	}

//	GetMain()->Connections.FreeConnection(Connection);

	return RetVal;
}




// Send the command queue to server
bool NvRequest::SendCommands(NvServerCommand *Command)
{
	// Check the stop on error flag
	string sStopOnError = Command->m_Parameters.GetValue("NV_STOP_ON_ERROR");
	MakeUpper(sStopOnError);
	CleanString(sStopOnError);
	bool bStopOnError = false;
	if(sStopOnError == "YES")
		bStopOnError = true;

	// Send the commands to the server
	bool RetVal = SendServerCommands(bStopOnError);

	return RetVal;
}



// Send an XML command to the server
// The XML command doesn't use the command buffer but directly
// access the server by HTTP with input XML and output XML
// The command takes an object file as input, an object file
// as output and an URL for the command.
bool NvRequest::XmlCommand(NvServerCommand *Command)
{
	GlobalError.SetLocalError();

	string Address = m_sServer;
	CleanString(Address);

	string Proxy = "";
	string ProxyUser = "";
	string ProxyPassword = "";

	// The proxy is eventually in the address
	// at this time the address starts with proxy::proxyaddress::...
	// proxyaddress can have some optional user and password information in brackets separated by a ; character
	string AddressHere = Address;
	MakeUpper(AddressHere);
	string::size_type index = AddressHere.find("PROXY::");
	if(index != string::npos)
	{
		Address = Address.substr(7);
		index = Address.find("::");
		if(index != string::npos)
		{
			Proxy = Address.substr(0, index);
			Address = Address.substr(index+2);
		}
		index = Address.find('(');
		if(index != string::npos)
		{
			ProxyUser = Address.substr(index+1);
			Address = Address.substr(0, index);
		}
		index = ProxyUser.find(')');
		if(index != string::npos)
			ProxyUser = ProxyUser.substr(0, index);
		
		index = ProxyUser.find(';');
		if(index != string::npos)
		{
			ProxyPassword = ProxyUser.substr(index+1);
			ProxyUser = ProxyUser.substr(0, index);
		}
	}
	else
	{
		// Compatibiliy IPV6
		index = AddressHere.find("PROXY%%");
		if(index != string::npos)
		{
			Address = Address.substr(7);
			index = Address.find("%%");
			if(index != string::npos)
			{
				Proxy = Address.substr(0, index);
				Address = Address.substr(index+2);
			}
			index = Address.find('(');
			if(index != string::npos)
			{
				ProxyUser = Address.substr(index+1);
				Address = Address.substr(0, index);
			}
			index = ProxyUser.find(')');
			if(index != string::npos)
				ProxyUser = ProxyUser.substr(0, index);
			
			index = ProxyUser.find(';');
			if(index != string::npos)
			{
				ProxyPassword = ProxyUser.substr(index+1);
				ProxyUser = ProxyUser.substr(0, index);
			}
		}
	}
	CleanString(Proxy);
	CleanString(ProxyUser);
	CleanString(ProxyPassword);
	CleanString(Address);

	int Port = 1081;
	bool UseSsl = b_UseSSL;
	if(UseSsl)
		Port = 1082;
	SplitAddress(Address, Port);
/*	index = Address.find(':');
	if(index != string::npos)
	{
		string Provistr = Address.substr(index+1);
		CleanString(Provistr);
		Port = atoi(Provistr.c_str());
		Address.erase(index);
		CleanString(Provistr);
	}
*/
	if(!Address.length())
		Address = "127.0.0.1";
	
	string ConnectedServer = m_sServer; 
	string FinalAddress = Address;
	int FinalPort = Port;
	bool FinalUseSsl = UseSsl;
	bool bUseProxy = false;
	if(Proxy.length())
	{
		string ProxyHere = Proxy;
		MakeUpper(ProxyHere);
		CleanString(ProxyHere);
		index = ProxyHere.find("HTTP://");
		if(index != string::npos)
		{
			Proxy = Proxy.substr(index+7);
			bUseProxy = true;
			UseSsl = false;
		}
		if(!bUseProxy)
		{
			index = ProxyHere.find("HTTPS://");
			if(index != string::npos)
			{
				Proxy = Proxy.substr(index+8);
				bUseProxy = true;
				UseSsl = true;
			}
		}
		if(bUseProxy)
		{
			Address = Proxy;
			Port = 4480;
			SplitAddress(Address, Port);
/*			index = Address.find(':');
			if(index != string::npos)
			{
				string Provistr = Address.substr(index+1);
				CleanString(Provistr);
				Port = atoi(Provistr.c_str());
				Address.erase(index);
			}*/
		}
	}

	int ProxyMode = NO_PROXY;
	if(bUseProxy)
	{
		if(FinalUseSsl)
			ProxyMode = PROXY_FINAL_SSL;
		else
			ProxyMode = PROXY_FINAL_NORMAL;
	}

	NvHttpConnection* Connection = (NvHttpConnection*)CurrentConnection;
	if(Connection == NULL)
	{
		Connection = GetMain()->Connections.GetConnection(Address.c_str(), Port, m_iServerConnectionTimeOut, UseSsl, ProxyMode, FinalAddress.c_str(), FinalPort, m_sCertificate.c_str());
		CurrentConnection = Connection;
	}
	if(Connection == NULL)
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_CANNOT_GET_SERVER_CONNECTION, ConnectedServer);
		return false;
	}

	// We got a connected connection
	Connection->sSocket.LogFile = m_sSocketLog;
	Connection->ProxyAuthInfo.m_sUser = ProxyUser;
	Connection->ProxyAuthInfo.m_sPassword = ProxyPassword;
	Connection->AuthInfo.SsoInfo.m_sPrincipal = SSOPrincipal;

	bool RetVal = false;
	
	// we clear the input message
	Connection->InMessage.Clear();
	
	// we clear the outpput message
	Connection->OutMessage.Clear();

	// PM TODO better because we must encode only the parameters
	// and not all the string
	string UrlCommand = Command->m_Parameters.GetValue("COMMAND");
	string Value;
	if(m_sSessionId.length())
	{
		UrlCommand += "&NV_SESSION_ID=";
		Value = m_sSessionId;
		EncodeUrlString(Value, false, true);
		UrlCommand += Value;
	}
	else
	{
		UrlCommand += "&NV_USER=";
		Value = m_sUser;
		EncodeUrlString(Value, false, true);
		UrlCommand += Value;
		UrlCommand += "&NV_PASSWORD=";
		Value = m_sPassword;
		EncodeUrlString(Value, false, true);
		UrlCommand += Value;
		if(bWithNewPassword)
		{
			UrlCommand += "&NV_NEW_PASSWORD=";
			Value = m_sNewPassword;
			EncodeUrlString(Value, false, true);
			UrlCommand += Value;
			if(bWithNewPasswordConfirm)
			{
				UrlCommand += "&NV_NEW_PASSWORD=";
				Value = m_sNewPasswordConfirm;
				EncodeUrlString(Value, false, true);
				UrlCommand += Value;
			}
		}
		UrlCommand += "&NV_APPLICATION=";
		Value = m_sAppli;
		EncodeUrlString(Value, false, true);
		UrlCommand += Value;
		if(m_iSessionTimeOut)
		{
			UrlCommand += "&NV_TIMEOUT=";
			char buffer[20];
			sprintf(buffer, "%d", m_iSessionTimeOut);
			Value = buffer;
			EncodeUrlString(Value, false, true);
			UrlCommand += Value;
		}
		// Set NvSessionOpen procedure
		if(m_sOpen.length())
		{
			UrlCommand += "&NV_SESSION_OPEN=";
			Value = m_sOpen;
			EncodeUrlString(Value, false, true);
			UrlCommand += Value;
		}
		if(m_sClose.length())
		{
			UrlCommand += "&NV_SESSION_CLOSE=";
			Value = m_sClose;
			EncodeUrlString(Value, false, true);
			UrlCommand += Value;
		}
	}

	if(m_bAutoClose)
		UrlCommand += "&NV_CLOSE_SESSION=YES";

	string IsSoap = Command->m_Parameters.GetValue("SOAP");
	CleanString(IsSoap);
	MakeUpper(IsSoap);
	bool bIsSoap = false;
	if(IsSoap == "YES")
		bIsSoap = true;

	string IsWebs = Command->m_Parameters.GetValue("WEBS");
	CleanString(IsWebs);
	MakeUpper(IsWebs);
	bool bIsWebs = false;
	if(IsWebs == "YES")
		bIsWebs = true;


	NvData *Data = new NvData;

	// For HTTP 1.1 we must include the host header
	char buffer[20];
	sprintf(buffer, "%d", FinalPort);
	FinalAddress += ":";
	FinalAddress += buffer;
	
	Data->sInfo = "POST ";
	if(bUseProxy)
	{
		if(FinalUseSsl)
			Data->sInfo += "https://";
		else
			Data->sInfo += "http://";
		Data->sInfo += FinalAddress;
	}

	string StartUrl = "/NVS?XMLCOMMAND";
	if(bIsSoap)
		StartUrl = "/NVS?SOAPCOMMAND";
	if(bIsWebs)
	{
		string WebService = Command->m_Parameters.GetValue("WEBSERVICE");
		CleanString(WebService, true);
		MakeLower(WebService);
		if(!WebService.length())
			WebService = "nvdef";
		if(!m_sAppli.length())
			m_sAppli = "nvdef";
		//StartUrl = "POST ";
		StartUrl = m_sAppli;
		StartUrl += "/";
		StartUrl += WebService;
		StartUrl += "/NVS?WEBS";
		string Operation = Command->m_Parameters.GetValue("OPERATION");
		if(Operation.length())
		{
			StartUrl += "&Operation=";
			StartUrl += Operation;
		}
	}
	Data->sInfo += StartUrl;
	Data->sInfo += UrlCommand;
	if(m_sSessionId.length())
	{
		Data->sInfo += "&NV_SESSION_ID=";
		Data->sInfo += m_sSessionId;
	}
	Data->sInfo += " HTTP/1.1";
	Connection->OutMessage.vDatas.push_back(Data);
	
	if(bIsWebs || bIsSoap)
	{
		string SoapAction = Command->m_Parameters.GetValue("SOAPACTION");
		Connection->OutMessage.AddHeader("SOAPAction", SoapAction);
	}


	// Set SSO
	if(b_UseSSO)
		Connection->OutMessage.AddHeader("Nv_Sso", "yes");

	// by default, we have no content in the body
	Connection->OutMessage.AddHeader("Content-Length", "0");
	
	// For HTTP 1.1 we must include the host header
	Connection->OutMessage.AddHeader("Host", FinalAddress);

	Connection->OutMessage.AddHeader("Nv_Source", HTTP_CLIENT_STRING);

	// We prepare the output global headers
	
	// Set content-type
	string sContentType = "text/xml";
	Connection->OutMessage.AddHeader("Content-Type", sContentType);
	
		
	// Write now the input XML
	// This comes from a local object named XMLIN


	NvObj* Object = m_cContainer.GetObject("XMLIN");
	if(Object == NULL)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_GET, "XMLIN");
		Connection->OutMessage.Clear();
		if(m_bAutoClose)
		{
			if(CurrentConnection != NULL)
				GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
			CurrentConnection = NULL;
		}
		return false;
	}
	int iObjectType = Object->GetType();
	if(iObjectType != NV_OBJTYPE_FILE)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_BAD_TYPE, "XMLIN");
		Connection->OutMessage.Clear();
		if(m_bAutoClose)
		{
			if(CurrentConnection != NULL)
				GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
			CurrentConnection = NULL;
		}
		return false;
	}


	NvInt64 FileSize = ((NvObjFile *)Object)->GetSize();
	bool bFileMessage = false;
	if(FileSize > (NvInt64)m_iMaxMessMemSize)	// for files greater than 1024 kbytes
		bFileMessage = true;
	
	if(bFileMessage)
	{
		string ObjectFileName = ((NvObjFile *)Object)->GetFilePathName();
		Connection->OutMessage.vDatas[0]->DataFile = ObjectFileName;
		Connection->OutMessage.vDatas[0]->bDeleteDataFile = false;	// Don't remove the file because we directly use the object file
	}
	else
	{
		// Not a too big file so we use memory data
		if(!((NvObjFile *)Object)->GetData(&(Connection->OutMessage.vDatas[0]->Data), &(Connection->OutMessage.vDatas[0]->DataSize)))
		{
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_GET_DATA, "XMLIN");
			Connection->OutMessage.Clear();
			if(m_bAutoClose)
			{
				if(CurrentConnection != NULL)
					GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
				CurrentConnection = NULL;
			}
			return false;
		}
		// For file object we must send the file origin in a dedicated http header
		Connection->OutMessage.AddHeader("Nv_File_Origin", ((NvObjFile *)Object)->GetOrigin(),0);
	}


	// Set the real body size
	NvInt64 BodySize = Connection->OutMessage.GetWriteBodySize();
	sprintf(buffer, "%llu", BodySize);
	Connection->OutMessage.AddHeader("Content-Length", buffer);


	// Now send the order
	RetVal = true;
	string ErrorInfo = "";
	int OrderResult = Connection->SendOrder(true, m_iMaxMessMemSize, m_sTempDir.c_str(), ErrorInfo);

	// we clear the output message
	Connection->OutMessage.Clear();

	if(OrderResult != 1)
	{
		// An error occured
		// we clear the input message
		Connection->InMessage.Clear();

		if(OrderResult == -2)
		{
			string Info = Address;
			Info += " - ";
			Info += ErrorInfo;
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_HTTP_PROXY_AUTHENTICATION_ERROR, Info);
		}
		else if(OrderResult == -1)
		{
			string Info = ConnectedServer;
			Info += " - ";
			Info += ErrorInfo;
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_HTTP_AUTHENTICATION_ERROR, Info);
		}
		else
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_NETWORK_ERROR, ConnectedServer);
		if(m_bAutoClose)
		{
			if(CurrentConnection != NULL)
				GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
			CurrentConnection = NULL;
		}
		return false;
	}
	
	// Get the Connection->OutMessage here


	if(Connection->InMessage.HeaderExist("NV_SESSION_ID", 0))
		m_sSessionId = Connection->InMessage.GetHeader("NV_SESSION_ID", 0);
	if(!Connection->InMessage.HeaderExist("NV_SERVER_VERSION", 1))
		m_sServerVersion = Connection->InMessage.GetHeader("NV_SERVER_VERSION", 1);
	CleanString(m_sServerVersion, true);


	// Now we get the returned XML data that we put into the XMLOUT file object
	// The XMLOUT objecty must exist
	Object = m_cContainer.GetObject("XMLOUT");
	if(Object == NULL)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_GET, "XMLOUT");
		Connection->OutMessage.Clear();
		if(m_bAutoClose)
		{
			if(CurrentConnection != NULL)
				GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
			CurrentConnection = NULL;
		}
		return false;
	}
	iObjectType = Object->GetType();
	if(iObjectType != NV_OBJTYPE_FILE)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_BAD_TYPE, "XMLOUT");
		Connection->OutMessage.Clear();
		if(m_bAutoClose)
		{
			if(CurrentConnection != NULL)
				GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
			CurrentConnection = NULL;
		}
		return false;
	}

	// We populate the object
	if(Connection->InMessage.vDatas[0]->DataFile.length())
	{
		// We populate the file object directly from the data file in command
		string ObjectFileName = ((NvObjFile *)Object)->GetFilePathName();
		if(!CopyFileEx(Connection->InMessage.vDatas[0]->DataFile.c_str(), ObjectFileName.c_str()))
		{
			Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_POPULATE, "XMLOUT");
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
			RetVal = false;
		}
	}
	else
	{
		if(!((NvObjFile *)Object)->Populate(Connection->InMessage.vDatas[0]->Data, Connection->InMessage.vDatas[0]->DataSize))
		{
			Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_POPULATE, "XMLOUT");
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_SERVER_COMMAND_ERROR, ConnectedServer);
			RetVal = false;
		}
	}

	// we clear the input message
	Connection->InMessage.Clear();

	if(m_bAutoClose)
	{
		if(CurrentConnection != NULL)
			GetMain()->Connections.FreeConnection((NvHttpConnection*)CurrentConnection);
		CurrentConnection = NULL;
	}

	return RetVal;
}




// Removes the command from command buffer
bool NvRequest::RemoveCommand(NvServerCommand *Command)
{
	// If the COMMAND_INDEX parameter is not set or set to 0,
	// we remove the last command
	int iCommandIndex = 0;
	string CommandIndex = Command->m_Parameters.GetValue("NV_COMMAND_INDEX");
	CleanString(CommandIndex);
	if(Command->m_Parameters.KeyExist("NV_COMMAND_INDEX"))
		iCommandIndex = atoi(CommandIndex.c_str());
	
	if(iCommandIndex == 0)
	{
		if(!m_vCommands.size())
			return true;
		iCommandIndex = (int)m_vCommands.size() - 1;
	}

	if((iCommandIndex < 0) || (iCommandIndex >= (int)m_vCommands.size()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_COMMAND_INDEX, CommandIndex);
		return false;
	}

	NvServerCommand *CommandInList = (NvServerCommand *)m_vCommands[iCommandIndex];
	if(CommandInList != NULL)
		delete CommandInList;

	m_vCommands.erase(m_vCommands.begin() + iCommandIndex);

	return true;
}


void NvRequest::GetNumCommands(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	char Buf[50];
	sprintf(Buf, "%d", m_vCommands.size());
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);
}



void NvRequest::GetError(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	string ErrorType = Command->m_Parameters.GetValue("ERROR_STRING");
	CleanString(ErrorType);
	MakeUpper(ErrorType);
	if(!ErrorType.length())
		ErrorType = "CODE";

	// By default, the global error is requested
	NvError *Error = &GlobalError;
	
	// We check if an error from a command in the command buffer has been requested
	// For that, the user must set the NV_COMMAND_INDEX parameter to a valid command index
	// A command index start at 1 (0 is reserved for global error)
	string CommandIndex = Command->m_Parameters.GetValue("NV_COMMAND_INDEX");
	CleanString(CommandIndex);
	int iCommandIndex = atoi(CommandIndex.c_str());

	if((iCommandIndex > 0) && (iCommandIndex <= (int)m_vCommands.size()))
	{
		NvServerCommand *CommandInList = (NvServerCommand *)(m_vCommands[iCommandIndex-1]);
		Error = &(CommandInList->m_Error);
	}

	// Now Error points to the requested error object
	
	// We get now the error string following the requested one
	string RetVal = "";
	if(ErrorType == "CODE")
	{
		char Buf[50];
		sprintf(Buf, "%d", Error->m_iErrorCode);
		RetVal = Buf;
	}
	if(ErrorType == "SERVICE")
		RetVal = Error->m_sErrorService;
	if(ErrorType == "CLASS")
		RetVal = Error->m_sErrorClass;
	if(ErrorType == "INFO")
		RetVal = Error->m_sErrorInfo;
	if(ErrorType == "DESCRIPTION")
		RetVal = Error->m_sErrorDesc;

	GetStringValue(Command, RetVal.c_str(), Buffer, BufferSize, DataSize, iEncoding);
}




void NvRequest::GetSessionId(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	GetStringValue(Command, m_sSessionId.c_str(), Buffer, BufferSize, DataSize, iEncoding);
}



void NvRequest::GetServerVersion(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	GetStringValue(Command, m_sServerVersion.c_str(), Buffer, BufferSize, DataSize, iEncoding);
}











//REQUEST SET_VARIABLE
void NvRequest::SetVariable(NvServerCommand *Command)
{
	// Get variable name that should be in command parameters
	string Name = Command->m_Parameters.GetValue("NAME");

	// Get value that should be in command parameters
	string Value = Command->m_Parameters.GetValue("VALUE");

	m_oVariables.SetValue(Name, Value);
}


//REQUEST GET_VARIABLE
void NvRequest::GetVariable(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string Name = Command->m_Parameters.GetValue("NAME");

	GetStringValue(Command, m_oVariables.GetValue(Name).c_str(), Buffer, BufferSize, DataSize, iEncoding);
}



//REQUEST REMOVE_VARIABLE
void NvRequest::RemoveVariable(NvServerCommand *Command)
{
	// Get variable name that should be in command parameters
	string Name = Command->m_Parameters.GetValue("NAME");
	if(!Command->m_Parameters.KeyExist("NAME"))
		m_oVariables.RemoveAll();		// Remove all variables
	else
		m_oVariables.Remove(Name);
}


//REQUEST VARIABLE EXIST
void NvRequest::VariableExist(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get variable name that should be in command parameters
	string Name = Command->m_Parameters.GetValue("NAME");

	string result = "NO";
	if(m_oVariables.KeyExist(Name))
		result = "YES";

	GetStringValue(Command, result.c_str(), Buffer, BufferSize, DataSize, iEncoding);
}



bool NvRequest::CreateObject(NvServerCommand *Command)
{
	// Get object name and type that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);
	if(!ObjName.length())
		ObjName = NV_OBJ_DEFAULT_NAME;

	string ObjType = Command->m_Parameters.GetValue("TYPE");
	CleanString(ObjType);
	MakeUpper(ObjType);
	
	if(!ObjType.length())
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_NO_TYPE, ObjName);
		return false;
	}

	int iObjectType = NV_OBJTYPE_UNKNOWN;
	if(ObjType == "BOOLEAN")
		iObjectType = NV_OBJTYPE_BOOLEAN;
	if(ObjType == "INTEGER")
		iObjectType = NV_OBJTYPE_INTEGER;
	if(ObjType == "STRING")
		iObjectType = NV_OBJTYPE_STRING;
	if(ObjType == "STRINGLIST")
		iObjectType = NV_OBJTYPE_STRINGLIST;
	if(ObjType == "INDSTRINGLIST")
		iObjectType = NV_OBJTYPE_INDSTRINGLIST;
	if(ObjType == "TABLE")
		iObjectType = NV_OBJTYPE_TABLE;
	if(ObjType == "FILE")
		iObjectType = NV_OBJTYPE_FILE;
	if(ObjType == "BINARY")
		iObjectType = NV_OBJTYPE_BINARY;

	
	if(iObjectType == NV_OBJTYPE_UNKNOWN)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_UNKNOWN_TYPE, ObjName);
		return false;
	}
	
	
	NvObj *Object = m_cContainer.GetObject(ObjName.c_str());

	if(Object != NULL)
	{
		// Check for the no-replace mode
		string ReplaceMode = Command->m_Parameters.GetValue("REPLACE");
		CleanString(ReplaceMode);
		MakeUpper(ReplaceMode);
		if(ReplaceMode != "YES")
		{
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_EVER_EXIST, ObjName);
			return false;
		}

		// The object exists so we remove it
		m_cContainer.RemoveObject(ObjName.c_str());
		Object = NULL;
	}
		
	if(!m_cContainer.CreateObject(ObjName.c_str(), iObjectType))
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_CREATE, ObjName);
		return false;
	}
	
	Object = m_cContainer.GetObject(ObjName.c_str());

	if(iObjectType == NV_OBJTYPE_FILE)
	{
		NvObjFile *ObjectFile = (NvObjFile *)Object;
		string PersistTimeFileInCommand = Command->m_Parameters.GetValue("PERSIST");
		CleanString(PersistTimeFileInCommand);
		MakeUpper(PersistTimeFileInCommand);
		int iPersistTimeFileInCommand = -1;
		if(PersistTimeFileInCommand.length())
			iPersistTimeFileInCommand = atoi(PersistTimeFileInCommand.c_str());
		// Cached files not authorized locally
		if(iPersistTimeFileInCommand != 0)
			iPersistTimeFileInCommand = -1;


		string LocalFileInCommand = Command->m_Parameters.GetValue("FILENAME");
		if(LocalFileInCommand.length())
			ObjectFile->SetFileName(LocalFileInCommand);
		else
		{
			// If no file name given
			// We try to create automatically a file following some information in the command
			if(!Command->m_Parameters.KeyExist("PERSIST"))
				iPersistTimeFileInCommand = 0;	// If filename is not given, we consider a temporary file
			
			string LocalFileDirInCommand = m_sTempDir;
			if(Command->m_Parameters.KeyExist("DIRECTORY"))
				LocalFileDirInCommand = Command->m_Parameters.GetValue("DIRECTORY");
			CleanString(LocalFileDirInCommand);
			string LocalFileExtensionInCommand = Command->m_Parameters.GetValue("EXTENSION");
			CleanString(LocalFileExtensionInCommand);
			string LocalFilePrefixInCommand = Command->m_Parameters.GetValue("PREFIX");
			CleanString(LocalFilePrefixInCommand);
			string LocalFileSuffixInCommand = Command->m_Parameters.GetValue("SUFFIX");
			CleanString(LocalFileSuffixInCommand);
			if(!ObjectFile->SetAutoFileName(LocalFileDirInCommand, LocalFileExtensionInCommand, LocalFilePrefixInCommand, LocalFileSuffixInCommand))
			{
				char *ErrorMessage = (char *)malloc(ObjName.length() + LocalFileDirInCommand.length() + LocalFileExtensionInCommand.length()  + LocalFilePrefixInCommand.length()  + LocalFileSuffixInCommand.length() + 100);
				sprintf(ErrorMessage, "obj:%s - dir:%s - ext:%s - prefix:%s - suffix:%s", ObjName.c_str(), LocalFileDirInCommand.c_str(), LocalFileExtensionInCommand.c_str(), LocalFilePrefixInCommand.c_str(), LocalFileSuffixInCommand.c_str());
				GlobalError.SetLocalError("OBJECT", NV_LOCAL_FILE_CANNOT_CREATE, ErrorMessage);
				free(ErrorMessage);
				m_cContainer.RemoveObject(ObjName.c_str());
				return false;
			}
		}

		ObjectFile->SetPersistTime(iPersistTimeFileInCommand);
	}
	if(iObjectType == NV_OBJTYPE_BOOLEAN)
	{
		NvObjBoolean *ObjectBoolean = (NvObjBoolean *)Object;
		
		if(Command->m_Parameters.KeyExist("VALUE"))
		{
			string ObjValue = Command->m_Parameters.GetValue("VALUE");
			CleanString(ObjValue);
			MakeUpper(ObjValue);

			// The default is false
			// Any other value than "TRUE" is considered as FALSE
			bool NewValue = false;
			if(ObjValue == "TRUE")
				NewValue = true;

			ObjectBoolean->SetValue(NewValue);
		}
	}
	if(iObjectType == NV_OBJTYPE_INTEGER)
	{
		NvObjInteger *ObjectInteger = (NvObjInteger *)Object;
		if(Command->m_Parameters.KeyExist("VALUE"))
		{
			string ObjValue = Command->m_Parameters.GetValue("VALUE");
			CleanString(ObjValue);
			MakeUpper(ObjValue);
			int NewValue = atoi(ObjValue.c_str());
			ObjectInteger->SetValue(NewValue);
		}
	}
	if(iObjectType == NV_OBJTYPE_STRING)
	{
		NvObjString *ObjectString = (NvObjString *)Object;
		if(Command->m_Parameters.KeyExist("VALUE"))
		{
			string ObjValue = Command->m_Parameters.GetValue("VALUE");
			ObjectString->SetValue(ObjValue);
		}
	}
	if(iObjectType == NV_OBJTYPE_STRINGLIST)
	{
		NvObjStringList *ObjectStringList = (NvObjStringList *)Object;
		if(Command->m_Parameters.KeyExist("VALUE"))
		{
			string ObjValue = Command->m_Parameters.GetValue("VALUE");
			string Separator = Command->m_Parameters.GetValue("SEPARATOR");
			ObjectStringList->Add(ObjValue, Separator);
		}
	}
	if(iObjectType == NV_OBJTYPE_INDSTRINGLIST)
	{
		NvObjIndStringList *ObjectIndStringList = (NvObjIndStringList *)Object;
		string CaseSensitive = Command->m_Parameters.GetValue("CASE_SENSITIVE");
		CleanString(CaseSensitive);
		MakeUpper(CaseSensitive);
		if(CaseSensitive == "YES")
			ObjectIndStringList->m_bCaseSensitive = true;
		if(Command->m_Parameters.KeyExist("VALUE"))
		{
			string ObjValue = Command->m_Parameters.GetValue("VALUE");
			string KeySep = ";";
			string ValSep = "=";
			string KeySepHere = Command->m_Parameters.GetValue("KEYSEP");
			CleanString(KeySepHere);
			if(KeySepHere.length())
				KeySep = KeySepHere;
			string ValSepHere = Command->m_Parameters.GetValue("VALSEP");
			CleanString(ValSepHere);
			if(ValSepHere.length())
				ValSep = ValSepHere;
			string ObjKey = "";
			ObjectIndStringList->SetValue(ObjKey, ObjValue, KeySep, ValSep);
		}
	}
	if(iObjectType == NV_OBJTYPE_TABLE)
	{
		NvObjTable *ObjectTable = (NvObjTable *)Object;

		string ColumnsHere = Command->m_Parameters.GetValue("COLUMNS");
		CleanString(ColumnsHere);
		if(ColumnsHere.length())
		{
			string::size_type index;
			while(ColumnsHere.length())
			{
				string Column = ColumnsHere;
				string ColumnDesc = "";
				bool ColumnNumeric = false;
				index = Column.find(';');
				if(index != string::npos)
				{
					Column = ColumnsHere.substr(0, index);
					ColumnsHere = ColumnsHere.substr(index+1);
				}
				else
					ColumnsHere = "";
				index = Column.find(':');
				if(index != string::npos)
				{
					ColumnDesc = Column.substr(index+1);
					Column = Column.substr(0, index);
				}
				index = ColumnDesc.find(':');
				if(index != string::npos)
				{
					string sColumnNumeric = ColumnDesc.substr(index+1);
					ColumnDesc = ColumnDesc.substr(0, index);
					CleanString(sColumnNumeric);
					MakeUpper(sColumnNumeric);
					if(sColumnNumeric == "Y")
						ColumnNumeric = true;
				}
				CleanString(Column, true);
				CleanString(ColumnDesc);
				if(!Column.length())
					continue;
				ObjectTable->InsertColumn(Column, ColumnDesc, ColumnNumeric);
			}
		}

		if(Command->m_Parameters.KeyExist("VALUE"))
		{
			string ObjValue = Command->m_Parameters.GetValue("VALUE");
			
			// Get eventual record separator
			string RecSep = Command->m_Parameters.GetValue("ROWSEP");
			if(!RecSep.length())
				RecSep = "\n";

			// Get eventual column separator
			string ColSep = Command->m_Parameters.GetValue("COLSEP");
			if(!ColSep.length())
				ColSep = ";";

			// Get eventual line separator
			string LineSep = Command->m_Parameters.GetValue("LINESEP");
			if(!LineSep.length())
				LineSep = "|";

			if(!ObjectTable->AddRecords(ObjValue.c_str(), RecSep.c_str(), ColSep.c_str(), LineSep.c_str()))
			{
				m_cContainer.RemoveObject(ObjName.c_str());
				GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_ADD_RECORDS, ObjName);
				return false;
			}
		}
	}
	
	return true;
}




void NvRequest::GetNumObjects(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	char Buf[50];
	sprintf(Buf, "%d", m_cContainer.m_mObjects.size());
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);
}




bool NvRequest::GetObjectName(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object index that should be in command parameters
	int iObjIndex = 0;
	string ObjIndex = Command->m_Parameters.GetValue("INDEX");
	CleanString(ObjIndex);
	iObjIndex = atoi(ObjIndex.c_str());
	
	if((iObjIndex < 1) || (iObjIndex > (int)m_cContainer.m_mObjects.size()))
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_INVALID_OBJECT_INDEX, ObjIndex);
		return false;
	}
	
	map<string,void *>::iterator pos;
	int i;
	NvObj* Object = NULL;
	for(i=0,pos = m_cContainer.m_mObjects.begin();i<iObjIndex;i++,++pos)
	{
		Object = (NvObj*)(pos->second);
	}
	string ObjectName = "";
	if(Object != NULL)
		ObjectName = Object->GetName();

	GetStringValue(Command, ObjectName.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;

}




bool NvRequest::GetObjectType(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObj* Object = m_cContainer.GetObject(ObjName.c_str());

	if(Object == NULL)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_INVALID_OBJECT, ObjName);
		return false;
	}

	char Buf[100];
	switch(Object->GetType())
	{
		case NV_OBJTYPE_BOOLEAN:
			strcpy(Buf, "BOOLEAN");
			break;
		case NV_OBJTYPE_INTEGER:
			strcpy(Buf, "INTEGER");
			break;
		case NV_OBJTYPE_STRING:
			strcpy(Buf, "STRING");
			break;
		case NV_OBJTYPE_STRINGLIST:
			strcpy(Buf, "STRINGLIST");
			break;
		case NV_OBJTYPE_INDSTRINGLIST:
			strcpy(Buf, "INDSTRINGLIST");
			break;
		case NV_OBJTYPE_TABLE:
			strcpy(Buf, "TABLE");
			break;
		case NV_OBJTYPE_FILE:
			strcpy(Buf, "FILE");
			break;
		case NV_OBJTYPE_BINARY:
			strcpy(Buf, "BINARY");
			break;
		default:
			strcpy(Buf, "UNKNOWN");
			break;
	}

	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;

}


NvObj* NvRequest::GetObjectCheckType(const char *ObjName, int ObjType)
{
	if(ObjName == NULL)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_INVALID_OBJECT);
		return NULL;
	}
	NvObj* Object = m_cContainer.GetObject(ObjName);

	if(Object == NULL)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_INVALID_OBJECT, ObjName);
		return NULL;
	}

	if((ObjType != -1) && (Object->GetType() != ObjType))
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_UNKNOWN_TYPE, ObjName);
		return NULL;
	}

	return Object;
}


//OBJECT COPY
bool NvRequest::CopyObject(NvServerCommand *Command)
{
	// Get object source and destination names that should be in command parameters
	string SObjName = Command->m_Parameters.GetValue("SNAME");
	CleanString(SObjName);
	string DObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(DObjName);

	NvObj* SourceObject = GetObjectCheckType(SObjName.c_str());

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	NvObj* DestinationObject = GetObjectCheckType(DObjName.c_str());

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	if(SourceObject == DestinationObject)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_SAME_OBJECTS, SObjName);
		return false;
	}

	int iType = SourceObject->GetType();
	if(iType != DestinationObject->GetType())
	{
		char *ErrorMessage = (char *)malloc(SObjName.length() + DObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", SObjName.c_str(), DObjName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECTS_NOT_COMPATIBLE, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	if((iType <= NV_OBJTYPE_UNKNOWN) || (iType > NV_OBJTYPE_MAXTYPEVALUE))
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_UNKNOWN_TYPE, SObjName);
		return false;
	}

	// Check if an offset and a number of bytes has been requested
	// They are used for file and binary types
	NvInt64 Offset = 0;
	NvInt64 BytesToCopy = -1;
	if(Command->m_Parameters.KeyExist("OFFSET"))
	{
		string sOffset = Command->m_Parameters.GetValue("OFFSET");
		CleanString(sOffset);
		Offset = NvAtoi64(sOffset.c_str());
	}
	if(Command->m_Parameters.KeyExist("NUM_BYTES"))
	{
		string sBytesToCopy = Command->m_Parameters.GetValue("NUM_BYTES");
		CleanString(sBytesToCopy);
		BytesToCopy = NvAtoi64(sBytesToCopy.c_str());
	}

	bool RetVal = true;
	if(iType == NV_OBJTYPE_BOOLEAN)
	{
		NvObjBoolean *Sobj = (NvObjBoolean*)SourceObject;
		NvObjBoolean *Dobj = (NvObjBoolean*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_INTEGER)
	{
		NvObjInteger *Sobj = (NvObjInteger*)SourceObject;
		NvObjInteger *Dobj = (NvObjInteger*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_STRING)
	{
		NvObjString *Sobj = (NvObjString*)SourceObject;
		NvObjString *Dobj = (NvObjString*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_STRINGLIST)
	{
		NvObjStringList *Sobj = (NvObjStringList*)SourceObject;
		NvObjStringList *Dobj = (NvObjStringList*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_INDSTRINGLIST)
	{
		NvObjIndStringList *Sobj = (NvObjIndStringList*)SourceObject;
		NvObjIndStringList *Dobj = (NvObjIndStringList*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_TABLE)
	{
		NvObjTable *Sobj = (NvObjTable*)SourceObject;
		NvObjTable *Dobj = (NvObjTable*)DestinationObject;
		Dobj->CopyFrom(Sobj);
	}
	if(iType == NV_OBJTYPE_FILE)
	{
		NvObjFile *Sobj = (NvObjFile*)SourceObject;
		NvObjFile *Dobj = (NvObjFile*)DestinationObject;
		if(!(Dobj->GetFileName().length()))
		{
			m_cContainer.RemoveObject(DObjName.c_str());
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_FILE_NO_FILE, DObjName);
		}
		else
		RetVal = Dobj->CopyFrom(Sobj, Offset, BytesToCopy);
	}
	if(iType == NV_OBJTYPE_BINARY)
	{
		NvObjBinary *Sobj = (NvObjBinary*)SourceObject;
		NvObjBinary *Dobj = (NvObjBinary*)DestinationObject;
		RetVal = Dobj->CopyFrom(Sobj, (int)Offset, (int)BytesToCopy);
	}

	if(!RetVal)
	{
		char *ErrorMessage = (char *)malloc(SObjName.length() + DObjName.length() + 100);
		sprintf(ErrorMessage, "%s -> %s", SObjName.c_str(), DObjName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_COPY, ErrorMessage);
		free(ErrorMessage);
	}
	return RetVal;
}







//OBJECT REMOVE
bool NvRequest::SetObjectName(NvServerCommand *Command)
{


	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);
	if(!ObjName.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:SET_NAME:NAME");
		return false;
	}

	// Get object name that should be in command parameters
	string NewObjName = Command->m_Parameters.GetValue("NEW_NAME");
	CleanString(NewObjName);
	if(!NewObjName.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:SET_NAME:NEW_NAME");
		return false;
	}


	NvObj* Object = m_cContainer.GetObject(NewObjName.c_str());
	if(Object == NULL)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_EVER_EXIST, NewObjName);
		return false;
	}


	Object = m_cContainer.GetObject(ObjName.c_str());
	if(Object == NULL)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_GET, ObjName);
		return false;
	}

	m_cContainer.m_mObjects.erase(Object->GetName());
	Object->SetName(NewObjName);
	m_cContainer.m_mObjects[Object->GetName()] = (void *)Object;

	return true;
}



//OBJECT REMOVE
bool NvRequest::RemoveObject(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	m_cContainer.RemoveObject(ObjName.c_str());

	return true;
}


//OBJECT EXIST
void NvRequest::IsObjectExist(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	char Buf[20];
	strcpy(Buf, "NO");
	if(m_cContainer.GetObject(ObjName.c_str()) != NULL)
		strcpy(Buf, "YES");

	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);
}


//OBJECT BOOLEAN_SET_VALUE
bool NvRequest::ObjBooleanSetValue(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = Command->m_Parameters.GetValue("VALUE");
	CleanString(ObjValue);
	MakeUpper(ObjValue);

	// The default is false
	// Any other value than "TRUE" is considered as FALSE
	bool NewValue = false;
	if(ObjValue == "TRUE")
		NewValue = true;



	NvObjBoolean* Object = (NvObjBoolean*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_BOOLEAN);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->SetValue(NewValue);

	return true;
}


//OBJECT BOOLEAN_GET_VALUE
bool NvRequest::ObjBooleanGetValue(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjBoolean* Object = (NvObjBoolean*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_BOOLEAN);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	bool Value = Object->GetValue();

	char RetVal[20];
	strcpy(RetVal, "FALSE");
	if(Value)
		strcpy(RetVal, "TRUE");
	
	GetStringValue(Command, RetVal, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}





//OBJECT INTEGER_SET_VALUE
bool NvRequest::ObjIntegerSetValue(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = Command->m_Parameters.GetValue("VALUE");
	CleanString(ObjValue);

	int NewValue = atoi(ObjValue.c_str());

	NvObjInteger* Object = (NvObjInteger*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_INTEGER);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->SetValue(NewValue);

	return true;
}


//OBJECT INTEGER_GET_VALUE
bool NvRequest::ObjIntegerGetValue(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjInteger* Object = (NvObjInteger*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_INTEGER);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int Value = Object->GetValue();

	char RetVal[50];
	sprintf(RetVal, "%d", Value);
	
	GetStringValue(Command, RetVal, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}







//OBJECT STRING_SET_VALUE
bool NvRequest::ObjStringSetValue(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = Command->m_Parameters.GetValue("VALUE");

	NvObjString* Object = (NvObjString*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_STRING);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->SetValue(ObjValue);

	return true;
}


//OBJECT STRING_GET_VALUE
bool NvRequest::ObjStringGetValue(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjString* Object = (NvObjString*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_STRING);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(Command, Object->GetValue().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}






//OBJECT STRINGLIST_SET_VALUE
bool NvRequest::ObjStringListSetValue(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = Command->m_Parameters.GetValue("VALUE");

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int iIndex = 1;
	bool WithIndex = false;
	if(Command->m_Parameters.KeyExist("INDEX"))
	{
		string sIndex = Command->m_Parameters.GetValue("INDEX");
		CleanString(sIndex);
		iIndex = atoi(sIndex.c_str());
		WithIndex = true;
	}

	if(!WithIndex)					// If no index, we just add the value to the list
	{
		// Get eventual separator
		string Separator = Command->m_Parameters.GetValue("SEPARATOR");
		Object->Add(ObjValue, Separator);
	}
	else
	{
		iIndex--;	// The real index starts at 0
		if(!Object->CheckIndex(iIndex))
		{
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_STRINGLIST_INDEX, ObjName);
			return false;
		}
		Object->SetValue(iIndex, ObjValue);
	}

	return true;
}


//OBJECT STRINGLIST_GET_VALUE
bool NvRequest::ObjStringListGetValue(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int iIndex = 1;
	bool WithIndex = false;
	bool LastIndex = false;
	bool FirstIndex = false;
	if(Command->m_Parameters.KeyExist("INDEX"))
	{
		string sIndex = Command->m_Parameters.GetValue("INDEX");
		CleanString(sIndex);
		MakeUpper(sIndex);
		if(sIndex == "FIRST")
			FirstIndex = true;
		if(sIndex == "LAST")
			FirstIndex = true;
		iIndex = atoi(sIndex.c_str());
		WithIndex = true;
	}
	if(!WithIndex)	// We take last index
		iIndex = Object->GetSize();
	if(FirstIndex)	// We take first index
		iIndex = 1;
	if(LastIndex)	// We take last index
		iIndex = Object->GetSize();

	iIndex--;	// The real index starts at 0
	if(!Object->CheckIndex(iIndex))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}

	GetStringValue(Command, Object->GetValue(iIndex).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}


//OBJECT STRINGLIST_GET_SIZE
bool NvRequest::ObjStringListGetSize(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	char Buf[50];
	sprintf(Buf, "%d", Object->GetSize());
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT STRINGLIST_REMOVE
bool NvRequest::ObjStringListRemove(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int iIndex = 1;
	bool WithIndex = false;
	bool LastIndex = false;
	bool FirstIndex = false;
	bool AllIndex = false;
	if(Command->m_Parameters.KeyExist("INDEX"))
	{
		string sIndex = Command->m_Parameters.GetValue("INDEX");
		CleanString(sIndex);
		MakeUpper(sIndex);
		if(sIndex == "FIRST")
			FirstIndex = true;
		if(sIndex == "LAST")
			LastIndex = true;
		if(sIndex == "ALL")
			AllIndex = true;
		iIndex = atoi(sIndex.c_str());
		WithIndex = true;
	}
	if((!WithIndex) || AllIndex)	// We remove all
	{
		Object->RemoveAll();
		return true;
	}
	if(FirstIndex)	// We take first index
		iIndex = 1;
	if(LastIndex)	// We take last index
		iIndex = Object->GetSize();

	iIndex--;	// The real index starts at 0
	if(!Object->CheckIndex(iIndex))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}
	if(LastIndex)	// We take last index
	{
		Object->RemoveLast();
		return true;
	}

	Object->Remove(iIndex);

	return true;
}




//OBJECT STRINGLIST_INSERT
bool NvRequest::ObjStringListInsert(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = Command->m_Parameters.GetValue("VALUE");

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int iIndex = 1;
	bool WithIndex = false;
	if(Command->m_Parameters.KeyExist("INDEX"))
	{
		string sIndex = Command->m_Parameters.GetValue("INDEX");
		CleanString(sIndex);
		iIndex = atoi(sIndex.c_str());
		WithIndex = true;
	}

	if(!WithIndex)					// If no index, we just add the value to the list
	{
		// Get eventual separator
		string Separator = Command->m_Parameters.GetValue("SEPARATOR");
		Object->Add(ObjValue, Separator);
	}
	else
	{
		iIndex--;	// The real index starts at 0
		if((!Object->CheckIndex(iIndex)) && (iIndex < Object->GetSize()))
		{
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_STRINGLIST_INDEX, ObjName);
			return false;
		}
		Object->Insert(iIndex, ObjValue);
	}

	return true;
}




//OBJECT STRINGLIST_ADD_STRINGLIST
bool NvRequest::ObjStringListAddStringList(NvServerCommand *Command)
{
	// Get object source and destination names that should be in command parameters
	string SObjName = Command->m_Parameters.GetValue("SNAME");
	CleanString(SObjName);
	string DObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(DObjName);

	NvObjStringList* SourceObject = (NvObjStringList*)GetObjectCheckType(SObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	NvObjStringList* DestinationObject = (NvObjStringList*)GetObjectCheckType(DObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	DestinationObject->Add(SourceObject);

	return true;
}






//OBJECT STRINGLIST_SWAP
bool NvRequest::ObjStringListSwap(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	int iIndex1 = -1;
	if(!Command->m_Parameters.KeyExist("INDEX1"))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}
	string sIndex = Command->m_Parameters.GetValue("INDEX1");
	CleanString(sIndex);
	iIndex1 = atoi(sIndex.c_str());
	
	int iIndex2 = -1;
	if(!Command->m_Parameters.KeyExist("INDEX2"))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}
	sIndex = Command->m_Parameters.GetValue("INDEX2");
	CleanString(sIndex);
	iIndex2 = atoi(sIndex.c_str());
	
	iIndex1--;	// The real index starts at 0
	iIndex2--;	// The real index starts at 0
	
	if(!Object->CheckIndex(iIndex1))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}

	if(!Object->CheckIndex(iIndex1))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_STRINGLIST_INDEX, ObjName);
		return false;
	}

	Object->Swap(iIndex1, iIndex2);

	return true;
}






//OBJECT STRINGLIST_SORT
bool NvRequest::ObjStringListSort(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	bool bNumeric = false;
	string sNumeric = Command->m_Parameters.GetValue("NUMERIC");
	CleanString(sNumeric);
	MakeUpper(sNumeric);
	if(sNumeric == "YES")
		bNumeric = true;


	string Natural = Command->m_Parameters.GetValue("NATURAL");
	CleanString(Natural);
	MakeUpper(Natural);
	bool bNatural = false;
	if(Natural == "YES")
		bNatural = true;

	Object->Sort(bNumeric, bNatural);

	return true;
}


//OBJECT STRINGLIST_GET_VALUES
bool NvRequest::ObjStringListGetValues(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	string What = Command->m_Parameters.GetValue("WHAT");
	CleanString(What);
	MakeUpper(What);
	if(!What.length())
		What = "NV_ALL";
	int From = 1;
	int To = Object->GetSize();
	if(What != "NV_ALL")
	{
		// range
		string sFrom = What;
		string sTo = "";
		string::size_type index = What.find('-');
		if(index == string::npos)
		{
			// Only one value telling exactly what to get
			From = atoi(What.c_str());
			To = From;
		}
		else
		{
			sFrom = What.substr(0, index);
			sTo = What.substr(index+1);
			From = atoi(sFrom.c_str());
			To = atoi(sTo.c_str());;
		}
		if(From <1)
			From = 1;
		if(To <1)
			To = 1;
		if(To > Object->GetSize())
			To = Object->GetSize();
		if(To < From)
			To = From;
	}

	// Real index starts from 0
	From--;
	To--;

	string Separator = Command->m_Parameters.GetValue("SEPARATOR");
	CleanString(Separator);
	MakeUpper(Separator);
	if(!Separator.length())
		Separator = ";";


	GetStringValue(Command, Object->GetValues(From, To, Separator).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}


//OBJECT STRINGLIST_SEARCH
bool NvRequest::ObjStringListSearch(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjStringList* Object = (NvObjStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_STRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	string Value = Command->m_Parameters.GetValue("VALUE");
	CleanString(Value);
	MakeUpper(Value);

	bool bCase = false;
	string sCase = Command->m_Parameters.GetValue("CASE");
	CleanString(sCase);
	MakeUpper(sCase);
	if(sCase == "YES")
		bCase = true;

	bool bFirst = true;
	string sFirst = Command->m_Parameters.GetValue("FIRST_ONLY");
	CleanString(sFirst);
	MakeUpper(sFirst);
	if(sFirst == "YES")
		bFirst = false;
	
	GetStringValue(Command, Object->Search(Value, bCase, bFirst).c_str(), Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}









//OBJECT INDSTRINGLIST_SET_VALUE
bool NvRequest::ObjIndStringListSetValue(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object value that should be in command parameters
	string ObjValue = Command->m_Parameters.GetValue("VALUE");

	// Get object key that should be in command parameters
	string ObjKey = Command->m_Parameters.GetValue("KEY");
	CleanString(ObjKey);
//	MakeUpper(ObjKey);

	string KeySep = ";";
	string ValSep = "=";
	if(!ObjKey.length())
	{
		string KeySepHere = Command->m_Parameters.GetValue("KEYSEP");
		CleanString(KeySepHere);
		if(KeySepHere.length())
			KeySep = KeySepHere;
		string ValSepHere = Command->m_Parameters.GetValue("VALSEP");
		CleanString(ValSepHere);
		if(ValSepHere.length())
			ValSep = ValSepHere;
	}


	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->SetValue(ObjKey, ObjValue, KeySep, ValSep);

	return true;
}


//OBJECT INDSTRINGLIST_GET_VALUE
bool NvRequest::ObjIndStringListGetValue(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object key that should be in command parameters
	string ObjKey = Command->m_Parameters.GetValue("KEY");
	CleanString(ObjKey);
//	MakeUpper(ObjKey);
	if(!ObjKey.length())
	{
		if(Command->m_Parameters.KeyExist("INDEX"))
		{
			int Index = 0;
			string ObjKeyIndex = Command->m_Parameters.GetValue("INDEX");
			CleanString(ObjKeyIndex);
			MakeUpper(ObjKeyIndex);
			if(ObjKeyIndex.length())
				Index = atoi(ObjKeyIndex.c_str());
			if(Index < 1)
				Index = 1;
			// The real index starts at 0
			Index--;
			ObjKey = Object->GetKey(Index);
		}
		if(!ObjKey.length())
		{
			GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_INDSTRINGLIST_KEY, ObjName);
			return false;
		}
	}


	if(!Object->KeyExist(ObjKey))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_INDSTRINGLIST_KEY, ObjName);
		return false;
	}

	GetStringValue(Command, Object->GetValue(ObjKey).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT INDSTRINGLIST_GET_VALUES
bool NvRequest::ObjIndStringListGetValues(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);
	
	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error
	
	string KeySep = Command->m_Parameters.GetValue("KEYSEP");
	CleanString(KeySep);
	if(!KeySep.length())
		KeySep = ";";
	string ValSep = Command->m_Parameters.GetValue("VALSEP");
	CleanString(ValSep);
	if(!ValSep.length())
		ValSep = "=";

	// Get output mode
	string Mode = Command->m_Parameters.GetValue("MODE");
	CleanString(Mode, true);
	bool bModeVariables = true;
	if(Mode == "STRING")
		bModeVariables = false;

	// Get eventual variable prefix
	string Prefix = Command->m_Parameters.GetValue("PREFIX");
	CleanString(Prefix, true);

	if(bModeVariables)
	{
		map<string,string>::iterator pos;
		for(pos = Object->m_mList.begin(); pos != Object->m_mList.end(); ++pos)
		{
			string VarName = Prefix;
			VarName += pos->first;
			m_oVariables.SetValue(VarName, pos->second);
		}
	}
	else
	{
		string Value = "";
		map<string,string>::iterator pos;
		int i=0;
		for(i=0,pos = Object->m_mList.begin(); pos != Object->m_mList.end(); ++pos,i++)
		{
			if(i!=0)
				Value += KeySep;
			Value += pos->first;
			Value += ValSep;
			Value += pos->second;
		}
		GetStringValue(Command, Value.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	}

	return true;
}


//OBJECT INDSTRINGLIST_GET_SIZE
bool NvRequest::ObjIndStringListGetSize(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	char Buf[50];
	sprintf(Buf, "%d", Object->GetSize());
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT INDSTRINGLIST_REMOVE
bool NvRequest::ObjIndStringListRemove(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object key that should be in command parameters
	string ObjKey = Command->m_Parameters.GetValue("KEY");
	CleanString(ObjKey);
//	MakeUpper(ObjKey);
	bool AllList = false;
	if(!ObjKey.length())
		AllList = true;

	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	if(AllList)
	{
		Object->RemoveAll();
		return true;
	}
	if(!Object->KeyExist(ObjKey))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_INDSTRINGLIST_KEY, ObjName);
		return false;
	}

	Object->Remove(ObjKey);

	return true;
}




//OBJECT INDSTRINGLIST_ADD_INDSTRINGLIST
bool NvRequest::ObjIndStringListAddIndStringList(NvServerCommand *Command)
{
	// Get object source and destination names that should be in command parameters
	string SObjName = Command->m_Parameters.GetValue("SNAME");
	CleanString(SObjName);
	string DObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(DObjName);

	NvObjIndStringList* SourceObject = (NvObjIndStringList*)GetObjectCheckType(SObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	NvObjIndStringList* DestinationObject = (NvObjIndStringList*)GetObjectCheckType(DObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	DestinationObject->Add(SourceObject);

	return true;
}






//OBJECT INDSTRINGLIST_KEY_EXIST
bool NvRequest::ObjIndStringListKeyExist(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object key that should be in command parameters
	string ObjKey = Command->m_Parameters.GetValue("KEY");
	CleanString(ObjKey);
//	MakeUpper(ObjKey);
	if(!ObjKey.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_INVALID_INDSTRINGLIST_KEY, ObjName);
		return false;
	}

	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	
	char Buf[20];
	strcpy(Buf, "NO");
	if(Object->KeyExist(ObjKey))
		strcpy(Buf, "YES");

	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);
	
	return true;
}


//OBJECT INDSTRINGLIST_GET_KEY
bool NvRequest::ObjIndStringListGetKey(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	// Get object index that should be in command parameters
	string ObjKeyIndex = Command->m_Parameters.GetValue("INDEX");
	CleanString(ObjKeyIndex);
	MakeUpper(ObjKeyIndex);
	int Index = 0;
	if(ObjKeyIndex.length())
		Index = atoi(ObjKeyIndex.c_str());
	if(Index < 1)
		Index = 1;

	// The real index starts at 0
	Index--;

	NvObjIndStringList* Object = (NvObjIndStringList*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_INDSTRINGLIST);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string Key = Object->GetKey(Index);

	GetStringValue(Command, Key.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//OBJECT TABLE_SET_DESCRIPTION
bool NvRequest::ObjTableSetDescription(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string ObjDescription = Command->m_Parameters.GetValue("DESCRIPTION");

	Object->SetDescription(ObjDescription);

	return true;
}



//OBJECT TABLE_GET_DESCRIPTION
bool NvRequest::ObjTableGetDescription(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(Command, Object->GetDescription().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_CLEAR
bool NvRequest::ObjTableClear(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->Clear();

	return true;
}




//OBJECT TABLE_CLEAR_DATA
bool NvRequest::ObjTableClearData(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->ClearData();

	return true;
}




//OBJECT TABLE_SEARCH
bool NvRequest::ObjTableSearch(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object result table that should be in command parameters
	string ObjResult = Command->m_Parameters.GetValue("RESULT");
	CleanString(ObjResult);

	if(!ObjResult.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SEARCH:RESULT");
		return false;
	}
	
	NvObjTable* Result = (NvObjTable *)(m_cContainer.GetObject(ObjResult.c_str()));

	if(Object != NULL)
	{
		// The object exists so we remove it
		m_cContainer.RemoveObject(ObjResult.c_str());
		Object = NULL;
	}
		
	if(!m_cContainer.CreateObject(ObjResult.c_str(), NV_OBJTYPE_TABLE))
	{
		Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_CREATE, ObjResult);
		return false;
	}
	
	Result = (NvObjTable *)(m_cContainer.GetObject(ObjResult.c_str()));

	string Query = Command->m_Parameters.GetValue("QUERY");
	CleanString(Query);
	if(!Query.length())
		Query = "*";		// Default query is all

	string Sort = Command->m_Parameters.GetValue("SORT");
	CleanString(Sort);

	int iMaxDocs;
	string sMaxDocs = Command->m_Parameters.GetValue("MAXDOCS");
	CleanString(sMaxDocs);
	if(sMaxDocs.length())
		iMaxDocs = atoi(sMaxDocs.c_str());

	string ColumnsHere = Command->m_Parameters.GetValue("COLUMNS");
	CleanString(ColumnsHere, true);

	bool RowIndexCol = false;
	string sRowIndexCol = Command->m_Parameters.GetValue("WITH_ROW_INDEX");
	CleanString(sRowIndexCol, true);
	MakeUpper(sRowIndexCol);
	if(sRowIndexCol == "YES")
		RowIndexCol = true;

	int iNumDocs = Object->Search(Query, Result, Sort, iMaxDocs, ColumnsHere, RowIndexCol);
	if(iNumDocs == -1)
	{
		char *ErrorMessage = (char *)malloc(Query.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), Query.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_SEARCH_ERROR, ErrorMessage);
		free(ErrorMessage);
	}

	char Buf[50];
	sprintf(Buf, "%d", iNumDocs);
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	if(iNumDocs == -1)
		return false;

	return true;
}




//OBJECT TABLE_SORT
bool NvRequest::ObjTableSort(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string Sort = Command->m_Parameters.GetValue("SORT");
	CleanString(Sort);

	string Natural = Command->m_Parameters.GetValue("NATURAL");
	CleanString(Natural);
	MakeUpper(Natural);
	bool bNatural = false;
	if(Natural == "YES")
		bNatural = true;

	Object->Sort(Sort, bNatural);

	return true;
}




//OBJECT TABLE_CREATE_FROM
bool NvRequest::ObjTableCreateFrom(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object name that should be in command parameters
	string SourceObjName = Command->m_Parameters.GetValue("SNAME");
	CleanString(SourceObjName);
	if(!SourceObjName.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_CREATE_FROM:SNAME");
		return false;
	}

	NvObjTable* SourceObject = (NvObjTable*)GetObjectCheckType(SourceObjName.c_str(), NV_OBJTYPE_TABLE);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->CreateFrom(SourceObject);	
	
	return true;
}




//OBJECT TABLE_SAVE
bool NvRequest::ObjTableSave(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string FileName = Command->m_Parameters.GetValue("FILENAME");
	if(!FileName.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SAVE:FILENAME");
		return false;
	}

	if(!Object->Save(FileName))
	{
		char *ErrorMessage = (char *)malloc(FileName.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_SAVE, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//OBJECT TABLE_LOAD
bool NvRequest::ObjTableLoad(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string FileName = Command->m_Parameters.GetValue("FILENAME");
	if(!FileName.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_LOAD:FILENAME");
		return false;
	}

	if(!Object->Load(FileName))
	{
		char *ErrorMessage = (char *)malloc(FileName.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_LOAD, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//OBJECT TABLE_IMPORT
bool NvRequest::ObjTableImport(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string FileName = Command->m_Parameters.GetValue("FILENAME");
	if(!FileName.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_IMPORT:FILENAME");
		return false;
	}


	bool WithLf = false;
	string sWithLf = Command->m_Parameters.GetValue("WITH_LF");
	CleanString(sWithLf);
	MakeUpper(sWithLf);
	if(sWithLf == "YES")
		WithLf = true;

	bool Append = false;
	string sAppend = Command->m_Parameters.GetValue("APPEND");
	CleanString(sAppend);
	MakeUpper(sAppend);
	if(sAppend == "YES")
		Append = true;

	// Get eventual column separator
	string ColSep = Command->m_Parameters.GetValue("COLSEP");
	if(!ColSep.length())
		ColSep = ";";

	// Get eventual column separator
	string LineSep = Command->m_Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";

	// Get eventual offset value
	string sOffset = Command->m_Parameters.GetValue("OFFSET");
	int iOffset = 0;
	if(sOffset.length())
		iOffset = atoi(sOffset.c_str());

	// Get eventual skip lines value
	string sSkipLines = Command->m_Parameters.GetValue("SKIPLINES");
	int iSkipLines = 0;
	if(sSkipLines.length())
		iSkipLines = atoi(sSkipLines.c_str());

	// Get eventual line length
	string sMaxLineLength = Command->m_Parameters.GetValue("LINE_LENGTH");
	int iMaxLineLength = 1024;
	if(sMaxLineLength.length())
		iMaxLineLength = atoi(sMaxLineLength.c_str());
	if(iMaxLineLength < 1024)
		iMaxLineLength = 1024;

	if(!Object->Import(FileName, Append, ColSep.c_str(), LineSep.c_str(), iMaxLineLength, iOffset, iSkipLines, WithLf))
	{
		char *ErrorMessage = (char *)malloc(FileName.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_IMPORT, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//OBJECT TABLE_EXPORT
bool NvRequest::ObjTableExport(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string FileName = Command->m_Parameters.GetValue("FILENAME");
	if(!FileName.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_EXPORT:FILENAME");
		return false;
	}

	bool WithLf = false;
	string sWithLf = Command->m_Parameters.GetValue("WITH_LF");
	CleanString(sWithLf);
	MakeUpper(sWithLf);
	if(sWithLf == "YES")
		WithLf = true;

	// Get eventual column separator
	string ColSep = Command->m_Parameters.GetValue("COLSEP");
	if(!ColSep.length())
		ColSep = ";";

	// Get eventual column separator
	string LineSep = Command->m_Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";


	// Get eventual line length
	string sMaxLineLength = Command->m_Parameters.GetValue("LINE_LENGTH");
	int iMaxLineLength = 1024;
	if(sMaxLineLength.length())
		iMaxLineLength = atoi(sMaxLineLength.c_str());
	if(iMaxLineLength < 1024)
		iMaxLineLength = 1024;

	if(!Object->Export(FileName, ColSep.c_str(), LineSep.c_str(), iMaxLineLength, WithLf))
	{
		char *ErrorMessage = (char *)malloc(FileName.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_IMPORT, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}






//OBJECT TABLE_SELECT_FROM
bool NvRequest::ObjTableSelectFrom(NvServerCommand *Command)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object name that should be in command parameters
	string ObjSourceName = Command->m_Parameters.GetValue("SOURCE");
	CleanString(ObjSourceName);

	NvObjTable* SourceObject = (NvObjTable*)GetObjectCheckType(ObjSourceName.c_str(), NV_OBJTYPE_TABLE);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string SelectedYes = "YES";
	if(Command->m_Parameters.KeyExist("SELECTED_YES"))
		SelectedYes = Command->m_Parameters.GetValue("SELECTED_YES");

	string SelectedNo = "NO";
	if(Command->m_Parameters.KeyExist("SELECTED_NO"))
		SelectedNo = Command->m_Parameters.GetValue("SELECTED_NO");

	string sSourceCol = Command->m_Parameters.GetValue("SOURCE_COL");
	CleanString(sSourceCol);
	string sSourceColName = Command->m_Parameters.GetValue("SOURCE_COLNAME");
	CleanString(sSourceColName);
	if((!sSourceCol.length()) && (!sSourceColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SELECT_FROM:SOURCE_COL");
		return false;
	}
	int source_col_index = atoi(sSourceCol.c_str());
	source_col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sSourceColName.length())
		source_col_index = SourceObject->GetColumnIndex(sSourceColName);

	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SELECT_FROM:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	string sSelectedCol = Command->m_Parameters.GetValue("SELECTED_COL");
	CleanString(sSelectedCol);
	string sSelectedColName = Command->m_Parameters.GetValue("SELECTED_COLNAME");
	CleanString(sSelectedColName);
	if((!sSelectedCol.length()) && (!sSelectedColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SELECT_FROM:SELECTED_COL");
		return false;
	}
	int selected_col_index = atoi(sSelectedCol.c_str());
	selected_col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sSelectedColName.length())
		selected_col_index = Object->GetColumnIndex(sSelectedColName);
	

	bool bCaseSensitive = true;
	string sCaseSensitive = Command->m_Parameters.GetValue("CASE_SENSITIVE");
	CleanString(sCaseSensitive);
	MakeUpper(sCaseSensitive);
	if(sCaseSensitive == "NO")
		bCaseSensitive = false;


	if(!Object->SelectFrom(SourceObject, source_col_index, col_index, selected_col_index, SelectedYes.c_str(), SelectedNo.c_str(), bCaseSensitive))
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_SELECT_FROM, ObjName);
		return false;
	}

	return true;
}




//OBJECT TABLE_JOIN
bool NvRequest::ObjTableJoin(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object name that should be in command parameters
	string ObjSourceName = Command->m_Parameters.GetValue("SNAME");
	CleanString(ObjSourceName);

	NvObjTable* SourceObject = (NvObjTable*)GetObjectCheckType(ObjSourceName.c_str(), NV_OBJTYPE_TABLE);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string sForeign = Command->m_Parameters.GetValue("FOREIGN");
	CleanString(sForeign);
	if(!sForeign.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_JOIN:FOREIGN");
		return false;
	}

	string sColumns = Command->m_Parameters.GetValue("COLUMNS");
	CleanString(sColumns, true);
	
	bool bReplace = false;
	string sReplace = Command->m_Parameters.GetValue("REPLACE");
	CleanString(sReplace);
	MakeUpper(sReplace);
	if(sReplace == "YES")
		bReplace = true;


	if(!Object->Join(SourceObject, sColumns.c_str(), sForeign.c_str(), bReplace))
	{
		string ErrorInfo = ObjName;
		ErrorInfo += ";";
		ErrorInfo += ObjSourceName;
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_TABLE_CANNOT_JOIN, ErrorInfo);
		return false;
	}

	return true;
}


//OBJECT TABLE_ADD_TABLE
bool NvRequest::ObjTableAddTable(NvServerCommand *Command)
{	
	// Get object source and destination names that should be in command parameters
	string SObjName = Command->m_Parameters.GetValue("SNAME");
	CleanString(SObjName);
	string DObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(DObjName);

	NvObjTable* SourceObject = (NvObjTable*)GetObjectCheckType(SObjName.c_str(), NV_OBJTYPE_TABLE);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	NvObjTable* DestinationObject = (NvObjTable*)GetObjectCheckType(DObjName.c_str(), NV_OBJTYPE_TABLE);

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	
	int iMode = ADD_MODE_NORMAL;
	string sMode = Command->m_Parameters.GetValue("MODE");
	CleanString(sMode);
	MakeUpper(sMode);
	if(sMode == "REPLACE")
		iMode = ADD_MODE_REPLACE;
	if(sMode == "NO_REPLACE")
		iMode = ADD_MODE_NOREPLACE;

	if(!DestinationObject->Add(SourceObject, iMode))
	{
		string ErrorInfo = SObjName;
		ErrorInfo += ";";
		ErrorInfo += DObjName;
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_TABLE_CANNOT_ADD, ErrorInfo);
		return false;
	}

	return true;
}


//OBJECT TABLE_CLEAR_CELL
bool NvRequest::ObjTableClearCell(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_CLEAR_CELL:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	Object->ClearCell(row_index, col_index);

	return true;
}




//OBJECT TABLE_SORT_CELL
bool NvRequest::ObjTableSortCell(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SORT_CELL:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	bool Ascending = false;
	string sAscending = Command->m_Parameters.GetValue("ASCENDING");
	CleanString(sAscending);
	MakeUpper(sAscending);
	if(sAscending == "NO")
		Ascending = false;

	string Natural = Command->m_Parameters.GetValue("NATURAL");
	CleanString(Natural);
	MakeUpper(Natural);
	bool bNatural = false;
	if(Natural == "YES")
		bNatural = true;

	Object->SortCell(row_index, col_index, Ascending, bNatural);

	return true;
}




//OBJECT TABLE_GET_CELL_NUM_LINES
bool NvRequest::ObjTableGetCellNumLines(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_GET_CELL_NUM_LINES:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	int NumCellLines = Object->GetNumCellLines(row_index, col_index);
	if(NumCellLines == -1)
	{
		char *ErrorMessage = (char *)malloc(sRow.length() + sCol.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - row:%s - col:%s", ObjName.c_str(), sRow.c_str(), sCol.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_INVALID_CELL, ErrorMessage);
		free(ErrorMessage);
	}

	char Buf[50];
	sprintf(Buf, "%d", NumCellLines);
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	if(NumCellLines == -1)
		return false;

	return true;
}




//OBJECT TABLE_GET_CELL_LINE
bool NvRequest::ObjTableGetCellLine(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_GET_CELL_LINE:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	// Get object value that should be in command parameters
	int line_index = 1;
	string sLineIndex = Command->m_Parameters.GetValue("LINE_INDEX");
	CleanString(sLineIndex);
	if(sLineIndex.length())
		line_index = atoi(sLineIndex.c_str());
	line_index--;	// The real index starts at 0

	// Get object name that should be in command parameters
	string sDefault = Command->m_Parameters.GetValue("DEFAULT");

	GetStringValue(Command, Object->GetCellLine(row_index, col_index, line_index, sDefault).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_SET_CELL_LINE
bool NvRequest::ObjTableSetCellLine(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SET_CELL_LINE:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	// Get object value that should be in command parameters
	int line_index = 1;
	string sLineIndex = Command->m_Parameters.GetValue("LINE_INDEX");
	CleanString(sLineIndex);
	if(sLineIndex.length())
		line_index = atoi(sLineIndex.c_str());
	line_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	string Line = Command->m_Parameters.GetValue("LINE");

	if(!Object->SetCellLine(row_index, col_index, Line, line_index))
	{
		char *ErrorMessage = (char *)malloc(sRow.length() + sCol.length() + sLineIndex.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - row:%s - col:%s - line:%s", ObjName.c_str(), sRow.c_str(), sCol.c_str(), sLineIndex.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_SET_CELL_LINE, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	
	return true;
}





//OBJECT TABLE_GET_CELL
bool NvRequest::ObjTableGetCell(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_GET_CELL:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	// Get eventual line separator
	string LineSep = Command->m_Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = ";";

	int i;
	int NumCells = Object->GetNumCellLines(row_index, col_index);
	string Value = "";
	for(i=0;i<NumCells;i++)
	{
		if(i!=0)
			Value += LineSep;
		Value += Object->GetCellLine(row_index, col_index, i);
	}

	GetStringValue(Command, Value.c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//OBJECT TABLE_SET_CELL
bool NvRequest::ObjTableSetCell(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_GET_CELL:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	// Get eventual line separator
	string LineSep = Command->m_Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = ";";

	// Get data
	string data = Command->m_Parameters.GetValue("DATA");
	
	int LineSeparatorLength = (int)LineSep.length();
	string::size_type index;


	bool bPrimaryColumn = false;
	string PrimaryColumnName;
	bool PrimaryCaseSensitive;
	sColName = Object->GetColumnName(col_index);
	Object->GetPrimaryInfo(PrimaryColumnName, PrimaryCaseSensitive);

	if(PrimaryColumnName == sColName)
		bPrimaryColumn = true;

	string line = "";
	Object->ClearCell(row_index, col_index);
	while(1)
	{
		if((index = data.find(LineSep)) == string::npos)
		{
			if(data.length())
				Object->InsertCellLine(row_index, col_index, data);
			break;
		}

		line = data.substr(0, index);
		data = data.substr(index+LineSeparatorLength);
		if(line.length())
		{
			Object->InsertCellLine(row_index, col_index, line);
		}
		if(bPrimaryColumn)	// only one line allowed for a primary colomn
			break;
	}

	return true;
}




//OBJECT TABLE_INSERT_CELL_LINE
bool NvRequest::ObjTableInsertCellLine(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_INSERT_CELL_LINE:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	// Get object value that should be in command parameters
	int line_index = 0;
	string sLineIndex = Command->m_Parameters.GetValue("LINE_INDEX");
	CleanString(sLineIndex);
	if(sLineIndex.length())
		line_index = atoi(sLineIndex.c_str());
	line_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	string Line = Command->m_Parameters.GetValue("LINE");

	if(!Object->InsertCellLine(row_index, col_index, Line, line_index))
	{
		char *ErrorMessage = (char *)malloc(sRow.length() + sCol.length() + sLineIndex.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - row:%s - col:%s - line:%s", ObjName.c_str(), sRow.c_str(), sCol.c_str(), sLineIndex.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_INSERT_CELL_LINE, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	
	return true;
}




//OBJECT TABLE_REMOVE_CELL_LINE
bool NvRequest::ObjTableRemoveCellLine(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_REMOVE_CELL_LINE:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	// Get object value that should be in command parameters
	int line_index = 1;
	string sLineIndex = Command->m_Parameters.GetValue("LINE_INDEX");
	CleanString(sLineIndex);
	if(sLineIndex.length())
		line_index = atoi(sLineIndex.c_str());
	line_index--;	// The real index starts at 0

	Object->RemoveCellLine(row_index, col_index, line_index);
	
	return true;
}




//OBJECT TABLE_SWAP_CELL_LINES
bool NvRequest::ObjTableSwapCellLines(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SWAP_CELL_LINES:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	// Get object value that should be in command parameters
	string sLineIndex1 = Command->m_Parameters.GetValue("LINE_INDEX1");
	CleanString(sLineIndex1);
	if(!sLineIndex1.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SWAP_CELL_LINES:LINE_INDEX1");
		return false;
	}
	int line_index1 = atoi(sLineIndex1.c_str());
	line_index1--;	// The real index starts at 0

	// Get object value that should be in command parameters
	string sLineIndex2 = Command->m_Parameters.GetValue("LINE_INDEX2");
	CleanString(sLineIndex2);
	if(!sLineIndex2.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SWAP_CELL_LINES:LINE_INDEX2");
		return false;
	}
	int line_index2 = atoi(sLineIndex2.c_str());
	line_index2--;	// The real index starts at 0

	Object->SwapCellLines(row_index, col_index, line_index1, line_index2);
	
	return true;
}




//OBJECT TABLE_GET_NUM_ROWS
bool NvRequest::ObjTableGetNumRows(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	int NumRows = Object->GetNumRow();

	char Buf[50];
	sprintf(Buf, "%d", NumRows);
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_REMOVE_ROW
bool NvRequest::ObjTableRemoveRow(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	Object->RemoveRow(row_index);
	
	return true;
}




//OBJECT TABLE_REMOVE_ROWS
bool NvRequest::ObjTableRemoveRows(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string Query = Command->m_Parameters.GetValue("QUERY");
	CleanString(Query);
	if(!Query.length())
		Query = "*";		// Default query is all

	if(!Object->RemoveRows(Query))
	{
		char *ErrorMessage = (char *)malloc(Query.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), Query.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_REMOVE_ROWS_ERROR, ErrorMessage);
		free(ErrorMessage);
	}

	return true;
}




//OBJECT TABLE_CLEAR_ROW
bool NvRequest::ObjTableClearRow(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	Object->ClearRow(row_index);
	
	return true;
}




//OBJECT TABLE_CLEAR_ROWS
bool NvRequest::ObjTableClearRows(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string Query = Command->m_Parameters.GetValue("QUERY");
	CleanString(Query);
	if(!Query.length())
		Query = "*";		// Default query is all

	if(!Object->ClearRows(Query))
	{
		char *ErrorMessage = (char *)malloc(Query.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), Query.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CLEAR_ROWS_ERROR, ErrorMessage);
		free(ErrorMessage);
	}

	return true;
}




//OBJECT TABLE_INSERT_ROWS
bool NvRequest::ObjTableInsertRows(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0


	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	// Get object value that should be in command parameters
	int num_row = 1;
	string sNumRow = Command->m_Parameters.GetValue("NUMROW");
	CleanString(sNumRow);
	if(sNumRow.length())
		num_row = atoi(sNumRow.c_str());

	Object->InsertRows(num_row, row_index);
	
	return true;
}




//OBJECT TABLE_SWAP_ROWS
bool NvRequest::ObjTableSwapRows(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sRow1 = Command->m_Parameters.GetValue("ROW1");
	CleanString(sRow1);
	if(!sRow1.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SWAP_ROWS:ROW1");
		return false;
	}
	int row_index1 = atoi(sRow1.c_str());
	row_index1--;	// The real index starts at 0

	// Get object value that should be in command parameters
	string sRow2 = Command->m_Parameters.GetValue("ROW2");
	CleanString(sRow2);
	if(!sRow2.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SWAP_ROWS:ROW2");
		return false;
	}
	int row_index2 = atoi(sRow2.c_str());
	row_index2--;	// The real index starts at 0

	Object->SwapRows(row_index1, row_index2);

	return true;
}



//OBJECT TABLE_GET_ROWS
bool NvRequest::ObjTableGetRows(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object result table that should be in command parameters
	string ObjResult = Command->m_Parameters.GetValue("RESULT");
	CleanString(ObjResult);

	if(!ObjResult.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SEARCH:RESULT");
		return false;
	}
	
	NvObjTable* Result = (NvObjTable *)(m_cContainer.GetObject(ObjResult.c_str()));

	if(Object != NULL)
	{
		// The object exists so we remove it
		m_cContainer.RemoveObject(ObjResult.c_str());
		Object = NULL;
	}
		
	if(!m_cContainer.CreateObject(ObjResult.c_str(), NV_OBJTYPE_TABLE))
	{
		Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_CREATE, ObjResult);
		return false;
	}
	
	Result = (NvObjTable *)(m_cContainer.GetObject(ObjResult.c_str()));

	string First = Command->m_Parameters.GetValue("FIRST");
	CleanString(First);
	if(!First.length())
		First = "1";		// Default to first row
	int iFirst = atoi(First.c_str());
	iFirst--; // starts in fact at 0
	if(iFirst <= 0)
		iFirst = 0;

	string Last = Command->m_Parameters.GetValue("LAST");
	CleanString(Last);
	if(!Last.length())
		Last = "0";		// Default to last row
	int iLast = atoi(Last.c_str());
	iLast--; // starts in fact at 0
	
	string Page = Command->m_Parameters.GetValue("PAGE");
	CleanString(Page);
	if(Page.length())
	{
		int iPage = atoi(Page.c_str());
		if(iPage < 1)
			iPage = 1;
		string PageSize = Command->m_Parameters.GetValue("PAGE_SIZE");
		int iPageSize = 10;
		CleanString(PageSize);
		if(PageSize.length())
			iPageSize = atoi(PageSize.c_str());
		if(iPageSize <= 0)
			First = -1;
		else
		{
			int NumRows = Object->GetNumRow();
			iFirst = (iPage-1)*iPageSize;
			iLast = iFirst + iPageSize-1;
		}
	}

	string ColumnsHere = Command->m_Parameters.GetValue("COLUMNS");
	CleanString(ColumnsHere, true);
	int iNumDocs = Object->GetRows(iFirst, iLast, Result, ColumnsHere.c_str());

	char Buf[50];
	sprintf(Buf, "%d", iNumDocs);
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}





//OBJECT TABLE_GET_ROW
bool NvRequest::ObjTableGetRow(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}


	// Get eventual column separator
	string ColSep = Command->m_Parameters.GetValue("COLSEP");
	if(!ColSep.length())
		ColSep = ";";


	// Get output mode
	string Mode = Command->m_Parameters.GetValue("MODE");
	CleanString(Mode, true);
	bool bModeVariables = true;
	if(Mode == "STRING")
		bModeVariables = false;

	// Get eventual line separator
	string LineSep = Command->m_Parameters.GetValue("LINESEP");
	if(!LineSep.length())
	{
		if(bModeVariables)
			LineSep = ";";
		else
			LineSep = "|";
	}


	// Get eventual variable prefix
	string Prefix = Command->m_Parameters.GetValue("PREFIX");
	CleanString(Prefix, true);


	int NumColumns = Object->GetNumColumn();
	int i;
	if(bModeVariables)
	{
		for(i=0;i<NumColumns;i++)
		{
			string ColumnName = Prefix;
			ColumnName += Object->GetColumnName(i);
			m_oVariables.SetValue(ColumnName, "");
		}
		int NumRows = Object->GetNumRow();
		if((row_index+1) > NumRows)
			return true;

		for(i=0;i<NumColumns;i++)
		{
			string ColumnName = Prefix;
			ColumnName += Object->GetColumnName(i);
			int NumCells = Object->GetNumCellLines(row_index, i);
			int j;
			string Value = "";
			for(j=0;j<NumCells;j++)
			{
				if(j!=0)
					Value += LineSep;
				Value += Object->GetCellLine(row_index, i, j);
			}
			m_oVariables.SetValue(ColumnName, Value);
		}
	}
	else
	{
		int NumRows = Object->GetNumRow();
		if((row_index+1) > NumRows)
			return true;

		string Value = "";
		for(i=0;i<NumColumns;i++)
		{
			if(i!=0)
				Value += ColSep;
			int NumCells = Object->GetNumCellLines(row_index, i);
			int j;
			for(j=0;j<NumCells;j++)
			{
				if(j!=0)
					Value += LineSep;
				Value += Object->GetCellLine(row_index, i, j);
			}
		}
		GetStringValue(Command, Value.c_str(), Buffer, BufferSize, DataSize, iEncoding);
	}

	return true;
}




//OBJECT TABLE_GET_COLUMN
bool NvRequest::ObjTableGetColumn(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_GET_COLUMN:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	// Get eventual row separator
	string RowSep = Command->m_Parameters.GetValue("ROWSEP");
	// there was a bug in first release of this command using COLSEP instead of ROWSEP so we must assume ascending compatibility
	if(!RowSep.length())
		RowSep = Command->m_Parameters.GetValue("COLSEP");
	if(!RowSep.length())
		RowSep = ";";

	// Get eventual line separator
	string LineSep = Command->m_Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";

	int i;
	int NumRows = Object->GetNumRow();
	int LengthRowSep = (int)RowSep.length();
	int LengthLineSep = (int)LineSep.length();


	// first count size of result
	int TotalSize = 0;
	for(i=0;i<NumRows;i++)
	{
		if(i!=0)
			TotalSize += LengthRowSep;
		int NumCells = Object->GetNumCellLines(i, col_index);
		int j;
		for(j=0;j<NumCells;j++)
		{
			if(j!=0)
				TotalSize += LengthLineSep;
			TotalSize += (int)(Object->GetCellLine(i, col_index, j).length());
		}
	}


	string Value = "";
	Value.reserve(TotalSize + 10);
	for(i=0;i<NumRows;i++)
	{
		if(i!=0)
			Value += RowSep;
		int NumCells = Object->GetNumCellLines(i, col_index);
		int j;
		for(j=0;j<NumCells;j++)
		{
			if(j!=0)
				Value += LineSep;
			Value += Object->GetCellLine(i, col_index, j);
		}
	}
	GetStringValue(Command, Value.c_str(), Buffer, BufferSize, DataSize, iEncoding);


	return true;
}




//OBJECT TABLE_SET_COLUMN
bool NvRequest::ObjTableSetColumn(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_GET_COLUMN:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	// Get eventual row separator
	string RowSep = Command->m_Parameters.GetValue("ROWSEP");
	// there was a bug in first release of this command using COLSEP instead of ROWSEP so we must assume ascending compatibility
	if(!RowSep.length())
		RowSep = Command->m_Parameters.GetValue("COLSEP");
	if(!RowSep.length())
		RowSep = ";";

	// Get eventual line separator
	string LineSep = Command->m_Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";

	// Get data
	string data = Command->m_Parameters.GetValue("DATA");
	
	bool bPrimaryColumn = false;
	string PrimaryColumnName;
	bool PrimaryCaseSensitive;
	sColName = Object->GetColumnName(col_index);
	Object->GetPrimaryInfo(PrimaryColumnName, PrimaryCaseSensitive);
	if(PrimaryColumnName == sColName)
	{
		bPrimaryColumn = true;
		Object->SetPrimaryColumn("", false);
	}

	int i;
	int RowSeparatorLength = (int)RowSep.length();
	int LineSeparatorLength = (int)LineSep.length();
	string::size_type index;
	int NumRows = Object->GetNumRow();
	string cell_value = "";
	string line = "";
	for(i=0;i<NumRows;i++)
	{
		Object->ClearCell(i, col_index);
		index = data.find(RowSep);
		if(index == string::npos)
		{
			cell_value = data;
			data = "";
		}
		else
		{
			cell_value = data.substr(0, index);
			data = data.substr(index+RowSeparatorLength);
		}

		while(1)
		{
			if((index = cell_value.find(LineSep)) == string::npos)
			{
				if(cell_value.length())
					Object->InsertCellLine(i, col_index, cell_value);
				break;
			}

			line = cell_value.substr(0, index);
			cell_value = cell_value.substr(index+LineSeparatorLength);
			if(line.length())
			{
				Object->InsertCellLine(i, col_index, line);
			}
			if(bPrimaryColumn)	// only one line allowed for a primary colomn
				break;
		}
	}

	if(bPrimaryColumn)
	{
		if(!Object->SetPrimaryColumn(sColName, PrimaryCaseSensitive))
		{
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_SET_PRIMARY, sColName);
			return false;
		}
	}

	return true;
}



//OBJECT TABLE_GET_NUM_COLUMNS
bool NvRequest::ObjTableGetNumColums(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	int NumCols = Object->GetNumColumn();

	char Buf[50];
	sprintf(Buf, "%d", NumCols);
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_GET_COLUMN_NAME
bool NvRequest::ObjTableGetColumnName(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	if(!sCol.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_GET_COLUMN_NAME:COL");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	GetStringValue(Command, Object->GetColumnName(col_index).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_GET_COLUMN_DESCRIPTION
bool NvRequest::ObjTableGetColumnDescription(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_GET_COLUMN_DESCRIPTION:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	GetStringValue(Command, Object->GetColumnDesc(col_index).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_IS_COLUMN_NUMERIC
bool NvRequest::ObjTableIsColumnNumeric(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_IS_COLUMN_NUMERIC:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	char Buf[20];
	strcpy(Buf, "NO");
	if(Object->IsNumericColumn(col_index))
		strcpy(Buf, "YES");

	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_GET_COLUMN_INDEX
bool NvRequest::ObjTableGetColumnIndex(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if(!sColName.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_GET_COLUMN_INDEX:COLNAME");
		return false;
	}

	int ColIndex = Object->GetColumnIndex(sColName);
	if(ColIndex == -1)
	{
		char *ErrorMessage = (char *)malloc(sColName.length() + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), sColName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_BAD_COLUMN_NAME, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	// The real ColumnIndex starts at 0
	ColIndex++;
	char Buf[50];
	sprintf(Buf, "%d", ColIndex);
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT TABLE_CLEAR_COLUMN
bool NvRequest::ObjTableClearColumn(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_CLEAR_COLUMN:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	Object->ClearColumn(col_index);

	return true;
}




//OBJECT TABLE_REMOVE_COLUMN
bool NvRequest::ObjTableRemoveColumn(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_REMOVE_COLUMN:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);


	Object->RemoveColumn(col_index);

	return true;
}




//OBJECT TABLE_INSERT_COLUMN
bool NvRequest::ObjTableInsertColumn(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if(!sColName.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_INSERT_COLUMN:COLNAME");
		return false;
	}

	// Get object value that should be in command parameters
	string sColDesc = Command->m_Parameters.GetValue("DESCRIPTION");

	bool bNumeric = false;
	string sNumeric = Command->m_Parameters.GetValue("NUMERIC");
	CleanString(sNumeric);
	MakeUpper(sNumeric);
	if(sNumeric == "YES")
		bNumeric = true;

	
	// Get object value that should be in command parameters
	int col_index = 0;
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	if(sCol.length())
		col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	Object->InsertColumn(sColName, sColDesc, bNumeric, col_index);



	bool bPrimary = false;
	string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
	CleanString(sPrimary);
	MakeUpper(sPrimary);
	if(sPrimary == "YES")
		bPrimary = true;

	if(bPrimary)
	{
		bool bPrimaryCs = false;
		string sPrimaryCs = Command->m_Parameters.GetValue("PRIMARY_CS");
		CleanString(sPrimaryCs);
		MakeUpper(sPrimaryCs);
		if(sPrimaryCs == "YES")
			bPrimaryCs = true;

		Object->SetPrimaryColumn(sColName, bPrimaryCs);
	}

	return true;
}






//OBJECT TABLE_ADD_COLUMNS
bool NvRequest::ObjTableAddColumns(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string ColumnsHere = Command->m_Parameters.GetValue("COLUMNS");
	if(ColumnsHere.length())
	{
		string::size_type index;
		while(ColumnsHere.length())
		{
			string Column = ColumnsHere;
			string ColumnDesc = "";
			bool ColumnNumeric = false;
			index = Column.find(';');
			if(index != string::npos)
			{
				Column = ColumnsHere.substr(0, index);
				ColumnsHere = ColumnsHere.substr(index+1);
			}
			else
				ColumnsHere = "";
			index = Column.find(':');
			if(index != string::npos)
			{
				ColumnDesc = Column.substr(index+1);
				Column = Column.substr(0, index);
			}
			index = ColumnDesc.find(':');
			if(index != string::npos)
			{
				string sColumnNumeric = ColumnDesc.substr(index+1);
				ColumnDesc = ColumnDesc.substr(0, index);
				CleanString(sColumnNumeric);
				MakeUpper(sColumnNumeric);
				if(sColumnNumeric == "Y")
					ColumnNumeric = true;
			}
			CleanString(Column);
			CleanString(ColumnDesc);
			if(!Column.length())
				continue;
			Object->InsertColumn(Column, ColumnDesc, ColumnNumeric);
		}
	}

	return true;
}


//OBJECT TABLE_MODIFY_COLUMN
bool NvRequest::ObjTableModifyColumn(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_MODIFY_COLUMN:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	string Query = Command->m_Parameters.GetValue("QUERY");
	CleanString(Query);
	if(!Query.length())
		Query = "*";		// Default query is all


	// Get object value that should be in command parameters
	string Value = Command->m_Parameters.GetValue("VALUE");

	if(!Object->ModifyColumn(Query, col_index, Value))
	{
		char *ErrorMessage = (char *)malloc(Query.length() + sCol.length()  + ObjName.length() + 100);
		sprintf(ErrorMessage, "%s - col:%s - %s", ObjName.c_str(), sCol.c_str(), Query.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_MODIFY_COLUMNS_ERROR, ErrorMessage);
		free(ErrorMessage);
		return false;
	}

	return true;
}




//OBJECT TABLE_SWAP_COLUMNS
bool NvRequest::ObjTableSwapColumns(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol1 = Command->m_Parameters.GetValue("COL1");
	CleanString(sCol1);
	if(!sCol1.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SWAP_COLUMNS:COL1");
		return false;
	}
	int col_index1 = atoi(sCol1.c_str());
	col_index1--;	// The real index starts at 0

	// Get object value that should be in command parameters
	string sCol2 = Command->m_Parameters.GetValue("COL2");
	CleanString(sCol2);
	if(!sCol2.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SWAP_COLUMNS:COL2");
		return false;
	}
	int col_index2 = atoi(sCol2.c_str());
	col_index2--;	// The real index starts at 0

	Object->SwapColumns(col_index1, col_index2);

	return true;
}





//OBJECT TABLE_FILTER_COLOMNS
bool NvRequest::ObjTableFilterColumns(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	string ColumnsHere = Command->m_Parameters.GetValue("COLUMNS");
	CleanString(ColumnsHere, true);

	Object->FilterColumns(ColumnsHere.c_str());

	return true;
}



//OBJECT TABLE_SET_PRIMARY_COLUMN
bool NvRequest::ObjTableSetPrimaryColumn(NvServerCommand *Command)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);

	bool bPrimaryCs = false;
	string sPrimaryCs = Command->m_Parameters.GetValue("PRIMARY_CS");
	CleanString(sPrimaryCs);
	MakeUpper(sPrimaryCs);
	if(sPrimaryCs == "YES")
		bPrimaryCs = true;

	if(!Object->SetPrimaryColumn(sColName, bPrimaryCs))
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_SET_PRIMARY, sColName);
		return false;
	}


	return true;
}



//OBJECT TABLE_SET_COLUMN_NAME
bool NvRequest::ObjTableSetColumnName(NvServerCommand *Command)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SET_COLUMN_NAME:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	string sNewName = Command->m_Parameters.GetValue("NEW_NAME");
	CleanString(sNewName);
	if(!sNewName.length())
		return true; // nothing to do

	if(col_index < 0)
		return true; // nothing to do

	if(Object->GetColumnIndex(sNewName) >= 0)
		return true; // nothing to do because the column ever exists

	Object->SetColumnName(col_index, sNewName);

	return true;
}



//OBJECT TABLE_SET_COLUMN_TYPE
bool NvRequest::ObjTableSetColumnType(NvServerCommand *Command)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error


	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SET_COLUMN_TYPE:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	if(col_index < 0)
		return true; // nothing to do

	bool bNumeric = false;
	string sNewType = Command->m_Parameters.GetValue("TYPE");
	CleanString(sNewType);
	MakeUpper(sNewType);
	if(sNewType == "NUMERIC")
		bNumeric = true;

	Object->SetColumnNumeric(col_index, bNumeric);

	return true;
}



//OBJECT TABLE_SET_COLUMN_DESCRIPTION
bool NvRequest::ObjTableSetColumnDescription(NvServerCommand *Command)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sCol = Command->m_Parameters.GetValue("COL");
	CleanString(sCol);
	string sColName = Command->m_Parameters.GetValue("COLNAME");
	CleanString(sColName);
	if((!sCol.length()) && (!sColName.length()))
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_SET_COLUMN_DESCRIPTION:COL;COLNAME");
		return false;
	}
	int col_index = atoi(sCol.c_str());
	col_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if(sColName.length())
		col_index = Object->GetColumnIndex(sColName);

	if(col_index < 0)
		return true; // nothing to do

	string sNewDesc = Command->m_Parameters.GetValue("DESCRIPTION");

	Object->SetColumnDesc(col_index, sNewDesc);

	return true;
}




//OBJECT TABLE_GET_PRIMARY_INFO
bool NvRequest::ObjTableGetPrimaryInfo(NvServerCommand *Command)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	m_cContainer.RemoveObject("PRIMARY_COLUMN");
	m_cContainer.RemoveObject("PRIMARY_COLUMN_CS");
	if(!m_cContainer.CreateObject("PRIMARY_COLUMN", NV_OBJTYPE_STRING))
	{
		Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_CREATE, "PRIMARY_COLUMN");
		return false;
	}
	if(!m_cContainer.CreateObject("PRIMARY_COLUMN_CS", NV_OBJTYPE_BOOLEAN))
	{
		Command->m_Error.SetLocalError("OBJECT", NV_LOCAL_OBJECT_CANNOT_CREATE, "PRIMARY_COLUMN_CS");
		m_cContainer.RemoveObject("PRIMARY_COLUMN");
		return false;
	}

	string sPrimary;
	bool bPrimaryCs;
	Object->GetPrimaryInfo(sPrimary, bPrimaryCs);
	
	NvObj *ObjectResult = m_cContainer.GetObject("PRIMARY_COLUMN_CS");
	((NvObjBoolean *)ObjectResult)->SetValue(bPrimaryCs);

	ObjectResult = m_cContainer.GetObject("PRIMARY_COLUMN");
	((NvObjString *)ObjectResult)->SetValue(sPrimary);

	return true;
}


//OBJECT TABLE_GET_ROW_INDEX
bool NvRequest::ObjTableGetRowIndex(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
	CleanString(sPrimary);
	if(!sPrimary.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:TABLE_GET_ROW_INDEX:PRIMARY");
		return false;
	}

	int RowIndex = Object->GetRowIndex(sPrimary);
//	if(RowIndex == -1)
//	{
//		string ErrorMessage = Object->m_sName + ":" + sPrimary;
//		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
//		return false;
//	}

	// The real RowIndex starts at 0
	RowIndex++;
	char Buf[50];
	sprintf(Buf, "%d", RowIndex);
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}



//OBJECT TABLE_SET_ROW
bool NvRequest::ObjTableSetRow(NvServerCommand *Command)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sData = Command->m_Parameters.GetValue("DATA");

	// Get eventual column separator
	string ColSep = Command->m_Parameters.GetValue("COLSEP");
	if(!ColSep.length())
		ColSep = ";";

	// Get eventual line separator
	string LineSep = Command->m_Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";


	// Get object value that should be in command parameters
	int row_index = 0;	// Default value 0 means last row
	string sRow = Command->m_Parameters.GetValue("ROW");
	CleanString(sRow);
	if(sRow.length())
		row_index = atoi(sRow.c_str());
	row_index--;	// The real index starts at 0

	// Get object value that should be in command parameters
	if((row_index == -1) && (Command->m_Parameters.KeyExist("PRIMARY")))
	{
		string sPrimary = Command->m_Parameters.GetValue("PRIMARY");
		CleanString(sPrimary);
		row_index = Object->GetRowIndex(sPrimary);
		if(row_index == -1)
		{
			string ErrorMessage = Object->m_sName + ":" + sPrimary;
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_GET_ROW_INDEX, ErrorMessage);
			return false;
		}
	}

	if(!Object->SetRowData(row_index, sData.c_str(), ColSep.c_str(), LineSep.c_str()))
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_SET_ROW_DATA, ObjName);
		return false;
	}

	return true;
}





//OBJECT TABLE_ADD_ROWS
bool NvRequest::ObjTableAddRows(NvServerCommand *Command)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjTable* Object = (NvObjTable*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_TABLE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object value that should be in command parameters
	string sData = Command->m_Parameters.GetValue("DATA");

	// Get eventual record separator
	string RecSep = Command->m_Parameters.GetValue("ROWSEP");
	if(!RecSep.length())
		RecSep = "\n";

	// Get eventual column separator
	string ColSep = Command->m_Parameters.GetValue("COLSEP");
	if(!ColSep.length())
		ColSep = ";";

	// Get eventual line separator
	string LineSep = Command->m_Parameters.GetValue("LINESEP");
	if(!LineSep.length())
		LineSep = "|";

	if(!Object->AddRecords(sData.c_str(), RecSep.c_str(), ColSep.c_str(), LineSep.c_str()))
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_TABLE_CANNOT_ADD_RECORDS, ObjName);
		return false;
	}

	return true;
}


//OBJECT FILE_SET_FILENAME
bool NvRequest::ObjFileSetFileName(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string LocalFileInCommand = Command->m_Parameters.GetValue("FILENAME");

	if(LocalFileInCommand.length())
		Object->SetFileName(LocalFileInCommand);
	else
	{
		// If no file name given
		// We try to create automatically a file following some information in the command
		string LocalFileDirInCommand = m_sTempDir;
		if(Command->m_Parameters.KeyExist("DIRECTORY"))
			LocalFileDirInCommand = Command->m_Parameters.GetValue("DIRECTORY");
		CleanString(LocalFileDirInCommand);
		string LocalFileExtensionInCommand = Command->m_Parameters.GetValue("EXTENSION");
		CleanString(LocalFileExtensionInCommand);
		string LocalFilePrefixInCommand = Command->m_Parameters.GetValue("PREFIX");
		CleanString(LocalFilePrefixInCommand);
		string LocalFileSuffixInCommand = Command->m_Parameters.GetValue("SUFFIX");
		CleanString(LocalFileSuffixInCommand);
		if(!Object->SetAutoFileName(LocalFileDirInCommand, LocalFileExtensionInCommand, LocalFilePrefixInCommand, LocalFileSuffixInCommand))
		{
			char *ErrorMessage = (char *)malloc(ObjName.length() + LocalFileDirInCommand.length() + LocalFileExtensionInCommand.length()  + LocalFilePrefixInCommand.length()  + LocalFileSuffixInCommand.length() + 100);
			sprintf(ErrorMessage, "obj:%s - dir:%s - ext:%s - prefix:%s - suffix:%s", ObjName.c_str(), LocalFileDirInCommand.c_str(), LocalFileExtensionInCommand.c_str(), LocalFilePrefixInCommand.c_str(), LocalFileSuffixInCommand.c_str());
			GlobalError.SetLocalError("OBJECT", NV_LOCAL_FILE_CANNOT_CREATE, ErrorMessage);
			free(ErrorMessage);
			return false;
		}
	}
	
	return true;
}







//OBJECT FILE_SET_PERSIST
bool NvRequest::ObjFileSetPersist(NvServerCommand *Command)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string PersistTimeFileInCommand = Command->m_Parameters.GetValue("PERSIST");
	CleanString(PersistTimeFileInCommand);
	MakeUpper(PersistTimeFileInCommand);
	int iPersistTimeFileInCommand = -1;	// The file is persistent by default on the client
	if(PersistTimeFileInCommand.length())
		iPersistTimeFileInCommand = atoi(PersistTimeFileInCommand.c_str());
	// Cached files not authorized locally
	if(iPersistTimeFileInCommand != 0)
		iPersistTimeFileInCommand = -1;

	// If the file has to be persistant
	// we must check that it's not in the application work dir
	Object->SetPersistTime(iPersistTimeFileInCommand);
	
	return true;
}

//OBJECT FILE_GET_PATHNAME
bool NvRequest::ObjFileGetPathName(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(Command, Object->GetFilePathName().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT FILE_GET_DIRNAME
bool NvRequest::ObjFileGetDirName(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(Command, Object->GetFileDir().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT FILE_GET_FILENAME
bool NvRequest::ObjFileGetFileName(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get with extension option
	bool bWithExt = true;
	string sWithExt = Command->m_Parameters.GetValue("WITH_EXT");
	CleanString(sWithExt);
	MakeUpper(sWithExt);
	if(sWithExt == "NO")
		bWithExt = false;


	GetStringValue(Command, Object->GetFileName(bWithExt).c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT FILE_GET_EXTENSION
bool NvRequest::ObjFileGetExtension(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(Command, Object->GetFileExtension().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT FILE_GET_SIZE
bool NvRequest::ObjFileGetSize(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	NvInt64 FileSize = Object->GetSize();
	if(FileSize == -1)
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_FILE_CANNOT_GET_SIZE, ObjName.c_str());
		return false;
	}

	char Buf[50];
	sprintf(Buf, "%llu", FileSize);
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}




//OBJECT FILE_REMOVE
bool NvRequest::ObjFileRemove(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	bool bForce = false;
	string sForce = Command->m_Parameters.GetValue("FORCE");
	CleanString(sForce);
	MakeUpper(sForce);
	if(sForce == "YES")
		bForce = true;

	Object->Remove(bForce);

	return true;
}




//OBJECT FILE_CREATE
bool NvRequest::ObjFileCreate(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	if(!Object->Create())
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_FILE_CANNOT_CREATE, ObjName.c_str());
		return false;
	}
	return true;
}




//OBJECT FILE_CLEAR
bool NvRequest::ObjFileClear(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	if(!Object->Clear())
	{
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_FILE_CANNOT_CLEAR, ObjName.c_str());
		return false;
	}
	return true;
}




//OBJECT FILE_APPEND
bool NvRequest::ObjFileAppend(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object source name that should be in command parameters
	string SourceObjName = Command->m_Parameters.GetValue("SNAME");
	CleanString(SourceObjName);

	NvObjFile* SourceObject = (NvObjFile*)GetObjectCheckType(SourceObjName.c_str(), NV_OBJTYPE_FILE);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Check if an offset and a number of bytes has been requested
	NvInt64 Offset = 0;
	NvInt64 BytesToCopy = -1;
	if(Command->m_Parameters.KeyExist("OFFSET"))
	{
		string sOffset = Command->m_Parameters.GetValue("OFFSET");
		CleanString(sOffset);
		Offset = NvAtoi64(sOffset.c_str());
	}
	if(Command->m_Parameters.KeyExist("NUM_BYTES"))
	{
		string sBytesToCopy = Command->m_Parameters.GetValue("NUM_BYTES");
		CleanString(sBytesToCopy);
		BytesToCopy = NvAtoi64(sBytesToCopy.c_str());
	}

	if(!Object->AppendFrom(SourceObject, Offset, BytesToCopy))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + SourceObjName.length() + 100);
		sprintf(ErrorMessage, "dest:%s - src:%s", ObjName.c_str(), SourceObjName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_FILE_CANNOT_APPEND, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}




//OBJECT FILE_APPEND
bool NvRequest::ObjFileSetAscii(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->SetAscii();

	return true;
}




//OBJECT FILE_EXIST
bool NvRequest::ObjFileExist(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	char Buf[20];
	strcpy(Buf, "NO");
	if(Object->Exist())
		strcpy(Buf, "YES");

	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}






//OBJECT FILE_SET_ORIGIN
bool NvRequest::ObjFileSetOrigin(NvServerCommand *Command)
{

	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	string Origin = Command->m_Parameters.GetValue("ORIGIN");
	CleanString(Origin);

	Object->SetOrigin(Origin);

	return true;
}



//OBJECT FILE_GET_ORIGIN
bool NvRequest::ObjFileGetOrigin(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	GetStringValue(Command, Object->GetOrigin().c_str(), Buffer, BufferSize, DataSize, iEncoding);

	return true;
}





//OBJECT FILE_COMPRESS
bool NvRequest::ObjFileCompress(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object destination name that should be in command parameters
	string DestinationObjName = Command->m_Parameters.GetValue("DNAME");
	CleanString(DestinationObjName);

	NvObjFile* DestinationObject = (NvObjFile*)GetObjectCheckType(DestinationObjName.c_str(), NV_OBJTYPE_FILE);

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get compression mode
	string Mode = Command->m_Parameters.GetValue("MODE");
	CleanString(Mode);
	int CompressMode = NV_OBJ_COMPRESS_HUF;
	if(Mode == "LZMA")
		CompressMode = NV_OBJ_COMPRESS_LZMA;

	if(!Object->Compress(DestinationObject, CompressMode))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + DestinationObjName.length() + 100);
		sprintf(ErrorMessage, "dest:%s - src:%s", DestinationObjName.c_str(), ObjName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_FILE_CANNOT_COMPRESS, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}





//OBJECT FILE_COMPRESS
bool NvRequest::ObjFileDecompress(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjFile* Object = (NvObjFile*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_FILE);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object destination name that should be in command parameters
	string DestinationObjName = Command->m_Parameters.GetValue("DNAME");
	CleanString(DestinationObjName);

	NvObjFile* DestinationObject = (NvObjFile*)GetObjectCheckType(DestinationObjName.c_str(), NV_OBJTYPE_FILE);

	if(DestinationObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	if(!Object->Decompress(DestinationObject))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + DestinationObjName.length() + 100);
		sprintf(ErrorMessage, "dest:%s - src:%s", DestinationObjName.c_str(), ObjName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_FILE_CANNOT_DECOMPRESS, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}


//OBJECT BINARY_GET_SIZE
bool NvRequest::ObjBinaryGetSize(NvServerCommand *Command, char *Buffer, int BufferSize, int *DataSize, int iEncoding)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjBinary* Object = (NvObjBinary*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_BINARY);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	char Buf[50];
	sprintf(Buf, "%d", Object->GetSize());
	GetStringValue(Command, Buf, Buffer, BufferSize, DataSize, iEncoding);

	return true;
}


//OBJECT BINARY_CLEAR
bool NvRequest::ObjBinaryClear(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjBinary* Object = (NvObjBinary*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_BINARY);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	Object->Clear();

	return true;
}


//OBJECT BINARY_APPEND
bool NvRequest::ObjBinaryAppend(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjBinary* Object = (NvObjBinary*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_BINARY);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object source name that should be in command parameters
	string SourceObjName = Command->m_Parameters.GetValue("SNAME");
	CleanString(SourceObjName);

	NvObjBinary* SourceObject = (NvObjBinary*)GetObjectCheckType(SourceObjName.c_str(), NV_OBJTYPE_BINARY);

	if(SourceObject == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Check if an offset and a number of bytes has been requested
	int Offset = 0;
	int BytesToCopy = -1;
	if(Command->m_Parameters.KeyExist("OFFSET"))
	{
		string sOffset = Command->m_Parameters.GetValue("OFFSET");
		CleanString(sOffset);
		Offset = atoi(sOffset.c_str());
	}
	if(Command->m_Parameters.KeyExist("NUM_BYTES"))
	{
		string sBytesToCopy = Command->m_Parameters.GetValue("NUM_BYTES");
		CleanString(sBytesToCopy);
		BytesToCopy = atoi(sBytesToCopy.c_str());
	}

	if(!Object->AppendFrom(SourceObject, Offset, BytesToCopy))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + SourceObjName.length() + 100);
		sprintf(ErrorMessage, "dest:%s - src:%s", ObjName.c_str(), SourceObjName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_BINARY_CANNOT_APPEND, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}



//OBJECT BINARY_SAVE
bool NvRequest::ObjBinarySave(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjBinary* Object = (NvObjBinary*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_BINARY);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object name that should be in command parameters
	string FileName = Command->m_Parameters.GetValue("FILENAME");
	if(!FileName.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:BINARY_SAVE:FILENAME");
		return false;
	}

	// Check if an offset and a number of bytes has been requested
	int Offset = 0;
	int BytesToCopy = -1;
	if(Command->m_Parameters.KeyExist("OFFSET"))
	{
		string sOffset = Command->m_Parameters.GetValue("OFFSET");
		CleanString(sOffset);
		Offset = atoi(sOffset.c_str());
	}
	if(Command->m_Parameters.KeyExist("NUM_BYTES"))
	{
		string sBytesToCopy = Command->m_Parameters.GetValue("NUM_BYTES");
		CleanString(sBytesToCopy);
		BytesToCopy = atoi(sBytesToCopy.c_str());
	}

	if(!Object->Save(FileName.c_str(), Offset, BytesToCopy))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + FileName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_BINARY_CANNOT_SAVE, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}



//OBJECT BINARY_LOAD
bool NvRequest::ObjBinaryLoad(NvServerCommand *Command)
{
	// Get object name that should be in command parameters
	string ObjName = Command->m_Parameters.GetValue("NAME");
	CleanString(ObjName);

	NvObjBinary* Object = (NvObjBinary*)GetObjectCheckType(ObjName.c_str(), NV_OBJTYPE_BINARY);

	if(Object == NULL)
		return false;	// The GetObjectCheckType function ever sets the global error

	// Get object name that should be in command parameters
	string FileName = Command->m_Parameters.GetValue("FILENAME");
	if(!FileName.length())
	{
		GlobalError.SetLocalError("REQUEST", NV_LOCAL_BAD_PARAMETER, "OBJECT:BINARY_SAVE:FILENAME");
		return false;
	}

	// Check if an offset and a number of bytes has been requested
	int Offset = 0;
	int BytesToCopy = -1;
	if(Command->m_Parameters.KeyExist("OFFSET"))
	{
		string sOffset = Command->m_Parameters.GetValue("OFFSET");
		CleanString(sOffset);
		Offset = atoi(sOffset.c_str());
	}
	if(Command->m_Parameters.KeyExist("NUM_BYTES"))
	{
		string sBytesToCopy = Command->m_Parameters.GetValue("NUM_BYTES");
		CleanString(sBytesToCopy);
		BytesToCopy = atoi(sBytesToCopy.c_str());
	}

	if(!Object->Load(FileName.c_str(), Offset, BytesToCopy))
	{
		char *ErrorMessage = (char *)malloc(ObjName.length() + FileName.length() + 100);
		sprintf(ErrorMessage, "%s - %s", ObjName.c_str(), FileName.c_str());
		GlobalError.SetLocalError("OBJECT", NV_LOCAL_BINARY_CANNOT_LOAD, ErrorMessage);
		free(ErrorMessage);
		return false;
	}
	return true;
}












